/* 
xagent offline NNTP news reader
xagent is registered Copyright (C) 1997 <Jan Mourer>
email: pante@pi.net
snail mail: PO BOX 61  SINT ANNA  9076ZP HOLLAND

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "xagent.h"

extern FD_xagent *fd_xagent;
extern FD_xagent *fdui;


struct article
	{
	char *name;/* this is the article id long decimal */
	char *subject;
	char *from;
	char *content_type;
	char *references;
	char *message_id;
	char *attachment_name;
	char *date;
	int retrieval_flag;	
	int have_body_flag;
	int follow_flag;
	int browser_line;/* used to lookup name (= id) if clicked in display,
						not saved on disk, assigned each time list
						is displayed */
	int read_flag;
	int lock_flag;
	int encoding;
	long lines;
	int not_availeble_flag;
	int summary_flag;
	int filter_flags;
	struct article *nxtentr;
	struct article *prventr;
	}; 
struct article *articletab[2]; /* first element points to first entry,
									second element to last entry */



struct article *lookup_article(char *name)
{
struct article *pa;

/*pa points to next entry*/
for(pa = articletab[0]; pa != 0; pa = pa -> nxtentr)
	{
	if(strcmp(pa -> name, name) == 0) return(pa);
	}
return(0);/*not found*/
}/* end function lookup_article */


struct article *install_article_at_end_of_list(char *name)
{
struct article *plast, *pnew;
struct article *lookup_article();
char *strsave();
char *malloc();
char *calloc();

if(debug_flag) fprintf(stdout, "install_article(): arg name=%s\n", name);

pnew = lookup_article(name);
if(pnew)
	{
	/* free previous definition */
	free(pnew -> subject);
	free(pnew -> from);
	free(pnew -> content_type);
	free(pnew -> references);
	free(pnew -> message_id);
	free(pnew -> attachment_name);
	free(pnew -> date);
	return(pnew);/* already there */
	}

/* create new structure */
pnew = (struct article *) calloc(1, sizeof(*pnew) );
if(! pnew) return(0);
pnew -> name = strsave(name);
if(! pnew -> name) return(0);

/* get previous structure */
plast = articletab[1]; /* end list */

/* set new structure pointers */
pnew -> nxtentr = 0; /* new points top zero (is end) */
pnew -> prventr = plast; /* point to previous entry, or 0 if first entry */

/* set previuos structure pointers */
if( !articletab[0] ) articletab[0] = pnew; /* first element in list */
else plast -> nxtentr = pnew;

/* set array end pointer */
articletab[1] = pnew;

return(pnew);/* pointer to new structure */
}/* end function install_article */


struct article *install_article_in_position(\
char *name, char *subject, char *references, int *follow)
{
struct article *pa, *pprev, *plastmatch, *pnew, *pnext;
struct article *lookup_article();
char *strsave();
char *malloc();
char *calloc();
char *test_string;
char *ptr1, *ptr2;

if(debug_flag)
	{
	fprintf(stdout,\
	"install_article_in_position():\nname=%s\nsubject=%s\nreferences=%s\n",\
	name, subject, references);
	}
	
/* argument check */
if(! name) return(0);
if(! subject) return(0);
if(! references) return(0);

*follow = 0;
plastmatch = 0;/* for gcc -Wall */

pa = lookup_article(name);/* find first, and this is any :) */
if(pa) 
	{
	*follow = pa -> follow_flag;
	return(pa); /* already there */
	}

if(strstr(subject, "Re:") == subject)/* follow up to something */
	{
	/* 
	Test for thread: 
	For a match, the reference line must be present in the new one,
	and the message_id must be the same as the last field in the new
	reference line.
	example:
	new reference line <flip> <flap> <flop>
	old reference line <flip> <flap>
	old message id <flop>
	add old msg id to old ref and do a strcmp().
	*/
	plastmatch = 0;/* no match yet */
	for(pa = articletab[1]; pa != 0; pa = pa -> prventr)
		{
		test_string =\
		(char *)malloc(strlen(pa -> references) +\
		strlen(" ") +\
		strlen(pa -> message_id) + 1);
		sprintf(test_string, "%s %s", pa -> references, pa -> message_id);

		if(strcmp(test_string, references) == 0)/* match */
			{
			free(test_string);
			*follow = pa -> follow_flag;
			plastmatch = pa;
			break;
			}

		free(test_string);
		}/* end for all articles */
	}/* end if a followup */
	
if(debug_flag)
	{
	if(plastmatch)
		{
		fprintf(stdout,\
		"reference found=%s *follow=%d\n", pa -> subject, *follow);
		}
	}

/*
If we did not find a references match, try to put it with same subject.
*/
if(! plastmatch)
	{
	/* the original may come later */
	/* strip any "Re: " from subject */
	if(strstr(subject, "Re: ") == subject)
		{
		ptr1 = subject + 4;
		}
	else ptr1 = subject;
		
	/* look from end to start for the first article with this subject */
	for(pa = articletab[1]; pa != 0; pa = pa -> prventr)
		{
		/* strip any "Re: " */
		if(strstr(pa -> subject, "Re: ") == pa -> subject)
			{
			ptr2 = pa -> subject + 4;
			}
		else ptr2 = pa -> subject;
		
		if(strcmp(ptr1, ptr2) == 0)
			{
			*follow = pa -> follow_flag;
			plastmatch = pa;
			if(debug_flag)
				{
				fprintf(stdout,\
				"subject found=%s *follow=%d\n",\
				pa -> subject, *follow);
				}
			break;
			}
		}/* end for all articles */
	}/* end if ! plastmatch */

/* plastmatch now points to insert point, or zero if not found, */

/* add to end list if no reference match and no subject match */
if(! plastmatch)
	{
	*follow = 0;
	plastmatch = articletab[1];
	}
/*
	{
	*follow = 0;
	pa = install_article_at_end_of_list(name);
	return(pa);
	}
*/

/* plastmatch now holds insert position, insert article */
/* start at end of list, move towards beginning */
pa = articletab[1];		
while(1)
	{		
	if(! pa)/* end of chain, add as last */
		{
		pa = install_article_at_end_of_list(name);
		return(pa);
		}
	/* never inserting as last one, done above */
	if(pa == plastmatch)/* some structure */
		{
		/* create new structure */
		pnew = (struct article *) calloc(1, sizeof(*pnew) );
		if(! pnew) return(0);
		pnew -> name = strsave(name);
		if(! pnew -> name) return(0);

		/* insert the structure AFTER plastmatch */
		
		/* get previous and next structure */
		pnext = plastmatch -> nxtentr;
		pprev = plastmatch;

		/* set pointers for new struture */
		pnew -> nxtentr = pnext;
		pnew -> prventr = pprev;

		/* set pointers for next structure */
		if(pnext == 0) articletab[1] = pnew;
		else pnext -> prventr = pnew;		

		/* set pointer for previous structure */
		if(pprev == 0) articletab[0] = pnew;
		else pprev -> nxtentr = pnew;
		
		return(pnew);/* pointer to new structure */
		}
	pa = pa -> prventr;
	}/* end while insert loop */
}/* end function install_article_in_position */


int delete_article(char *name)/* delete entry from double linked list */
{
struct article *pa, *pprev, *pdel, *pnext;

if(debug_flag) fprintf(stdout, "delete_article(): arg name=%s\n", name);

/* parameter check */
if(! name) return(0);

pa = articletab[0];
while(1)
	{
	/* if end list, return not found */
	if(! pa) return(0);

	/* test for match in name */
	if(strcmp(name, pa -> name) != 0) /* no match */
		{
		/* point to next element in list */
		pa = pa -> nxtentr;

		/* loop for next element in list */
		continue;
		}

	/* we now know which struture to delete */
	pdel = pa;

	/* get previous and next structure */
	pnext = pa -> nxtentr;
	pprev = pa -> prventr;

	/* set pointers for previous structure */
	/* if first one, modify articletab[0] */
	if(pprev == 0) articletab[0] = pnext;
	else pprev -> nxtentr = pnext;

	/* set pointers for next structure */
	/* if last one, modify articletab[1] */
	if(pnext == 0) articletab[1] = pprev;
	else pnext -> prventr = pprev;
	
	/* delete structure */	
	free(pdel -> name);
	free(pdel -> subject);
	free(pdel -> from);
	free(pdel -> content_type);
	free(pdel -> references);
	free(pdel -> message_id);		
	free(pdel -> attachment_name);
	free(pdel -> date);
	free(pdel); /* free structure */

	/* return OK deleted */
	return(1);
	}/* end for all structures */
}/* end function delete_article */


int delete_all_articles()/* delete all entries from table */
{
struct article *pa;

if(debug_flag) fprintf(stdout, "delete_all_articles() arg none\n");

while(1)
	{	
	pa = articletab[0];
	if(! pa) break;
	articletab[0] = pa -> nxtentr;
	free(pa -> name);
	free(pa -> subject);
	free(pa -> from);
	free(pa -> content_type);
	free(pa -> references);
	free(pa -> message_id);
	free(pa -> attachment_name);			
	free(pa -> date);
	free(pa);/* free structure */
	}/* end while all structures */
articletab[1] = 0;
if(debug_flag) fprintf(stdout, "returning from delete_all_articles\n");
return(1);
}/* end function delete_all_articles */


int load_articles(char *group)
{
int a;
FILE *articles_dat_file;
char temp[READSIZE];
char pathfilename[512];
char name[512];
int retrieval_flag;
int have_body_flag;
int follow_flag;
int read_flag;
int lock_flag;
int encoding;
long lines;
int not_availeble_flag;
int summary_flag;
int filter_flags;
int browser_line;
extern long atol();
struct article *pa;
extern char *strsave();
static char last_group[512];/* preserve between calls */
char *subject;
char *from;
char *references;
char *content_type;
char *message_id;
char *attachment_name;
char *date;
int first_article_flag;

if(debug_flag)
	{
	fprintf(stdout, "load_articles() arg group=%s (last_group=%s\n",\
	group, last_group);
	}
	
if(! group) return(0);

/* test if this group was already loaded, (structure may be modified) */
if(strcmp(group, last_group) == 0) return(1);

/*
test if any group was loaded, if so save it,
structure may have been modified
*/
if(last_group[0] != 0) save_articles(last_group);

strcpy(last_group, group);

delete_all_articles();

sprintf(pathfilename, "%s/.xagent/%s/articles.dat", home_dir, group);
articles_dat_file = fopen(pathfilename, "r");
if(! articles_dat_file)
	{
	if(debug_flag)fprintf(stdout, "could not load file %s\n", pathfilename);
	return(0);
	} 
first_article_flag = 1;
while(1)
	{
	/* 
	This should make it possible toi read old formats, that do not have a
	filter_flags.
	*/
	filter_flags = 0;

	a = readline(articles_dat_file, temp);
	if(a == EOF) return(0);/* file format error */

	sscanf(temp, "%s %d %d %d %d %d %d %ld %d %d %d %d",\
	name, &retrieval_flag, &have_body_flag, &follow_flag, &read_flag,\
	&lock_flag, &encoding, &lines, &browser_line, &not_availeble_flag,\
	&summary_flag, &filter_flags);

	if(first_article_flag)
		{
		selected_article = atol(name);
		first_article_flag = 0;
		}

	/* Subject: */
	a = readline(articles_dat_file, temp);
	if(a == EOF) return(0);/* file format error */
	subject = strsave(temp);
	if(! subject) return(0);

	/* From: */
	a = readline(articles_dat_file, temp);
	if(a == EOF) return(0);/* file format error */
	from = strsave(temp);
	if(! from) return(0);
	 
	/* Content-Type: */
	a = readline(articles_dat_file, temp);
	if(a == EOF) return(0);/* file format error */
	content_type = strsave(temp);
	if(! content_type) return(0);

	/* References: */
	a = readline(articles_dat_file, temp);
	if(a == EOF) return(0);/* file format error */
	references = strsave(temp);
	if(! references) return(0);

	/* Mesage-ID: */
	a = readline(articles_dat_file, temp);
	if(a == EOF) return(0);/* file format error */
	message_id = strsave(temp);
	if(! message_id) return(0);

	/* Attachment_name: */
	a = readline(articles_dat_file, temp);
	if(a == EOF) return(0);/* file format error */
	attachment_name = strsave(temp);
	if(! attachment_name) return(0);

	/* Date: */
	a = readline(articles_dat_file, temp);
	date = strsave(temp);
	if(! date) return(0);

	/* create a structure entry */
	pa = install_article_at_end_of_list(name);
	if(! pa)
		{
		if(debug_flag)fprintf(stdout,\
		"load_articles: cannot install article %s\n", temp);
		return(0);
		}
	
	/* data to structure */
	pa -> subject = subject;
	pa -> from = from;
	pa -> content_type = content_type;
	pa -> references = references;
	pa -> message_id = message_id;
	pa -> attachment_name = attachment_name;
	pa -> date = date;
	pa -> retrieval_flag = retrieval_flag;
	pa -> have_body_flag = have_body_flag;
	pa -> follow_flag = follow_flag;
	pa -> read_flag = read_flag;
	pa -> lock_flag = lock_flag;
	pa -> encoding = encoding;
	pa -> lines = lines;
	pa -> browser_line = browser_line;
	pa -> not_availeble_flag = not_availeble_flag;
	pa -> summary_flag = summary_flag;
	pa -> filter_flags = filter_flags;
	if(a == EOF) return(1);
	}
}/* end function load articles */


int save_articles(char *group)
{
FILE *articles_dat_file;
char pathfilename[512];
struct article *pa;
extern char *strsave();

if(debug_flag) fprintf(stdout, "save_articles() arg group=%s\n", group);

/* argument check */
if(! group) return(0);

sprintf(pathfilename, "%s/.xagent/%s/articles.dat", home_dir, group);
articles_dat_file = fopen(pathfilename, "w");
if(! articles_dat_file)
	{
	if(debug_flag)fprintf(stdout, "could not open file %s for write\n", pathfilename);
	return(0);
	} 
for(pa = articletab[0]; pa != 0; pa = pa -> nxtentr)	
	{
	fprintf(articles_dat_file, "%s %d %d %d %d %d %d %ld %d %d %d %d\n",\
	pa -> name, pa -> retrieval_flag, pa -> have_body_flag,\
	pa -> follow_flag, pa -> read_flag, pa -> lock_flag,\
	pa -> encoding, pa -> lines, pa -> browser_line,\
	pa -> not_availeble_flag, pa -> summary_flag, pa -> filter_flags);
	fprintf(articles_dat_file, "%s\n", pa -> subject);
	fprintf(articles_dat_file, "%s\n", pa -> from);
	fprintf(articles_dat_file, "%s\n", pa -> content_type);
	fprintf(articles_dat_file, "%s\n", pa -> references);
	fprintf(articles_dat_file, "%s\n", pa -> message_id);
	fprintf(articles_dat_file, "%s\n", pa -> attachment_name);
	fprintf(articles_dat_file, "%s\n", pa -> date);
	}/* end for all structures in the chain starting from articletab[0] */

fclose(articles_dat_file);

return(1);/* ok */
}/* end function save_articles */


int add_article_header(char *group, long id)
{
int c, i;
struct article *install_article_in_position(), *pa;
char name[512];
extern char *strsave();
FILE *header_file;
char path_filename[512];
char temp[READSIZE];
char *subject;
char *from;
char *content_type;
char *references;
char *message_id;
char *date;
long lines;
int summary_flag;
extern long atol();
int follow;
int lf_flag;
int first_char;
int first_char_flag;
int folded_flag;

if(debug_flag)
	{
	fprintf(stdout, "add_article_header(): arg group=%s id=%ld\n",\
	group, id);
	}

/* argument check */
if(! group) return(0);
if(id < 0) return(0);

first_char = 0;/* only for clean compilation with -Wall */

/* fill the structure for this group */
load_articles(group);

sprintf(path_filename, "%s/.xagent/%s/head.%ld",\
home_dir, group, id);
header_file = fopen(path_filename, "r");
if(! header_file) return(0);

from = strsave("");
subject = strsave("");
content_type = strsave("");
references = strsave("");
message_id = strsave("");
date = strsave("");
lines = 0;
follow = 0;
summary_flag = 0;

/* scan the headerfile */
temp[0] = 0;/* mark empty */
first_char_flag = 0;
while(1)/* all entries (unfolded header lines) in header file */
	{
	folded_flag = 0;	/* the function of this variable is to find out if
							the line was folded */
	i = 0;
	lf_flag = 0;
	while(1)/* all characters in a line */
		{
		if(first_char_flag)
			{
			temp[i] = first_char;
			i++;
			first_char_flag = 0;
			}

		while(1)/* error re read */
			{
			c = getc(header_file);
			if(! ferror(header_file) ) break;
			perror("add_article_header(): read failed ");
			}/* end while error re read */	

		if(feof(header_file) || (c == EOF) )
			{
			fclose(header_file);
			c = EOF;
			break;
			}
		
		if(lf_flag)
			{
			if( (c != ' ') && (c != 9) ) /* TAB = 9, no LWS */
				{
				/*
				line ends, save this character, it is the first one in the
				next line.
				*/
				first_char = c;
				first_char_flag = 1;
				break;
				}
			/* line continous */
			lf_flag = 0;
			folded_flag = 1;
			/* store char in line */
			}
			
		if(c == '\n')
			{
			lf_flag = 1;		
			continue;/* get next char, test for linear white space */
			}
			
		temp[i] = c;
		i++;
		if(i >= READSIZE - 1) break; /* prevent overflow */
		}/* end while all characters in a line */
	temp[i] = 0;/* string termination */

	if(strstr(temp, "From:") == temp)
		{
		free(from);
		from = strsave(strstr(temp, ":") + 2);
		}
	if(strstr(temp, "Subject:") == temp)
		{
		if(folded_flag)/* subject line was multiple line */
			{
			/* 
			use 1 as a flag for the display routine that it should
			read from the header flag, instead of pa -> subject
			used by display_articles() and display_article_body()
			Note: cannot set flag in position 0, need to check for Re:
			later.
			*/
			strcat(temp, "\001");
			free(subject);
			subject = strsave(strstr(temp, ":") + 2);
			}
		else
			{
			free(subject);
			subject = strsave(strstr(temp, ":") + 2);
			}
		}
	if(strstr(temp, "Content-Type:") == temp)
		{
		free(content_type);
		content_type = strsave(strstr(temp, ":") + 2);
		}
	if(strstr(temp, "References:") == temp)
		{
		free(references);
		references = strsave(strstr(temp, ":") + 2);
		}
	if(strstr(temp, "Message-ID:") == temp)
		{
		free(message_id);
		message_id = strsave(strstr(temp, ":") + 2);
		}
	if(strstr(temp, "Date:") == temp)
		{
		free(date);
		date = strsave(strstr(temp, ":") + 2);
		}
	if(strstr(temp, "Lines:") == temp)
		{
		lines = atol( (strstr(temp, ":") + 2) );
		}
	if(strstr(temp, "Summary:") == temp)
		{
		summary_flag = 1;
		}
		
	if(c == EOF) break;
	if(feof(header_file) ) break;
	}/* end while all entries (unfolded header lines) in header file */

sprintf(name, "%ld", id);
pa = install_article_in_position(name, subject, references, &follow);
if(! pa) return(0);

/* set the follow_flag if set in the preceeding article */

if(follow)
	{
	pa -> follow_flag = follow;
	pa -> retrieval_flag = 1;
	
	/*
	this is a global, used by process_command_queue(),
	and if set will add articles marked for retrieval to the command
	queue, so these bodies will always be retrievd.
	*/
	new_thread_bodies_flag = 1;
fprintf(stdout, "SETTING new_thread_bodies_flag, pa->follow_flag=%d\n",
pa -> follow_flag);
	}
else
	{
	pa -> follow_flag = 0;
	pa -> retrieval_flag = 0;
	}

pa -> subject = subject;
pa -> from = from;
pa -> content_type = content_type;
pa -> references = references;
pa -> message_id = message_id;
pa -> date = date;
pa -> lines = lines;
pa -> summary_flag = summary_flag;

pa -> attachment_name = strsave("");/* not known yet at this time */
pa -> not_availeble_flag = 0; /* not availeble, not here, maybe not needed */ 
pa -> have_body_flag = 0;
pa -> read_flag = 0;
pa -> lock_flag = 0;
pa -> encoding = 0;
pa -> browser_line = 0;
pa -> filter_flags = FILTER_OFF;/* filter not applied yet */
save_articles(group);

if(debug_flag) fprintf(stdout, "RETURNING from add_article_header\n");

return(1);
}/* end function add_article_header */


int add_article_body(char *group, long article_id)
{
struct article *lookup_article(), *pa;
char name[512];

if(debug_flag)
	{
	fprintf(stdout, "add_article_body(): arg group=%s article_id=%ld\n",\
	group, article_id);
	}

/* argument check */
if(! group) return(0);
if(article_id < 0) return(0);

load_articles(group);

sprintf(name, "%ld", article_id);
pa = lookup_article(name);
if(! pa) return(0);/* no header for this body */

pa -> have_body_flag = 1;
pa -> retrieval_flag = 0;

save_articles(group);
return(1);
}/* end function add_article_body */


int show_articles(char *group)
{
int i;
char temp[READSIZE];
struct article *pa;
char formatstr[20];
char infostr[40];
int browser_line;
int browser_topline;
int browser_last_selected_line;
static char *last_group;
char *strsave();
char *ptr;
char *cptr;

if(debug_flag)fprintf(stdout, "show_articles(): arg group=%s\n", group);

/* argument check */
if(! group) return(0);

/* remember the last line selected */
browser_last_selected_line = fl_get_browser(fdui -> article_list_browser);

/* remember vertical scrollbar position */
browser_topline = fl_get_browser_topline(fdui -> article_list_browser);

/* first clear display in order not to confuse reader with old data */
fl_clear_browser(fdui -> article_list_browser);

load_articles(group);

/* if load failed (no articles.dat file yet), continue anyway */	

browser_line = 1;
for(pa = articletab[0]; pa != 0; pa = pa -> nxtentr)	
	{
	/* test if diplay allowed */
	if(filters_enabled_flag)
		{
		if(pa -> filter_flags & FILTER_DO_NOT_SHOW_HEADER) continue;
		}
		
	strcpy(formatstr, "@f");
	strcat(formatstr, "@C1");/* red is new unread header */
	if(pa -> retrieval_flag) strcat(formatstr, "@C5");/* magenta */
	if(pa -> have_body_flag) strcat(formatstr, "@C4");/* blue */	
	if(pa -> retrieval_flag && pa -> have_body_flag)
		{
		strcat(formatstr, "@C21");/* dark cyan */
		}
	if(pa -> read_flag) strcat(formatstr, "@C0");/* black */
	if(pa -> not_availeble_flag) strcat(formatstr, "@C2");/* green */
	if(pa -> follow_flag & FOLLOW_THIS_SUBJECT)
		{
		if(pa -> lock_flag) strcpy(infostr, "*LOCK=");
		else strcpy(infostr, "-----=");
		}
	else if(pa -> follow_flag & FOLLOW_THIS_THREAD)
		{
		if(pa -> lock_flag) strcpy(infostr, "*LOCK<");
		else strcpy(infostr, "-----<");
		}
	else
		{
		if(pa -> lock_flag) strcpy(infostr, "*LOCK*");
		else strcpy(infostr, "      ");
		}

	/* test if followup to something */
	if(strstr(pa -> subject, "Re:") == pa -> subject)
		{
		strcat(infostr, "     ");/* identinate */
		/*
		in case of a thread, add as many spaces as there are entries
		in references.
		entries are embedded in '<xxx>', so counting '<'.
		*/
		for(cptr = pa -> references; *cptr != 0; cptr++)
			{
			if(*cptr == '<') strcat(infostr, " ");
			}
		}

	/*
	test if we have to get the subject from disk (in case formatted)
	or just use pa -> subject (not formatted, but faster).
	Note: add_article_header() sets pa -> subject to "use header\001" 
	if it is formatted (unfolded, LF removed).
	*/
	if(!strstr(pa -> subject, "\001" ) )
		{
		sprintf(temp, "%s %s %s %s %ld %s (%s)",\
		formatstr, infostr, pa -> subject, pa -> from, pa -> lines,\
		pa -> date, pa -> name);

		fl_add_browser_line(fdui -> article_list_browser, temp);
		pa -> browser_line = browser_line;/* first one is 1 */
		browser_line++;
		}
	else /* a formatted subject, read from header file */
		{
		pa -> browser_line = browser_line;
		ptr = (char *) get_formatted_header_data(\
		group, atol(pa -> name), "Subject:");
		if(ptr)
			{
 			strcat(infostr, " ");
 			cptr = ptr;
			while(1)/* for all lines in the summary field */
				{
				sprintf(temp, "%s %s", formatstr, infostr);
				i = strlen(temp);
				while(1)/* extract a line */
					{
					if(*cptr == '\n') break;
					temp[i] = *cptr;
					if(! *cptr) break;
					i++;
					cptr++;
					}/* end while all characters in a line */
				temp[i] = 0;
				fl_add_browser_line(fdui -> article_list_browser, temp);
				browser_line ++;

				if(! *cptr) break;
				cptr++;
				}/* end while for all lines in summary */
			free(ptr);
			}/* end if subject read OK */

		sprintf(temp, "%s %s %s %s %ld %s (%s)",\
		formatstr, infostr, "", pa -> from, pa -> lines,\
		pa -> date, pa  -> name);
		fl_add_browser_line(fdui -> article_list_browser, temp);
		browser_line++;

		}/* end else formatted subject */

	if(pa -> summary_flag)
		{
		if(debug_flag)
			{
			fprintf(stdout, "show_articles(): Processing summary\n");
			}
		/* if a summary field is present, show it*/	
		ptr = (char *) get_formatted_header_data(\
		group, atol(pa -> name), "Summary:");
		if(ptr)
			{
			strcat(infostr, "  ");

			sprintf(temp, "%s %sSummary:", formatstr, infostr);
			fl_add_browser_line(fdui -> article_list_browser, temp);
			browser_line++;

 			cptr = ptr;
			while(1)/* for all lines in the summary field */
				{
				sprintf(temp, "%s %s", formatstr, infostr);
				i = strlen(temp);
				while(1)/* extract a line */
					{
					if(*cptr == '\n') break;
					temp[i] = *cptr;
					if(! *cptr) break;
					i++;
					cptr++;
					}/* end while all characters in a line */
				temp[i] = 0;
				fl_add_browser_line(fdui -> article_list_browser, temp);
				browser_line ++;

				if(! *cptr) break;
				cptr++;
				}/* end while for all lines in summary */
			free(ptr);
			}/* end if summary read OK */
		}/* end if pa -> summary_flag */
	}/* end for all structures in the chain starting from articletab[0] */

/* select the line that was last double_clicked again */
if(last_group)
	{
	if(strcmp(selected_group, last_group) == 0)
		{
		fl_select_browser_line(fdui -> article_list_browser,\
		browser_last_selected_line);
		}
	else
		{
		free(last_group);
		}
	}
last_group = strsave(selected_group);

/* re adjust browser for same position of top line (vertical scrollbar) */	
fl_set_browser_topline(fdui -> article_list_browser, browser_topline);

/* structure was modified */
save_articles(group);

return(1);
}/* end function show_articles */


int show_article_body(char *group, long article_id)
{
int c;
char temp[READSIZE];	
FILE *load_filefd;
struct article *pa;
char *space, *spaceptr;
char *strsave();
struct stat *statptr;
char *expanded_space;

/* argument check */
if(! group) return(0);
if(! article_id) return(0);

fl_set_input(fdui -> article_body_input_field, "");

load_articles(group);

/* show subject and sender in description display */
sprintf(temp, "%ld", article_id);
pa = lookup_article(temp);
if(! pa) return(0);

sprintf(temp, "%s %s (%s)", pa -> subject, pa -> from, pa -> name);
fl_set_object_label(fdui -> description_display, temp);

/* test if allowed */
if(filters_enabled_flag)
	{
	if(pa -> filter_flags & FILTER_DO_NOT_SHOW_BODY)
		{
		fl_set_input(fdui -> article_body_input_field,\
		"Filter: Permission denied");
		return(1);
		}
	}

/* article present, load it */
sprintf(temp, "%s/.xagent/%s/body.%ld", home_dir, group, article_id);
if(debug_flag) fprintf(stdout, "Opening file%s\n", temp);

load_filefd = fopen(temp, "r");
if(! load_filefd)
	{
	if(debug_flag)
		{
		fprintf(stdout,\
		"show_article_body(): cannot open %s for read\n", temp);
		}
	fl_show_alert(\
	"show_article_body():", "cannot open file for read", temp, 0);
	return(0);
	}

statptr = (struct stat*) malloc(sizeof(struct stat) );
if(! statptr) return(0);

fstat(fileno(load_filefd), statptr);

space = malloc(statptr -> st_size + 1);
free(statptr);
if(! space)
	{
	if(debug_flag)
		{
		fprintf(stdout,\
		"show_article_body(): malloc could not allocate space\n");
		}
	return(0);
	}

spaceptr = space;
while(1)
	{
	while(1)/* error re read */
		{
		c = getc(load_filefd);
		if(! ferror(load_filefd) ) break;
		perror("show_article_body(): read failed ");
		}/* end while error re read */	

	if(feof(load_filefd) )
		{
		fclose(load_filefd);
		break;
		}
	*spaceptr = c;
	spaceptr++;
	}/* end while all lines from article body */
*spaceptr = 0;/* string termination */

expanded_space = (char *) expand_tabs(space, tab_size);
free(space);

fl_set_input(\
fdui -> article_body_input_field, expanded_space);
free(expanded_space);

/* set scollbar to top of article */
fl_set_input_topline(fdui -> article_body_input_field, 1);

pa -> read_flag = 1;
save_articles(group);
get_and_set_unread_articles(group);

return(1);
}/* end function show_article_body */


int set_retrieval_flag(char *group, long article_id, int state)
{
char temp[50];	
struct article *pa, *lookup_article();

if(debug_flag)
	{
	fprintf(stdout,\
	"set_retieval_flag(): arg group=%s article_id=%ld state=%d\n",\
	group, article_id, state);
	}

/* argument check */
if(! group) return(0);
if(article_id < 0) return(0);

load_articles(group);

sprintf(temp, "%ld", article_id);
pa = lookup_article(temp);
if(! pa) return(0);/* fatal error */
pa -> retrieval_flag = state;
save_articles(group);
return(1);
}/* end function set_retrieval_flag */


int set_all_read_flags(char *group, int state)
{
struct article *pa;

if(debug_flag)
	{
	fprintf(stdout, "set_all_read_flags(): arg group=%s state=%d\n",\
	group, state);
	}
	
/* argument check */
if(! group) return(0);

load_articles(group);

/*pa points to next entry*/
for(pa = articletab[0]; pa != 0; pa = pa -> nxtentr)
	{
	pa -> read_flag = state;
	}

save_articles(group);
return(1);
}/* end function set_all_read_flags */


int set_read_flag(char *group, long article_id, int state)
{
char temp[50];	
struct article *pa, *lookup_article();

if(debug_flag)
	{
	fprintf(stdout,\
	"set_retieval_flag(): arg group=%s article_id=%ld state=%d\n",\
	group, article_id, state);
	}

/* argument check */
if(! group) return(0);
if(article_id < 0) return(0);

load_articles(group);

sprintf(temp, "%ld", article_id);
pa = lookup_article(temp);
if(! pa) return(0);/* fatal error */
pa -> read_flag = state;

save_articles(group);
return(1);
}/* end function set_retrieval_flag */


int exists_article_body(char *group, long article_id)
{
char temp[512];	
struct article *pa, *lookup_article();

load_articles(group);/* clears structure first */

/* test if we have the body of this article, else mark for retrieval */
sprintf(temp, "%ld", article_id);
pa = lookup_article(temp);
if(! pa) return(0);/* fatal error */

if(filters_enabled_flag)
	{
	if(pa -> filter_flags & FILTER_DO_NOT_SHOW_BODY)
		{
		fl_set_object_label(fdui -> command_status_display,\
		"Filter: Permission denied");
		return(0);
		}
	}/* end if filters_enabled_flag */

if(! pa -> have_body_flag)
	{
	pa -> retrieval_flag = 1 - pa -> retrieval_flag;
	save_articles(group);
	return(0);/* no article body to show */
	}

return(1);/* exists */
}/* end function exists_article_body */


int line_to_article_id(char *group, int line, long *article_id)
{
struct article *pa;
extern long atol();

if(debug_flag)
	{
	fprintf(stdout, "line_to_article_id(): arg group=%s line=%d\n",\
	group, line);
	}

/* argument check */
if(! group) return(0);
if(line < 0) return(0);

load_articles(group);

for(pa = articletab[0]; pa != 0; pa = pa -> nxtentr)	
	{
	if(pa -> browser_line == line)
		{
		*article_id = atol(pa -> name);
		return(1);
		}
	}
/* no save_articles, nothing was changed */
return(0);
}/* end function line_to_article_id */


int add_marked_article_bodies_in_group_to_command_queue(char *group)
{
struct article *pa;
extern long atol();

if(debug_flag)
	{
	fprintf(stdout,\
	"add_marked_article_bodies_in_group_to_command_queue(): arg group=%s\n",\
	group);
	}

/* argument check */
if(! group) return(0);

load_articles(group);

for(pa = articletab[0]; pa != 0; pa = pa -> nxtentr)	
	{
	if(pa -> retrieval_flag)
		{
		add_to_command_queue(group, atol(pa -> name), GET_ARTICLE_BODY);
		}
	}
/* no save_articles nothing has changed */
return(1);
}/* end function add_marked_article_bodies_in_group_to_command_queue */


char *get_article_attachment(char *group, long article_id)
{
struct article *pa, *lookup_article();
char temp[50];
extern char* strsave();

if(debug_flag)
	{
	fprintf(stdout,\
	"get_article_attachment(): arg group=%s article_id=%ld\n",\
	group, article_id);
	}

/* parameter_check */
if(! group) return(0);
if(article_id < 0) return(0);

load_articles(group);

sprintf(temp, "%ld", article_id);
pa = lookup_article(temp);
if(! pa) return(0);

/* no save, nothing was changed */

return(pa -> attachment_name);
}/* end function get_article_attachment */ 


int delete_article_entry_header_body(char *group, long article_id)
{
int a;
int permission;
char temp[512];
char temp2[512];
struct article *pa, *lookup_article();
char *ptr;

if(debug_flag)
	{
	fprintf(stdout,\
	"delete_article_entry_header_body(): arg group=%s article_id=%ld\n",\
	group, article_id);
	}

/* argument check */
if(! group) return(0);
if(article_id < 0) return(0);

load_articles(group);

sprintf(temp, "%ld", article_id);
pa = lookup_article(temp);
if(! pa) return(0);

if(pa -> lock_flag) return(1);
if(pa -> retrieval_flag) return(1);
if( (pa -> have_body_flag) && (! pa -> read_flag) ) return(1);	

sprintf(temp, "%s/.xagent/%s/body.%ld", home_dir, group, article_id);
unlink(temp);

sprintf(temp, "%s/.xagent/%s/head.%ld", home_dir, group, article_id);
unlink(temp);

/* test if binary attachment was launched before */
/* if so erase it
*/
if(! get_article_encoding(group, article_id, &a, &permission) ) return(0);
if(a)
	{
	ptr = (char *) get_article_attachment(group, article_id);
	if(! ptr) return(0);/* trouble */

	sprintf(temp, "%s/.xagent/%s/%s", home_dir, group, ptr);
	unlink(temp);
	
	/*
	erase .desc file generated by muncpac (if any ),
	example: picture.jpg may have picture.desc
	*/
	strcpy(temp2, ptr);
	ptr = strstr(temp2, ".");
	if(ptr)
		{
		*ptr = 0;/* replace dot by string termination, always safe */
		strcat(temp2, ".desc");
		sprintf(temp, "%s/.xagent/%s/%s", home_dir, group, temp2);
		unlink(temp);
		}
	}/* end if a */

sprintf(temp, "%ld", article_id);
delete_article(temp);

save_articles(group);
return(1);
}/* end function delete_article_body */


int set_lock_flag(char *group, long article_id, int lock)
{
struct article *pa, * lookup_article();
char temp[512];

if(debug_flag)
	{
	fprintf(stdout, "set_lock_flag(): arg group=%s article_id=%ld lock=%d\n",\
	group, article_id, lock);
	}

if(! group) return(0);
if(article_id < 0) return(0);

load_articles(group);

sprintf(temp, "%ld", article_id);
pa = lookup_article(temp);
if(! pa) return(0);
if(pa -> lock_flag != 4) pa -> lock_flag = lock;/* xagent.help articles are
													4 and cannot be erased */
save_articles(group);
return(1);
}/* end funtion set_lock_flag */


char *lookup_content_type(char *group, long article_id)
{
struct article *pa, *lookup_article();
char temp[50];

if(debug_flag)
	{
	fprintf(stdout, "lookup_content_type(): arg group=%s article_id=%ld\n",\
	group, article_id);
	}

if(! group) return(0);
if(article_id < 0) return(0);

load_articles(group);

sprintf(temp, "%ld", article_id);
pa = lookup_article(temp);
if(!pa) return(0);/* no such article_id */

return(pa -> content_type);/* a char * */
}/* end function lookup_content_type */


int set_article_encoding(char *group, long article_id, int encoding)
{
struct article *pa, *lookup_article();
char temp[50];

if(debug_flag)
	{
	fprintf(stdout,\
	"set_article_encoding(): arg group=%s article_id=%ld encoding=%d\n",\
	group, article_id, encoding);
	}

if(! group) return(0);
if(article_id < 0) return(0);

load_articles(group);

sprintf(temp, "%ld", article_id);
pa = lookup_article(temp);
if(! pa) return(0);

pa -> encoding = encoding;

save_articles(group);

return(1);
}/* end function set_article_encoding */


int get_article_encoding(\
char *group, long article_id, int *encoding, int *view_permission)
{
struct article *pa, *lookup_article();
char temp[50];

if(debug_flag)
	{
	fprintf(stdout,\
	"get_article_encoding(): arg group=%s article_id=%ld\n",\
	group, article_id);
	}

if(! group) return(0);
if(article_id < 0) return(0);

load_articles(group);

sprintf(temp, "%ld", article_id);
pa = lookup_article(temp);
if(! pa) return(0);

/* test if allowed */
if(filters_enabled_flag)
	{
	if(pa -> filter_flags & FILTER_DO_NOT_SHOW_ATTACHMENT)
		{
		*view_permission = 0;
		}
	}/* end if filters_enabled_flag */
		
*encoding = pa -> encoding;
/* no save, nothing was changed */
return(1);
}/* end function get_article_encoding */


int set_article_attachment(\
char *group, long article_id, char *attachment_name)
{
struct article *pa, *lookup_article();
char temp[50];
extern char* strsave();

if(debug_flag)
	{
	fprintf(stdout,\
"set_article_attachment(): arg group=%s article_id=%ld attachment_name=%s\n",\
	group, article_id, attachment_name);
	}

/* parameter_check */
if(! group) return(0);
if(article_id < 0) return(0);
if(! attachment_name) return(0);

load_articles(group);

sprintf(temp, "%ld", article_id);
pa = lookup_article(temp);
if(! pa) return(0);

free(pa -> attachment_name);
pa -> attachment_name = (char *) strsave(attachment_name);

save_articles(group);

return(1);
}/* end function set_article_attachment */ 


int set_article_no_longer_availeble_flag(char *group, long article_id)
{
struct article *lookup_article(), *pa;
char temp[50];

if(debug_flag)
	{
	fprintf(stdout,\
	"set_article_no_longer_availeble_flag(): arg group=%s article_id=%ld\n",\
	group, article_id);
	}

if(! group) return(0);
if(article_id < 0) return(0);

load_articles(group);

sprintf(temp, "%ld", article_id);
pa = lookup_article(temp);
if(! pa) return(1);

pa -> not_availeble_flag = 1;
pa -> retrieval_flag = 0;/* do not try again */

save_articles(group);
return(1);
}/* end function set_article_no_longer_availeble_flag */


int decode_articles_with_content_type_id(char *group, char *content_type_id)
{
int c, i, j;
struct article *pa;
char *ptr;
char temp[512];
extern long atol();

if(debug_flag)
	{
	fprintf(stdout, "decode_articles_with_content_type_id(): arg id=%s\n",\
	content_type_id);
	}

load_articles(group);

/* for all articles */
pa = articletab[0];
while(1)
	{	
	if(! pa) return(1);/* end chain */
	i = 0;
	/* extract the id= part */
	while(1)/* for all parts of content_type */
		{
		/* get article from partial_articles */
		j = 0;
		while(1)/* for all character in a field */
			{
			c = pa -> content_type[i];
			if( (c == ' ') || (c == 10) || (c == 13) )
				{
				i++;
				continue;
				}
			temp[j] = c;
			if(c == 0) break;/* end of content_type */
			if(c == ';')/* end of field in content_type */
				{
				temp[j] = 0;/* string termination */
				break;
				}
			i++;
			j++;
			}/* end while field in content_type */
		/* check if this field holds "id=" */
		ptr = strstr(temp, "id=");
		if(ptr) /* found id */
			{
			/* extract id */
			if(debug_flag)
				{
				fprintf(stdout,\
				"decode_articles_with_content_type_id(): found id=%s\n",\
				temp);
				}
			/* test if same content_type_id */
			if(strcmp(content_type_id, ptr + 3) == 0)
				{
				decode_article(group, atol(pa -> name), pa -> content_type);
				}
			}/* end found id */
		if(c == 0) break;
		i++;/* scip the ';' */
		}/* end while all parts of content_type */
	pa = pa -> nxtentr;
	}/* end while all articles (structures) */
}/* end function decode_articles_with_content_type_id */


int get_and_set_unread_articles(char *group)
{
struct article *pa;
long unread_articles;

if(debug_flag)
	{
	fprintf(stdout, "get_and_set_unread_articles(): arg group=%s\n", group);
	}
/* argument check */
if(! group) return(0);

load_articles(group);

unread_articles = 0;
for(pa = articletab[0]; pa != 0; pa = pa -> nxtentr)
	{
	if(! pa -> read_flag) unread_articles++;	
	}
set_new_articles(group, unread_articles); 

save_articles(group);
return(1);
}/* end function get_and_set_unread_articles */


int delete_all_not_locked_articles(char *group)
{
int view_permission;
int delete, encoded;
char temp[512];
char temp2[512];
struct article *pa, *pprev, *pdel, *pnext;
long article_id;
extern long atol();
char *ptr;

if(debug_flag)
	{
	fprintf(stdout, "delete_all_not_locked_articles(): arg group=%s\n",\
	group);
	}
	
sprintf(temp, "Deleting all not locked articles in %s", group);
fl_set_object_label(fdui -> command_status_display, temp);
XSync(fl_get_display(), 0);

/* argument check */
if(! group) return(0);

load_articles(group);
pa = articletab[0];
while(1)
	{
	if(! pa) break;
	delete = 1;
	if(pa -> lock_flag) delete = 0;
	if(pa -> retrieval_flag) delete = 0;
	if( (pa -> have_body_flag) && (! pa -> read_flag) ) delete = 0;	
	if(! delete)
		{
		pa = pa -> nxtentr;
		continue;
		}

	/* delete body */
	article_id = atol(pa -> name);
	sprintf(temp, "%s/.xagent/%s/body.%ld", home_dir, group, article_id);
	unlink(temp);

	/* delete header */
	sprintf(temp, "%s/.xagent/%s/head.%ld", home_dir, group, article_id);
	unlink(temp);

	/* test if binary attachment was launched before */
	/* if so erase it */
	article_id = atol(pa -> name);
	if(! get_article_encoding(\
	group, article_id, &encoded, &view_permission) ) return(0);
	if(debug_flag)
		{
		fprintf(stdout, "after get_article_encoding encoded=%d\n", encoded);
		}
	if(encoded)
		{
		ptr = (char *) get_article_attachment(group, article_id);
		if(! ptr) return(0);/* more trouble */

		/* delete attachment */
		sprintf(temp, "%s/.xagent/%s/%s", home_dir, group, ptr);
		unlink(temp);
		/*
		erase .desc file generated by muncpac (if any ),
		example: picture.jpg may have picture.desc
		*/
		strcpy(temp2, ptr);
		ptr = strstr(temp2, ".");
		if(ptr)
			{
			*ptr = 0;/* replace dot by string termination, always safe */
			strcat(temp2, ".desc");
			sprintf(temp, "%s/.xagent/%s/%s", home_dir, group, temp2);
			unlink(temp);
			}
		}/* end if encoded */

	/* remove structure from linked list */
	/* get the address of the to be deleted structure */
	pdel = pa;

	/* get next and previous structure */
	pnext = pa -> nxtentr;
	pprev = pa -> prventr;
	
	/* set pointers for previous structure */
	/* if first one, modify articletab[0] */
	if(pprev == 0) articletab[0] = pnext;
	else pprev -> nxtentr = pnext;
	
	/* set pointers for next structure */
	/* if last one, modify articletab[1] */
	if(pnext == 0) articletab[1] = pprev;
	else pnext -> prventr = pprev;

	/* point to next one */
	pa = pa -> nxtentr;

	/* delete structure data */	
	free(pdel -> name);/* free name */
	free(pdel -> subject);
	free(pdel -> from);
	free(pdel -> content_type);
	free(pdel -> references);
	free(pdel -> message_id);		
	free(pdel -> attachment_name);
	free(pdel -> date);
	free(pdel);/* free structure */
	}/* end for all structures */

save_articles(group);
get_and_set_unread_articles(group);

/* clear message in command status display */
fl_set_object_label(fdui -> command_status_display, "");
return(1);
}/* end function delete_all_not_locked_articles */


int display_next_unread_article()
{
struct article *pa, *lookup_article();
char temp[80];
extern long atol();
char *ptr;
extern char *strsave();

if(debug_flag)
	{
	fprintf(stdout, "display_next_unread_article():\n\
	arg using selected_group=%s selected_article=%ld\n",\
	selected_group, selected_article);	
	}

/* argument check */
if(! selected_group) return(0);
if(selected_article < 0) return(0);

/* remember where we are */
strcpy(start_unread_group, selected_group);
start_unread_article = selected_article;

load_articles(selected_group);

/* move to present position */
sprintf(temp, "%ld", selected_article);
pa = lookup_article(temp);
/* if no article selected error */
if(! pa) return(0);

while(1)/* for all groups */
	{
	/*
	search for next unread that we have body from from this point
	onwards
	*/
	while(1)
		{
		if(pa -> have_body_flag)
			{
			if(! pa -> read_flag)
				{
				selected_article = atol(pa -> name);

/*
				if(pa -> encoding)
					{
					lounch_binary_attachment(\
					selected_group, selected_article);
					return(1);
					}
				else
*/	
					{
					show_browser(ARTICLE_BODY_BROWSER);
					refresh_screen(selected_group);
					return(1);
					}/* end not encoded */
				}/* end read */
			}/* end have body */

		/* move to next article */
		pa = pa -> nxtentr;
		
		if(! pa)/* end of article list in this group */
			{
			/* search for next not empty group */
			while(1)
				{
				/* move to next group */
				ptr = (char *) get_next_subscribed_group(selected_group);
				if(! ptr) return(0);/* possibly no subscribed groups */
	
				if(selected_group) free(selected_group);
				selected_group = strsave(ptr);
				load_articles(selected_group);

				/* first article in next group */
				pa = articletab[0];
				/* if group not empty (valid pa) test article */
				if(pa) break;
				}/* end while search for not empty group */
			}/* end if end of article list */
				
		/* test for round */
		if(strcmp(selected_group, start_unread_group) == 0)
			{
			if(atol(pa -> name) == start_unread_article)
				{
				fl_show_alert("All articles have been read", "", "", 0);
				return(0);
				}
			}/* end if round */
		}/* end while all articles in this group */
	}/* end while all articles */
return(1);
}/* end function display_next_unread_article */


int display_next_unread_article_in_subject()
{
struct article *pa, *lookup_article(), *pselected;
char *pssubject, *pasubject;
char temp[512];
extern long atol();
extern char *strsave();
int match;

/*
I have a problem here,
this function is impossible.
Threads may split, that would require a decision by xagent which branch to
follow.
A choice could be presented to the user, but making a choice would leave
out all other branches in the original starting point.
To make a usefull choice you would have to read the articles first :).
So now implementing as next unread in subject.
*/

if(debug_flag)
	{
	fprintf(stdout, "display_next_unread_article_in_subject() arg none\n");
	}
	
/* argument check */
if(! selected_group) return(0);
if(selected_article < 0) return(0);

sprintf(temp, "%ld", selected_article);
pselected = lookup_article(temp);
if(! pselected) return(0);

/* test if reference line present */
/*Thread if(! pselected -> references) return(0);*/

/*
Note: start_article and start_group set to selected_article and
selected_group, by search_accept_button_cb.
*/

match = 0;
for(pa = articletab[0]; pa != 0; pa = pa -> nxtentr)
	{
	if( (pa -> have_body_flag) && (! pa -> read_flag) )
		{

		/* test if reference line present, or exeption, if not there! */
/*		Thread if(! pa -> references) continue;*/
		
			/*
			If the selected_article is a Re:
			For an article to be a followup to the selected_article,
			the references line of the selected_article must be present in
			the reference line of the article.
			*/

			/*
			else
			The reference line of the article must be the same as the 
			mesage_id of the selected_article.
			*/

			/*
			Easing the requirement, to message_id of the selected_article
			must be present in the reference line of the article.
			Now if missing articles in thread, 
			then later ones are also shown.
			*/

			/*
			Gave up on this function as 'next thread',
			see comment at top of function.
			*/

		/* The 'next unread in subject' implementation */
		
		/* strip any "Re: " in selected_article subject */
		if(strstr(pselected -> subject, "Re: ") == pselected -> subject)
			{
			pssubject = pselected -> subject + 4;
			}
		else pssubject = pselected -> subject;
		
		/* strip any "Re: " in article subject */
		if(strstr(pa -> subject, "Re: ") == pa -> subject)
			{
			pasubject = pa -> subject + 4;
			}
		else pasubject = pa -> subject;
		
		/* now we can compare subjects */ 
		if(strcmp(pssubject, pasubject) == 0) match = 1;
				
		if(match)
			{
			selected_article = atol(pa -> name);

			/*
			if(pa -> encoding)
				{
				lounch_binary_attachment(\
				selected_group, selected_article);
				return(1);		
				}
			else
			*/

				{
				show_browser(ARTICLE_BODY_BROWSER);
				refresh_screen(selected_group);/* always in this group */
 				return(1);
				}/* end not encoded */

			}/* end if match */
		}/* end not read and have body */
	}/* end for all structures in chain */
fl_show_alert("No more unread articles on this subject", "", "", 0);
return(1);
} /* end function display_next_unread_article_in_subject */


int set_follow_flag(char *group, long article_id, int value)
{
struct article *pa, *ps, *lookup_article();
char temp[80];
char *ptr1, *ptr2;

if(debug_flag)
	{
	fprintf(stdout,\
	"set_follow_flag(): arg group=%s article_id=%ld value=%d\n",\
	group, article_id, value);
	}

/* argument check */
if(! group) return(0);
if(article_id < 0) return(0);

load_articles(group);

sprintf(temp, "%ld", article_id);
ps = lookup_article(temp);
if(! ps) return(0);

if(value == FOLLOW_THREAD)
	{
	ps -> follow_flag |= FOLLOW_THIS_THREAD;
	save_articles(group);
	return(1);
	}
if(value == DO_NOT_FOLLOW_THREAD)
	{
	ps -> follow_flag &= (255 - FOLLOW_THIS_THREAD);
	save_articles(group);
	return(1);
	}

if( (value == FOLLOW_SUBJECT) || (value == DO_NOT_FOLLOW_SUBJECT) )
	{
	/* strip possible "Re: " */
	if(strstr(ps -> subject, "Re: ") == ps -> subject)
		{
		ptr2 = ps -> subject + 4;
		}
	else ptr2 = ps -> subject;

	/* modify the follow flag in all articles with this subject */
	for(pa = articletab[0]; pa != 0; pa = pa -> nxtentr)
		{
		/* strip the possible "Re: ". */
		if(strstr(pa -> subject, "Re: ") == pa -> subject)
			{
			ptr1 = pa -> subject + 4;
			}
		else ptr1 = pa -> subject;

		/* test if subject match */
		if(strcmp(ptr1, ptr2) == 0)
			{
			if(value == FOLLOW_SUBJECT)
				{
				pa -> follow_flag |= FOLLOW_THIS_SUBJECT;
				}
			if(value == DO_NOT_FOLLOW_SUBJECT)
				{
				pa -> follow_flag &= (255 - FOLLOW_THIS_SUBJECT);
				}
			}/* end match */
		}/* end for all articles */

	save_articles(group);
	return(1);
	}/* end if value is follow subject or value is do not follow subject */

/* if here illegal value */
return(0);
}/* end function set_follow_flag */


int search_next_article(char *search_groups_keywords)
{
struct article *pa, *lookup_article();
char temp[80];
extern long atol();
char *ptr;
extern char *strsave();
int group_match;

if(debug_flag)
	{
	fprintf(stdout,\
	"search_next_article(): arg search_groups_keywords=%s\n\
selected_group=%s selected_artile=%ld\n",\
	search_groups_keywords, selected_group, selected_article);	
	}

/* in case called from idle callback, prevent looping */
go_search_next_article_from_idle_cb_flag = 0;

search_last_search = SEARCH_ARTICLES;

load_articles(selected_group);

/* move to present position */

sprintf(temp, "%ld", selected_article);
pa = lookup_article(temp);
if(! pa) pa = articletab[0];
else pa = pa -> nxtentr; /* select next position */
/* and if pa == 0, for() will end, and next group is selected */

while(1)/* for all groups */
	{
	if(strlen(search_groups_keywords) == 0) group_match = 1;
	else
		{
		/* test if match search_groups_keywords for this group */
		group_match = search_in_for(\
		selected_group, search_groups_keywords, search_case_sensitive_flag);
		}
	/* search for next article from this point onwards */
	/* for all articles in group */
	for(pa = pa; pa != 0; pa = pa -> nxtentr)
		{
		/* test for permission to show header */
		if(filters_enabled_flag)
			{
			if(pa -> filter_flags & FILTER_DO_NOT_SHOW_HEADER) continue;
			}/* end if filters_enabled_flag */
			
		/*
		a special case is when there is only one matching article,
		and it is the selected one.
		If we do not stop here, it will be found over and over again.
		*/
		if( (atol(pa -> name) == first_matching_article) &&\
		(strcmp(selected_group, first_matching_group) == 0) )
			{
/*			ask_question_flag = 0;*/
			fl_show_alert(\
			"All has been searched", "", "", 0);
			return(0);/* round */
			}

		if(group_match)
			{
			if(search_all(\
			selected_group, pa -> name, pa -> encoding,\
			pa -> attachment_name, pa -> have_body_flag, SEARCH_ARTICLES) )
				{
				selected_article = atol(pa -> name);
				show_browser(ARTICLE_BODY_BROWSER);
				refresh_screen(selected_group);
				if(first_matching_article == -1)
					{
					first_matching_article = atol(pa -> name);
					free(first_matching_group);
					first_matching_group = strsave(selected_group);
					}
				return(1);
				}/* end found */
			}/* end if group match */
		/* test for round */
		if(strcmp(selected_group, start_group) == 0)
			{
			if(atol(pa -> name) == start_article)
				{
/*				ask_question_flag = 0;*/
				go_search_next_article_from_idle_cb_flag = 0;
				fl_show_alert("All has been searched", "", "", 0);
				return(0);/* round */
				}
			}
		}/* end for all structures in this group */
	/* no more in this group */
	ptr = (char *) get_next_subscribed_group(selected_group);

	if(debug_flag)
		{
		fprintf(stdout, "search_next_article(): ptr=%s\n", ptr);
		}

	if(! ptr) return(0);/* possibly no subscribed groups */
	if(debug_flag)
		{
		fprintf(stdout,\
		"search_next_article(): next subscribed group=%s\n", ptr);
		}
	if(selected_group) free(selected_group);
	selected_group = strsave(ptr);
	load_articles(selected_group);
	pa = articletab[0];
	}/* end while */

return(1);
}/* end function search_next_article */


int set_all_filter_flags_in_group(char *group)
{
struct article *pa;

if(debug_flag)
	{
	fprintf(stdout, "set_all_filter_flags_in_group(): arg group=%s\n",\
	group);
	}
	
/* argument check */
if(! group) return(0);

load_articles(group);

for(pa = articletab[0]; pa != 0; pa = pa -> nxtentr)
	{
	/* get the filter result */
	pa -> filter_flags = 0;/* the default */

	get_filter_flags(\
	group, pa -> name, pa -> encoding, pa -> attachment_name,\
	pa -> have_body_flag, SEARCH_ARTICLES, &(pa -> filter_flags) );
	
	}/* end for all structures */

save_articles(group);
return(1);
}/* end function set_all_filter_flags_in_group */


int apply_filters_to_article(char *group, long article_id)
{
struct article *pa, *lookup_article();
char article_name[50];

if(debug_flag)
	{
	fprintf(stdout,\
	"apply_filters_to_article_header(): arg group=%s article_id=%ld\n",\
	group, article_id);
	}

/* argument check */
if(! group) return(0);
if(article_id < 0) return(0);

load_articles(group);

sprintf(article_name, "%ld", article_id);
pa = lookup_article(article_name);
if(! pa) return(0);

get_filter_flags(\
group, article_name, pa -> encoding, pa -> attachment_name,\
pa -> have_body_flag, SEARCH_ARTICLES, &(pa -> filter_flags) );

save_articles(group);
return(1);
}/* end function apply_filters_to_article_header */


int set_all_retrieval_flags_in_new_articles(char *group, int state)
{
struct article *pa;

if(debug_flag)
	{
	fprintf(stdout,\
	"set_all_retrieval_flags_in_new_articles(): arg group=%s state=%d\n",\
	group, state);
	}
	
/* argument check */
if(! group) return(0);

load_articles(group);

/*pa points to next entry*/
for(pa = articletab[0]; pa != 0; pa = pa -> nxtentr)
	{
	if(! pa -> have_body_flag)
		{
		pa -> retrieval_flag = state;
		}
	}

save_articles(group);
return(1);
}/* end function set_all_retrieval_flags_in_new_articles */


