/* 
xagent offline NNTP news reader
xagent is registered Copyright (C) 1997 <Jan Mourer>
email: pante@pi.net
snail mail: PO BOX 61  SINT ANNA  9076ZP HOLLAND

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "xagent.h"

extern FD_xagent *fd_xagent;
extern FD_xagent *fdui;

FL_APPEVENT_CB my_idle_cb;


struct passwd *userinfo;

struct in_addr local_ip_address;

struct command
	{
	char *name;
	int type;
	int send_flag;
	char *group;
	long article;
	long sequence;
	struct command *nxtentr;
	struct command *prventr;
	};
struct command *commandtab[2];


struct command *lookup_command(char *name)
{
struct command *pa;

if(debug_flag) 
	{
	fprintf(stdout, "lookup_command(): arg name=%s\n", name);
	}

for(pa = commandtab[0]; pa != 0; pa = pa -> nxtentr)
	{
	if(strcmp(pa -> name, name) == 0) return(pa);
	}
return(0); /* not found */
}


struct command *install_command_at_end_of_list(char *name)
{
struct command *plast, *pnew;
struct command *lookup_command();
char *strsave();
char *malloc();
char *calloc();

if(debug_flag)
	{
	fprintf(stdout, "install_command_at_end_of_list(): arg name=%s\n", name);
	}

pnew = lookup_command(name);
if(pnew)
	{
	/* else already there */
	fprintf(stdout,\
	"install_command_at_end_of_list(): SERIOUS ERRROR, ALREADY THERE\n");
	return(0);
	}

/* create new structure */
pnew = (struct command *) calloc(1, sizeof(*pnew) );
if(! pnew) return(0);
pnew -> name = strsave(name);
if(! pnew -> name) return(0);
pnew -> group = strsave("");
if(! pnew -> group) return(0);

/* get previous structure */
plast = commandtab[1]; /* end list */

/* set new structure pointers */
pnew -> nxtentr = 0; /* new points top zero (is end) */
pnew -> prventr = plast; /* point to previous entry, or 0 if first entry */

/* set previuos structure pointers */
if( !commandtab[0] ) commandtab[0] = pnew; /* first element in list */
else plast -> nxtentr = pnew;

/* set array end pointer */
commandtab[1] = pnew;

return(pnew);/* pointer to new structure */
}/* end function install_command_at_end_of_list */


int delete_command(char *name)/* delete entry from double linked list */
{
struct command *pa, *pprev, *pdel, *pnext;

if(debug_flag)
	{
	fprintf(stdout, "delete_command(): arg name=%s\n", name);
	}

/* parameter check */
if(! name) return(0);

pa = commandtab[0];
while(1)
	{
	/* if end list, return not found */
	if(! pa) return(0);

	/* test for match in name */
	if(strcmp(name, pa -> name) != 0) /* no match */
		{
		/* point to next element in list */
		pa = pa -> nxtentr;

		/* loop for next element in list */
		continue;
		}

	/* we now know which struture to delete */
	pdel = pa;

	/* get previous and next structure */
	pnext = pa -> nxtentr;
	pprev = pa -> prventr;

	/* set pointers for previous structure */
	/* if first one, modify commandtab[0] */
	if(pprev == 0) commandtab[0] = pnext;
	else pprev -> nxtentr = pnext;

	/* set pointers for next structure */
	/* if last one, modify commandtab[1] */
	if(pnext == 0) commandtab[1] = pprev;
	else pnext -> prventr = pprev;
	
	/* delete structure */	
	/* delete structure data */	
	free(pdel -> name);/* free name */
	free(pdel -> group);/* free group */
	free(pdel); /* free structure */

	/* return OK deleted */
	return(1);
	}/* end for all structures */
}/* end function delete_command */


int delete_all_commands()/* delete all entries from table */
{
struct command *pa;

if(debug_flag)fprintf(stdout, "delete_all_commands(): arg none\n");

while(1)
	{
	pa = commandtab[0];
	if(! pa) break;
	commandtab[0] = pa -> nxtentr;/* commandtab entry points to next one,
												            this could be 0 */
	free(pa -> name);/* free name */
	free(pa -> group);
	free(pa);/* free structure */
	}/* end for all structures */
commandtab[1] = 0;
return(0);/* not found */
}/* end function delete_all_commands */


int add_to_command_queue(char *group, long article, int command)
{
struct command *pa;
extern char *strsave();
static long sequence;
char temp[50];

if(debug_flag)
	{
	fprintf(stdout,\
	"add_to_command_queue(): arg group=%s article=%ld command=%d\n",\
	group, article, command);
	}

/* argument check */
if(! group) return(0);
if(! command) return(0);
if(article < 0) return(0);

if(command == POST_ARTICLE)
	{
	if(server_mode != POSTING_ALLOWED)
		{
		fl_show_alert(\
		"No posting allowed on this server", "posting cancelled", "", 0);
		return(0);
		}
	}
/* test if command already in queue */
for(pa = commandtab[0]; pa != 0; pa = pa -> nxtentr)
	{
	if(pa -> article == article)
		{
		if(pa -> type == command)
			{
			if(strcmp(pa -> group, group) == 0) return(1);/* already there */
			}
		}	
	}/* end for all structures */

sequence++;/* avoid the zero for a start */

sprintf(temp, "%ld", sequence);
pa = install_command_at_end_of_list(temp);
if(!pa) return(0);

pa -> type = command;

free(pa -> group);
pa -> group = strsave(group);
if(! pa -> group) return(0);

pa -> article = article;
pa -> send_flag = 0;
return(1);
}/* end function add_to_command_queue */


int process_command_queue()
/*
called from idle callback
connect to server,
send a command from the structure,
remove that entry from the structure,
return and wait for a server reply (in io_read_cb)
if no more entries in structure close server connection.
*/
{
struct command *pa;
char temp[512];
char info[512];
int queue_size;
extern char *strsave();
extern int idle_cb();

if(debug_flag)fprintf(stdout, "process_command_queue(): arg none\n");

for(pa = commandtab[0]; pa != 0; pa = pa -> nxtentr)
	{
	/* connect to server, if not already there (not for email) */
	if( (server_status == DISCONNECTED) && (pa -> type != EMAIL_ARTICLE) )
		{
		if(! connect_to_server(news_server_name, news_server_port) )
			{
			/* clear command status display */
			to_command_status_display("");

			if(auto_disconnect_from_net_flag) auto_disconnect_from_net();
			return(0);
			}
		
		/* start the timer */
		online_timer = time(0);
		bytes_received = 0;
			
		fl_add_io_callback(socketfd,\
		FL_READ, (FL_IO_CALLBACK) io_read_callback, 0);
		
		if(last_acquire_group) free(last_acquire_group);
		last_acquire_group = 0;/* no group */
			
		clear_new_articles_list();
		}
		
	/* report the tasks remaining */
	queue_size = get_command_queue_size();
	sprintf(temp, "Tasks %d", queue_size);
	fl_set_object_label(fdui -> line_status_display, temp);

	/* make the stop button red */
	fl_set_object_color(fdui -> stop_button, FL_RED, FL_COL1);

	/* xagent_cb.c needs to know what we are doing */
	free(acquire_group);
	acquire_group = strsave(pa -> group);
	acquire_article = pa -> article;/* long */

	/* send the command */
	if(pa -> type == LIST)
		{
		sprintf(info, "Refreshing group list");
		sprintf(temp, "LIST\n");
		command_in_progress = LIST;
		}
	if(pa -> type == GET_NEW_HEADERS_IN_GROUP)
		{
		sprintf(info, "Getting new headers in group %s", pa -> group);
		sprintf(temp, "GROUP %s\n", pa -> group);
		}	 	
	if(pa -> type == GET_ARTICLE_BODY)
		{
		sprintf(info, "Getting article body");
		/* 
		like get new headers, but command_in_progress is
		checked for in io_callback, and then ONE body is requested
		for this group. (we already have the header)
		
		if not in this group select group,
		this will also retrieve the article body.
		*/
		if( (! last_acquire_group) ||\
		(strcmp(pa -> group, last_acquire_group) != 0) )
			{
			sprintf(temp, "GROUP %s\n", pa -> group);
			}
		else /* get only body */
			{
			sprintf(temp, "BODY %ld\n", pa -> article);
			}
		}
	if(pa -> type == POST_ARTICLE)
		{
		sprintf(info, "posting article %ld", pa -> article);
		posting_article = pa -> article;
		sprintf(temp, "POST\n");
		}
	if(pa -> type == EMAIL_ARTICLE)
		{
		sprintf(info, "mailing article %ld", pa -> article);
   		my_command_status_display(info);
		post_email(pa -> article);
		my_command_status_display("");
		delete_command(pa -> name);
   	    refresh_screen("");/* using dummy for group */
		if(server_status == DISCONNECTED)
			{
			fl_set_object_label(fdui -> line_status_display, "Offline");
			/* set the color for the stop button */
			set_stop_button_off_state_color();	
			}
        return(1);/* the next time here called from idle_cb */
		}	
	send_to_server(temp);	
	pa -> send_flag = atol(pa -> name);
	command_in_progress = pa -> type;
	expect_numeric_flag = 1;

	delete_command(pa -> name);
	/* 
	pa NO LONGER DEFINED/
	now waiting for a response from the server,
	after that is processed the command_in_progress is reset
	*/
	/* inform user what is happening */
/*
	fl_set_object_label(fdui -> command_status_display, info);
*/
	my_command_status_display(info);	
	return(1);/* the next time here called from idle_cb */
	}/* end for all structures */

/* 
	command queue now empty 
	There may be new headers that are marked by the thread routines in
	articles.c
*/
/*goto no_threads;*/ /*_______________ DISABLED FOR TEST _______________ */

if(new_thread_bodies_flag)
	{
fprintf(stdout, "MARKING FOR THREAD\n");

	add_marked_article_bodies_in_subscribed_groups_to_command_queue();
 	/* and reset this flag (so no loop). */
	new_thread_bodies_flag = 0;

fprintf(stdout, "LOOPING FOR THREAD\n");
	return(1);
	}

if(server_status != DISCONNECTED)
	{
	send_to_server("QUIT\n");

	fl_remove_io_callback(socketfd,\
	FL_READ, (FL_IO_CALLBACK) io_read_callback);

	close(socketfd);
	server_status = DISCONNECTED;

	fl_set_object_label(fdui -> line_status_display, "Offline");

	my_command_status_display("");

	/* set the color for the stop button */
	set_stop_button_off_state_color();	

	save_subscribed_groups();
	
	if(auto_disconnect_from_net_flag) auto_disconnect_from_net();

	set_new_data_flag(1);
	
	if(filters_enabled_flag) set_all_filter_flags_for_new_articles();
	
	}/* end server status != disconnected */

return(0);
}/* end function process_command_queue */


int get_command_queue_size()
{
struct command *pa;
int queue_size;

if(debug_flag)fprintf(stdout, "get_command_queue_size() arg none\n");

queue_size = 0;
for(pa = commandtab[0]; pa != 0; pa = pa -> nxtentr)
	{
	queue_size++;
	}/* end for all structures */
return(queue_size);
}/* end function get_command_queue_size */


int connect_to_server(char *server, int port)
{
struct hostent *hp;
struct sockaddr_in sa; /* -Wall gets confused */
int a, i;
char temp[512];
char dotted_quad[512];
int flags;
time_t connect_timer;
extern int idle_cb();
FILE *exec_filefd;
FILE *test_filefd;
int test_seconds;
int connect_flag;

if(debug_flag)
	{
	fprintf(stdout, "connect_to_server(): arg server=%s port=%d\n",\
	server, port);
	}
	
/* argument check */
if(! server) return(0);
if(port <= 0) return(0);

/* Connect to the internet if auto connect enabled */
if(auto_connect_to_net_flag)
	{
	sprintf(temp, "Auto connect to net: Testing for %s",\
	pid_test_program_name);
	if(debug_flag)
		{
		fprintf(stdout, "%s\n", temp);
		}
	test_seconds = 0;
	while(1)
		{
		/* if pppd was succesful, it makes a pid file /var/run/ppp0.pid */
		if(debug_flag)
			{
			fprintf(stdout, "testing for %s seconds=%d timeout=120\n",\
			pid_test_program_name, test_seconds);
			}
		test_filefd = fopen(pid_test_program_name, "r");
		if(! test_filefd)
			{
			if(test_seconds == 0)
				{
				sprintf(temp, "starting connect to net script %s",\
				connect_to_net_program_name);
				if(debug_flag)
					{
					fprintf(stdout, "%s\n", temp);
					}
				to_command_status_display(temp);

				exec_filefd = popen(connect_to_net_program_name, "r");
				if(! exec_filefd)
					{
					my_idle_cb = fl_set_idle_callback(0, 0);
					fl_show_alert("Cannot execute file:",\
					connect_to_net_program_name, "", 0);
					fl_set_idle_callback(idle_cb, my_idle_cb);
					return(0);
					}
				pclose(exec_filefd);
				}/* end test_seconds = 0 */
			if(test_seconds > 120)
				{
				my_idle_cb = fl_set_idle_callback(0, 0);
				fl_show_alert("Cannot connect to net",\
				"pppd not running", "", 0);
				fl_set_idle_callback(idle_cb, my_idle_cb);
				return(0);
				}
			}
		else 
			{
			fclose(test_filefd);		
			break;
			}
		sleep(1);/* take it easy */
		test_seconds++;
		}/* end while check for pppd0 .pid */
	
	}/* end if auto_connect_to_net_flag */

if(debug_flag) fprintf(stdout, "connected\n");

/* execute any auto execute program */
if(auto_execute_program_flag)
	{
	sprintf(temp, "starting autoexecute script %s",\
	auto_execute_program_name);
	if(debug_flag)
		{
		fprintf(stdout, "%s\n", temp);
		}
	to_command_status_display(temp);

	exec_filefd = popen(auto_execute_program_name, "r");
	if(! exec_filefd)
		{
		my_idle_cb = fl_set_idle_callback(0, 0);
		fl_show_alert("Cannot execute file:", auto_execute_program_name,\
		 "", 0);
		fl_set_idle_callback(idle_cb, my_idle_cb);
		return(0);
		}
	pclose(exec_filefd);
	}/* end if auto_exec_program_flag */

sprintf(temp, "getting host %s by name", server);
if(debug_flag)
	{
	fprintf(stdout, "%s\n", temp);
    }
to_command_status_display(temp);

hp = gethostbyname(server);
if(hp == 0)
	{
	sprintf(temp,\
	"gethostbyname failed: cannot get host %s by name", server);
	if(debug_flag) fprintf(stdout, "%s\n", temp);

	/* 
	make sure if running unattended that program does not wait for user
	to press OK in fl_show_alert, since the dialup would never disconnect.
	*/
	if(! time_match_flag)
		{
		my_idle_cb = fl_set_idle_callback(0, 0);
		fl_show_alert(temp, "", "", 0);
		fl_set_idle_callback(idle_cb, my_idle_cb);
		}
	else
		{
		/* NO WAIT */
		to_command_status_display(temp);
		}
	return(0);
	}

/* gethostbyname() leaves port and host address in network byte order */

bzero(&sa, sizeof(sa) );
bcopy(hp -> h_addr, (char *)&sa.sin_addr, hp -> h_length);
sa.sin_family = AF_INET;
sa.sin_port = htons( (u_short)port);

/* sa.sin_addr and sa.sin_port now in network byte order */

/* create a socket */
socketfd = socket(hp -> h_addrtype, SOCK_STREAM, 0);
if(socketfd < 0)
	{
	sprintf(temp, "Cannot create socket");
	if(debug_flag) fprintf(stdout, "%s\n", temp);
				
	my_idle_cb = fl_set_idle_callback(0, 0);
	fl_show_alert(temp, "", "", 0);
	fl_set_idle_callback(idle_cb, my_idle_cb);
				
	return(0);
	}/* end socketfd < 0 */

sprintf(dotted_quad, "%s", (char *)inet_ntoa(sa.sin_addr.s_addr) );

/* set for nonblocking socket */
if (fcntl(socketfd, F_SETFL, O_NONBLOCK) < 0)
	{
	sprintf(temp, "Cannot set socket non blocking");
	if(debug_flag) fprintf(stdout, "%s\n", temp);
				
	if(! time_match_flag)
		{
		my_idle_cb = fl_set_idle_callback(0, 0);
		fl_show_alert(temp, "", "", 0);
		fl_set_idle_callback(idle_cb, my_idle_cb);
		}
	else
		{
		to_command_status_display(temp);
		}
	return(0);
	}

/* connect */
sprintf(temp, "Connecting to server %s (%s)  port %d  timeout %d",\
server, dotted_quad, port, connect_to_server_timeout);
if(debug_flag) fprintf(stdout, "%s\n", temp);
to_command_status_display(temp);

/* server sometimes does not seem to answere to the first connect */
/* not so sure this is the way to do it, may change this later */
for(i = 0; i < 5; i++)
	{
	/* start the connect timer */
	connect_flag = 0;
	connect_timer = time(0);
	while(1)
		{
		a = connect(socketfd, (struct sockaddr*)&sa, sizeof(sa) );
		if(a == 0)
			{
			connect_flag = 1;
			break;/* connected */
			}
		if(debug_flag) fprintf(stdout, "In connect errno=%d\n", errno);
	
		/* test for connect time out */
		if( (time(0) - connect_timer) > connect_to_server_timeout)
			{
			sprintf(temp,\
			"Connecting to server %s (%s)  port %d  timeout %d  retry %d (of 4)",\
			server, dotted_quad, port, connect_to_server_timeout, i + 1);
			if(debug_flag) fprintf(stdout, "Connect timeout %s\n", temp);
				
			if(! time_match_flag)
				{
				/* tired of pressing OK 4 times or more */
/*
				my_idle_cb = fl_set_idle_callback(0, 0);
				fl_show_alert(temp, "", "", 0);
				fl_set_idle_callback(idle_cb, my_idle_cb);
*/
				to_command_status_display(temp);
				}
			else
				{
				to_command_status_display(temp);
				}
				
			/* try again */
			break;
		
			/* close the socket */
			close(socketfd);
		
			return(0);
			}/* end timeout */
		}/* end while connect */

	if(connect_flag) break;

	}/* end for */

if(! connect_flag)
	{
	/* close the socket */
	close(socketfd);
		
	return(0);
	}

/* now set blocking */
flags = fcntl(socketfd, F_GETFL, 0);
flags &= ~O_NONBLOCK;
fcntl(socketfd, F_SETFL, flags);

server_status = CONNECTED;

sprintf(temp, "Connected to server %s (%s)  port %d",\
server, dotted_quad, port);
if(debug_flag) fprintf(stdout, "%s\n", temp);
to_command_status_display(temp);

return(1);
}/* end function connect_to_server */


int send_to_server(char *text)/* send data to server */
{
int a, c, i, j;
char txbuf[READSIZE];

if(debug_flag)
	{
	fprintf(stdout, "send_to_server(): arg\ntext=%s (socketfd=%d)\n",\
	text, socketfd);
	}
	
if(server_status == DISCONNECTED)
	{
	my_show_alert("You are not connected to a server", "", "");
	return(0);
	}

/*fprintf(stdout, "text=%s\n", text);*/

/* expand any LF ('\n' 10 decimal) to CRLF (13 10) */
i = 0;
j = 0;
while(1)
	{
	c = text[i];
	if(c == '\n')
		{
/*fprintf(stdout, "expanding\n");*/
		txbuf[j] = 13;
		j++;
		if(j > READSIZE)
			{
			if(debug_flag)
				{
				fprintf(stdout,\
				"send_to_server(): overflow in expanding text=%s\n", text);
				}
			return(0);
			}		
		}
	txbuf[j] = c;
	if(! c) break;
	i++;
	j++;
	}

/*fprintf(stdout, "expanded text=%s\n", txbuf);*/

a = write(socketfd, txbuf, strlen(txbuf) );     
if(a < 0)
	{
	if(debug_flag)
		{
		fprintf(stdout, "\nwrite failed, errno=%d\n", errno);
		}
	perror("send_to_server(): write failed:");
	return(0);
	}

if(debug_flag)
	{
	fprintf(stdout, "txbuf:%s", txbuf);
	}

return(1);
}/* end function send_to_server */


int auto_disconnect_from_net()
{
extern int idle_cb();
char temp[512];
FILE *disconnect_filefd;

if(debug_flag) fprintf(stdout, "auto_disconnect_from_net(): arg none\n");

to_command_status_display("Starting auto disconnect from net script");

/* 
la = fl_exe_command(disconnect_from_net_program_name, 1);
if(la != 256)
*/

disconnect_filefd = popen(disconnect_from_net_program_name, "r");
if(! disconnect_filefd)
	{
	/* 
	let the user know something is wrong, and he should manually
	disconnect from the ISP, or pay the phone.
	*/
	audio_alarm_flag = 1;/* pushing STOP will stop alarm */
	if(! time_match_flag)
		{
		my_idle_cb = fl_set_idle_callback(0, 0);
		fl_show_alert(\
		"Cannot execute file:", disconnect_from_net_program_name,\
		"Cannot disconnect from net", 0);
		fl_set_idle_callback(idle_cb, my_idle_cb);
		}
	else
		{
/*
		fl_show_alert(\
		"Cannot execute file:", disconnect_from_net_program_name,\
		"Cannot disconnect from net", 0);
*/
		sprintf(temp, "Cannot execute file %s cannot disconnect from net",\
		disconnect_from_net_program_name);
		to_command_status_display(temp);
		}
	return(0);
	}/* end if error */
pclose(disconnect_filefd);
to_command_status_display("");
return(1);
}/* end function auto_disconnect_from_net */


int to_command_status_display(char *text)
{
if(debug_flag)
	{
	fprintf(stdout, "to_command_status_display(): arg text=%s\n", text);
	}

/* argument check */
if(! text) return(0);
	
fl_set_object_label(fdui -> command_status_display, text);
/* This is a trick to force command status display to be shown */
XSync(fl_get_display(), 0);
return(1);
}/* end function to_command_status_display */


