/* 
xagent offline NNTP news reader
xagent is registered Copyright (C) 1997 <Jan Mourer>
email: pante@pi.net
snail mail: PO BOX 61  SINT ANNA  9076ZP HOLLAND

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/* 
In order to improve performance I did the following:
there are 2 files, groups.dat and subscribed_groups.dat.
At load these are merged together, in such a way that:
pa -> subscribed_flag, pa -> new_articles, and pa -> article_ptr are taken
from subscribed_groups.dat.
Now anytime online either new_articles or article_ptr is changed, only
the structure is changed, and subscribed_groups.dat is written to disk.
Since the groups.dat is VERY long (in my case more then 850 KByte), and
subscribed_groups.dat is much shorter (normally :) ), this reduces online
time and saves money.
groups.dat is only saved after a LIST (online, then refresh group list).
This also prevents file corruption of groups.dat if anything goes wrong.
*/


#include "xagent.h"

extern FD_xagent *fdui;

#define GROUPS_HASH_SIZE 100

int hash(s)/* form hash value for string s */
char *s;
{
int hashval;

for(hashval = 0; *s != '\0';) hashval += *s++;
/* sum of ascii value of characters divided by tablesize */
return(hashval % GROUPS_HASH_SIZE);
}


struct group
	{
	char *name;
	char posting_allowed_flag;
	int subscribed_flag;
	long article_ptr;
	int group_status;
	long new_articles;
	int filter_flags;
	struct group *nxtentr;
	struct group *prventr;
	};
struct group *grouptab[GROUPS_HASH_SIZE];


struct group *lookup_group(char *name)
{
struct group *pa;

if(debug_flag)
	{
	if(debug_flag)fprintf(stdout, "lookup_group(): arg name=%s\n", name);
	}
	
/*
fprintf(stdout,\
"lookup_group(): looking up name=%s at grouptab=%d loc=%ld\n",\
name, hash(name), grouptab[hash(name)]);
*/

for(pa = grouptab[hash(name)]; pa != 0; pa = pa -> nxtentr)
	{
	if(strcmp(pa -> name, name) == 0) return(pa);/* found sequence entry */
	}
return(0); /* not found */
}/* end function lookup_group */


struct group *install_group(char *name)
{
struct group *pa, *pnew, *pnext, *lookup_group();
char *strsave();
char *malloc();
int hashval;

if(debug_flag)
	{
	if(debug_flag)fprintf(stdout, "install_group(): arg name=%s\n", name);
	}
	
pa = lookup_group(name);

if(!pa) /* not found */
	{
	/* create new structure */
	pnew = (struct group *) calloc(1, sizeof(*pnew) );
	if(! pnew) return(0);
	pnew -> name = strsave(name);
	if(! pnew -> name) return(0);

	/* get next structure */
	hashval = hash(name);
	pnext = grouptab[hashval];/* may be zero, if there was nothing */

/*
fprintf(stdout, "install_group(): installing name=%s at grouptab=%d\n",\
name, hashval);
*/
	
	/* insert before next structure (if any, else at start) */
	grouptab[hashval] = pnew;

	/* set pointers for next structure */
	if(pnext) pnext -> prventr = pnew;
	
	/* set pointers for new structure */
	pnew -> nxtentr = pnext;
	pnew -> prventr = 0;/* always inserting at start of chain of structures */
	
	return(pnew);/* pointer to new structure */
	}/* end if not found */
return(pa);
}/* end function install_group */


int delete_all_groups()
{
struct group *pa;
int i;

for(i = 0; i < GROUPS_HASH_SIZE; i++)/* for all structures at this position */
	{
	while(1)
		{
		pa = grouptab[i];
		if(! pa) break;
		grouptab[i] = pa -> nxtentr;/* grouptab entry points to next one,
															this could be 0
															*/
		free(pa -> name);/* free name */
		free(pa);/* free structure */
		}/* end while all structures hashing to this value */ 
	}/* end for all entries in grouptab */
return(0);/* not found */
}/* end function delete_all_groups */


int get_article_ptr(char *group, long *article_ptr)
{
struct group *lookup_group(), *pa;

if(debug_flag)
	{
	fprintf(stdout, "get_article_ptr(): arg group=%s article_ptr=%ld\n",\
	group, *article_ptr);
	}

pa = lookup_group(group);
if(! pa) return(0);

*article_ptr = pa -> article_ptr;
return(1);
}/* end function get_article_ptr */


int set_article_ptr(char *group, long article_ptr)
{
struct group *lookup_group(), *pa;

if(debug_flag)
	{
	fprintf(stdout, "set_article_ptr(): arg group=%s article_ptr=%ld\n",\
	group, article_ptr);
	}
	
pa = lookup_group(group);
if(! pa) return(0);

pa -> article_ptr = article_ptr;
return(1);
}/* end function set_article_ptr */


int set_new_articles(char *group, long new_articles)
{
struct group *lookup_group(), *pa;

if(debug_flag)
	{
	fprintf(stdout, "set_new_articles(): arg group=%s new_articles=%ld\n",\
	group, new_articles);
	}
	
/* argument_check */
if(! group) return(0);
if(new_articles < 0) return(0);

pa = lookup_group(group);
if(! pa) return(0);

pa -> new_articles = new_articles;

save_subscribed_groups();
return(1);
}/* end function set_new_articles */


int mark_all_groups_cancelled()
{
int i;
struct group *pa;

if(debug_flag) fprintf(stdout, "mark_all_groups_cancelled(): arg none\n");

for(i = 0; i < GROUPS_HASH_SIZE; i++)/* for all structures at this position */
	{
	for(pa = grouptab[i]; pa != 0; pa = pa -> nxtentr)	
		{
		pa -> group_status = CANCELLED_GROUP;
		}/* end for all structures in the chain */
	}/* end for all entries in grouptab */
return(1);
}/* end function mark_all_groups_canncelled */


int load_groups()
{
int a;
FILE *allgroupsfile;
char temp[512];/* lines in groups dat are of know size shorter then this */
char pathfilename[512];
char name[512];
char posting_allowed_flag;
int subscribed_flag;
long article_ptr;
int group_status;
long new_articles;
int filter_flags;
int hashval;
struct group *pa;
extern char *strsave();

if(debug_flag) fprintf(stdout, "load_groups(): arg none\n");

if(groups_loaded_flag) return(1);

delete_all_groups();

to_command_status_display("Loading groups file, this may take a while");

sprintf(pathfilename, "%s/.xagent/groups.dat", home_dir);
allgroupsfile = fopen(pathfilename, "r");
if(! allgroupsfile)
	{
	if(debug_flag)
		{
		fprintf(stdout, "could not load file %s\n", pathfilename);
		}
	return(0);
	} 

while(1)
	{
	a = readline(allgroupsfile, temp);
	if(a == EOF)
		{
		fclose(allgroupsfile);

		/* merge subscribed groups here */
		load_subscribed_groups(MERGE_DATA);

		groups_loaded_flag = 1;
		to_command_status_display("");		
		return(1);
		}

	sscanf(temp, "%s %c %d %ld  %d %ld %d",\
	name, &posting_allowed_flag, &subscribed_flag, &article_ptr,\
	&group_status, &new_articles, &filter_flags);
	
	pa = (struct group *) malloc( sizeof(*pa) );
	if(! pa)
		{
		if(debug_flag)
			{
			fprintf(stdout, "load_groups: cannot install group %s\n", temp);
			}
		return(0);
		}

	pa -> name = strsave(name);
	if(! pa -> name) return(0);

	hashval = hash(name);
	pa -> nxtentr = grouptab[hashval];
	if(grouptab[hashval]) /* not first one */
		{
		grouptab[hashval] -> prventr = pa;
		}
	pa -> prventr = 0;/* sorry forgot that one in xagent.05 and before */
	grouptab[hashval] = pa;

	pa -> posting_allowed_flag = posting_allowed_flag;
	pa -> subscribed_flag = subscribed_flag;
	pa -> article_ptr = article_ptr;
	pa -> group_status = group_status;
	pa -> new_articles = new_articles;
	pa -> filter_flags = filter_flags;
	}/* end while all lines in file */
}/* end function load_groups */


int load_subscribed_groups(int mode)
{
int a;
FILE *subscribedgroupsfile;
char temp[512];/* lines in groups dat are of know size shorter then this */
char pathfilename[512];
char name[512];
char posting_allowed_flag;
int subscribed_flag;
long article_ptr;
int group_status;
long new_articles;
int filter_flags;
 
struct group *pa;
extern char *strsave();

if(debug_flag)
	{
	fprintf(stdout, "load_subscribed_groups(): arg mode=%d\n", mode);
	}

pa = 0;/* only for clean compilation with -Wall */

sprintf(pathfilename, "%s/.xagent/subscribed_groups.dat", home_dir);
subscribedgroupsfile = fopen(pathfilename, "r");
if(! subscribedgroupsfile)
	{
	if(debug_flag) fprintf(stdout, "could not load file %s\n", pathfilename);
	return(0);
	} 
while(1)
	{
	a = readline(subscribedgroupsfile, temp);
	if(a == EOF)
		{
		fclose(subscribedgroupsfile);
		return(1);
		}

	sscanf(temp, "%s %c %d %ld  %d %ld %d",\
	name, &posting_allowed_flag, &subscribed_flag, &article_ptr,\
	&group_status, &new_articles, &filter_flags);

	if(mode == MERGE_DATA) pa = lookup_group(name);
	if(mode == USE_ALL_DATA) pa = install_group(name);
	if(! pa)
		{
		if(mode == MERGE_DATA)
			{
			if(debug_flag)
				{
				fprintf(stdout,\
				"load_subscribed_groups: cannot lookup group %s\n", temp);
				}
			continue;/* xagent.help is not there likely */
			}
		if(mode == USE_ALL_DATA)
			{
			if(debug_flag)
				{
				fprintf(stdout,\
				"load_subscribed_groups: cannot install group %s\n", temp);
				}
			return(0);/* problem memory alocation */
			}
		}
		
	if(debug_flag)
		{
		fprintf(stdout,\
		"load_subscribed_groups(): article_ptr=%ld new_articles=%ld\n",\
		article_ptr, new_articles);
		}

	if(mode == MERGE_DATA)
		{
		pa -> article_ptr = article_ptr;
		pa -> new_articles = new_articles;
		pa -> subscribed_flag = 1;
		pa -> filter_flags = filter_flags;
		}/* end mode merge data */

	if(mode == USE_ALL_DATA)
		{
		pa -> posting_allowed_flag = posting_allowed_flag;
		pa -> group_status = group_status;

		pa -> article_ptr = article_ptr;
		pa -> new_articles = new_articles;
		pa -> subscribed_flag = subscribed_flag;
		pa -> filter_flags = filter_flags;
		}/* end mode use all data */
		
	}/* end while all lines in file */
}/* end function load groups_subscribed_groups */


int save_groups()
{
int i;
FILE *allgroupsfile;
char pathfilename[512];
struct group *pa, *pb;
extern char *strsave();

if(debug_flag) 
	{
	fprintf(stdout, "save_groups(): arg none\n");
	}

sprintf(pathfilename, "%s/.xagent/groups.dat", home_dir);
allgroupsfile = fopen(pathfilename, "w");
if(! allgroupsfile)
	{
	if(debug_flag)
		{
		fprintf(stdout, "could not open file %s for write\n", pathfilename);
		}
	return(0);
	} 

for(i = 0; i < GROUPS_HASH_SIZE; i++)/* for all structures at this position */
	{
	/* find end of chain of structures */
	pb = 0;
	for(pa = grouptab[i]; pa != 0; pa = pa -> nxtentr)
		{
		pb = pa;
		}
	/* pb now points to last structure */
	pa = pb;
	while(1)
		{
		if(! pa) break;
		/* 
		Note: writing pa -> subscribed_flag = 0, do not want subscribed
		info in groups.dat, else on doing a show all groups you will be
		subscribed again to anything you had unsubscribed from within
		subscribed groups :)
		*/

		if(debug_flag)
			{
			fprintf(stdout, "save_groups(): pa->name=%s\n", pa -> name);
			}

		fprintf(allgroupsfile, "%s %c  %d %ld %d  %ld %d\n",
		pa -> name, pa -> posting_allowed_flag,\
		0, (long) 0, pa -> group_status,\
		(long) 0, 0);

		pa = pa -> prventr;
		}/* end for all structures */
	}/* end for all entries in grouptab */

fclose(allgroupsfile);

save_subscribed_groups();

return(1);
}/* end function save_groups */


int save_subscribed_groups()
{
int i;
FILE *subscribedgroupsfile;
char pathfilename[512];
struct group *pa, *pb;
extern char *strsave();

if(debug_flag)
	{
	fprintf(stdout, "save_subscribed_groups(): arg none\n");
	}

sprintf(pathfilename, "%s/.xagent/subscribed_groups.dat", home_dir);
subscribedgroupsfile = fopen(pathfilename, "w");
if(! subscribedgroupsfile)
	{
	if(debug_flag)
		{
		fprintf(stdout, "could not open file %s for write\n", pathfilename);
		}
	return(0);
	} 

for(i = 0; i < GROUPS_HASH_SIZE; i++)/* for all structures at this position */
	{
	/* find end of chain of structures */
	pb = 0;
	for(pa = grouptab[i]; pa != 0; pa = pa -> nxtentr)
		{
		pb = pa;
		}
	/* pb now points to last structure */
	pa = pb;
	while(1)
		{
		if(! pa) break;
		if(pa -> subscribed_flag)
			{
			fprintf(subscribedgroupsfile, "%s %c  %d %ld %d  %ld %d\n",
			pa -> name, pa -> posting_allowed_flag,\
			pa -> subscribed_flag, pa -> article_ptr, pa -> group_status,\
			pa -> new_articles, pa -> filter_flags);
			}
		pa = pa -> prventr;
		}/* end for all structures */
	}/* end for all entries in grouptab */

fclose(subscribedgroupsfile);
return(1);
}/* end function save_subscribed_groups */


int subscribe_group(char *group, int subscribe)
{
struct group *pa, *lookup_group();
char *strsave();

if(debug_flag)
	{
	fprintf(stdout, "subscribe_group(): arg group=%s\n subscribe=%d",\
	group, subscribe);
	}

/* argument check */
if(! group) return(0);
if(strlen(group) == 0)
	{
	fl_show_alert("NO GROUP SPECIFIED", "COMMAND NOT EXECUTED", "", 0);
	return(0);
	}

pa = lookup_group(group);
if(!pa) /* not found */
	{
	if(subscribe == 1)
		{
		fl_show_alert(\
		"CANNOT SUBSCRIBE TO GROUP:", group, "NOT IN ACTIVE FILE", 0);
		if(debug_flag)
			{
			fprintf(stdout, "\nCannot subscribe to group %s\n", group);
			}
		return(0);
		}
	if(subscribe == 0)
		{
		fl_show_alert(\
		"CANNOT UNSUBSCRIBE GROUP:", group, "NO SUCH GROUP", 0);
		if(debug_flag)
			{
			fprintf(stdout, "\nCannot unsubscribe group %s\n", group);
			}
		return(0);
		}
	}

if(subscribe && (pa -> subscribed_flag) )
	{
	fl_show_alert(\
	"ALREADY SUBSCRIBED TO:", group, "", 0);
	return(1);
	}
	
if(pa -> group_status == CANCELLED_GROUP)
	{
	fl_show_alert(\
	"WARNING: This group is cancelled", "not in active file on this server",\
	"online operation will fail", 0);
	}
	
if(subscribe == 0)
	{
	if(strcmp(group, "xagent.help") == 0)
		{
		fl_show_alert("Group xagent.help cannot be unsubscribed", "", "", 0);
		return(1);
		}
	}

pa -> subscribed_flag = subscribe;
pa -> new_articles = 0;
pa -> article_ptr = 0;
pa -> filter_flags = 0;
return(1);
}/* end function subscribe_group */


int add_group(char *data)
{
struct group *install_group(), *pa;
char group[512];
char first_article[512];
char last_article[512];
char posting_allowed_flag[512];

if(debug_flag) fprintf(stdout, "add_group(): arg data=%s\n", data);

sscanf(data, "%s %s %s %s",\
group, last_article, first_article, posting_allowed_flag);

/*
NOTE: xagent_cb.c has marked all groups CANCELLED_GROUP
in response to LIST
*/

/* test if this is an existing group */
pa = lookup_group(group);
if(pa) /* existing group */
	{
	pa -> posting_allowed_flag = posting_allowed_flag[0];
	pa -> group_status = ACTIVE_GROUP;
	/* last article and first article are not used */
	return(1);
	}

/* new group */
pa = install_group(group);
if(! pa)
	{
	if(debug_flag)fprintf(stdout, "add_group: cannot install group\n");
	return(0);
	}
pa -> posting_allowed_flag = posting_allowed_flag[0];
pa -> group_status = NEW_GROUP;
pa -> new_articles = 0;
pa -> article_ptr = 0;
pa -> subscribed_flag = 0;
pa -> filter_flags = 0;
/* So any group that was there, but not in this LIST, is now CANCELLED_GROUP */
return(1);
}/* end function add_group */


int show_groups()
{
int i;
char temp[512];
struct group *pa;
extern char *strsave();
char formatstr[512];
int browser_topline;
int browser_last_selected_line;
char *ptra, *ptrb, *ptrc;
int match_failed_flag;
int last_string_flag;

if(debug_flag) fprintf(stdout, "show_groups(): arg none\n");

/* remember whch line was double clicked */
browser_last_selected_line = fl_get_browser(fdui -> group_list_browser);

/* remember vertical slider position */
browser_topline = fl_get_browser_topline(fdui -> group_list_browser);

/* clear browser */
fl_clear_browser(fdui -> group_list_browser);

for(i = 0; i < GROUPS_HASH_SIZE; i++)
	{
	for(pa = grouptab[i]; pa != 0; pa = pa -> nxtentr)	
		{
		/* test if diplay allowed */
		if(filters_enabled_flag)
			{
			if(pa -> filter_flags & FILTER_DO_NOT_SHOW_GROUP) continue;
			}

		if(groups_display_filter[0])/* some filter */
			{
			/* test if filter name present in group name */
			/* for all entries in filter */
			ptra = groups_display_filter;
			ptrb = strsave(groups_display_filter);/* create space */
			match_failed_flag = 0;
			last_string_flag = 0;
			while(1)
				{
				ptrc = ptrb;/* point to start availeble space */
				while(1)/* string in filter to ptrb */
					{
					*ptrc = *ptra;
					if(! *ptra) break;/* end filter */
					if(*ptra == ' ')/* end of string in filter */
						{
						*ptrc = 0;
						ptra++;/* scip the space */
						break;
						}
					ptra++;
					ptrc++;
					}/* end while all characters in string */
				/* test if string present in name */
				if(! strstr(pa -> name, ptrb) )
					{
					match_failed_flag = 1;
					break;
					}
				if(! *ptra) break;/* end filter */
				}/* end while all strings in filter */	
			free(ptrb);/* release space */
			if(match_failed_flag) continue;
			}/* end if filter */
		if(groups_show_selection == SHOW_ONLY_SUBSCRIBED_GROUPS)
			{	
			if(pa -> subscribed_flag)
				{
				strcpy(formatstr, "@C0");/* black */
				if(pa -> group_status == CANCELLED_GROUP)
					{
					strcpy(formatstr, "@C2");/* green */
					}		

				sprintf(temp, "%s %s %ld",\
				formatstr, pa -> name, pa -> new_articles);
		
				if(pa -> posting_allowed_flag == 'n')
					{
					strcat(temp, " NO POSTING ALLOWED");
					}
				fl_add_browser_line(fdui -> group_list_browser, temp);
				}
			}/* end subscribed groups */
		if(groups_show_selection == SHOW_ONLY_NEW_GROUPS)
			{
			if(pa -> group_status == NEW_GROUP)
				{
				strcpy(formatstr, "@C1");/* red */
			
				sprintf(temp, "%s %s %ld",\
				formatstr, pa -> name, pa -> new_articles);

				if(pa -> posting_allowed_flag == 'n')
					{
					strcat(temp, " NO POSTING ALLOWED");
					}
				fl_add_browser_line(fdui -> group_list_browser, temp);
				}
			}/* end new groups */
		if(groups_show_selection == SHOW_ALL_GROUPS)
			{
			strcpy(formatstr, "@C0");/* black */
			if(pa -> group_status == NEW_GROUP)
				{
				strcpy(formatstr, "@C1");/* red */
				}
			if(pa -> group_status == ACTIVE_GROUP)
				{
				strcpy(formatstr, "@C0");/* black */
				}
			if(pa -> group_status == CANCELLED_GROUP)
				{
				strcpy(formatstr, "@C2");/* green */
				}		
			if(pa -> subscribed_flag) strcpy(formatstr, "@C4");/* blue */
			
			sprintf(temp, "%s %s %ld",\
			formatstr, pa -> name, pa -> new_articles);
			
			if(pa -> posting_allowed_flag == 'n')
				{
				strcat(temp, " NO POSTING ALLOWED");
				}
			fl_add_browser_line(fdui -> group_list_browser, temp);
			}/* end all groups */
		}/* end for all structures at this location in hash table */
	}/* end for all entries in hash table */

/* restore double_clicked_line */
fl_select_browser_line(fdui -> group_list_browser,\
browser_last_selected_line);

/* readjust the vertical slider position to the old situation */
fl_set_browser_topline(fdui -> group_list_browser, browser_topline);

to_command_status_display("");

return(1);
}/* end function show_groups */


int add_new_headers_in_subscribed_groups_to_command_queue()
{
struct group *pa;
int i;

if(debug_flag)
	{
	fprintf(stdout,\
	"add_new_headers_in_subscribed_groups_to_command_queue(): arg none\n");
	}

for(i = 0; i < GROUPS_HASH_SIZE; i++)/* for all structures at this position */
	{
	for(pa = grouptab[i]; pa != 0; pa = pa -> nxtentr)	
		{
		if(pa -> subscribed_flag)
			{
			add_to_command_queue(pa -> name, 0, GET_NEW_HEADERS_IN_GROUP);
			}
		}/* end for all structures */
	}/* end for all entries in grouptab */
return(1);
}/* end function add_new_headers_in_subscribed_groups_to_command_queue */


int add_marked_article_bodies_in_subscribed_groups_to_command_queue()
{
struct group *pa;
int i;

if(debug_flag)
	{
	fprintf(stdout,\
	"add_marked_article_bodies_in_subscribed_groups_to_command_queue():\
	arg none\n");
	}

/* for all subscribed groups */
for(i = 0; i < GROUPS_HASH_SIZE; i++)/* for all structures at this position */
	{
	for(pa = grouptab[i]; pa != 0; pa = pa -> nxtentr)	
		{
		if(pa -> subscribed_flag)
			{
			add_marked_article_bodies_in_group_to_command_queue(pa -> name);			
			}		
		}/* end for all structures */
	}/* end for all entries in grouptab */
return(1);
}/* end  add_marked_article_bodies_in_subscribed_groups_to_command_queue */


int subscribed_to_group(char *group)
{
struct group *pa, *lookup_group();

if(debug_flag)
	{
	fprintf(stdout, "subscribed_to_group(): arg group=%s\n", group);
	}

pa = lookup_group(group);
if(! pa) return(0);

if(! pa -> subscribed_flag) return(0);
return(1);
}


int delete_all_not_locked_articles_in_subscribed_groups()
{
struct group *pa;
extern char *strsave();
int i;
char start_group[512];
long start_article;

if(debug_flag)
	{
	fprintf(stdout,\
	"delete_all_not_locked_articles_in_subscribed_groups() arg none\n");
	}

strcpy(start_group, selected_group);
start_article = selected_article;

for(i = 0; i < GROUPS_HASH_SIZE; i++)/* for all structures at this position */
	{
	for(pa = grouptab[i]; pa != 0; pa = pa -> nxtentr)	
		{
		if(! pa -> subscribed_flag) continue;

		delete_all_not_locked_articles(pa -> name);
		}/* end for all structures */
	}/* end for all entries in grouptab */

/*
we need to select a valid group and article, or next unread button will hang if pressed
immediatly after this function.
*/

select_first_group_and_article();

return(1);
}/* end function delete_all_not_locked_articles_in_subscribed_groups */


char *get_next_subscribed_group(char *group)
{
struct group *pa, *pstart, *lookup_group();
int hashval;
int i;

if(debug_flag)
	{
	fprintf(stdout, "get_next_subscribed_group(): arg group=%s\n", group);
	}

/* parameter check */
if(! group) return(0);

pa = lookup_group(group);
if(! pa) return(0); /* group does not exists */

hashval = hash(pa -> name);

/* set start search for subscribed groups */
pa = pa -> nxtentr;/* point to next group */
if(! pa) /* last one in the chain, or empty chain */
	{
	hashval++; /* point tot next postion in grouptab, or start if last */
	if(hashval >= GROUPS_HASH_SIZE) hashval = 0;
	pstart = grouptab[hashval];
/*fprintf(stdout, "!pa hashval=%d pstart=%ld\n", hashval, pstart);*/
	}
else
	{
/*fprintf(stdout, "pa hashval=%d pstart=%ld\n", hashval, pstart);*/
	pstart = pa;
	}
	
/* search from here to end of this chain */
for(pa = pstart; pa != 0; pa = pa -> nxtentr)
	{
	if(pa -> subscribed_flag)
		{
/*fprintf(stdout, "found in this chain name=%s\n", pa -> name);*/
		return(pa -> name);
		}
	}/* end for */
hashval ++;
if(hashval >= GROUPS_HASH_SIZE) hashval = 0;

/*fprintf(stdout, "moving to i = hashval=%d\n", hashval);*/

/* search form start next chain to end of chains */	
for(i = hashval; i < GROUPS_HASH_SIZE; i++)
	{
	for(pa = grouptab[i]; pa != 0; pa = pa -> nxtentr)
		{
		if(pa -> subscribed_flag)
			{
/*fprintf(stdout, "found in second half name=%s\n", pa -> name);*/
			return(pa -> name);
			}
		}/* end for */
	}/* end for all entries in grouptab */
return(0);/* not found */
}/* end function get_next_subscribed_group */


int completely_delete_group(char *group)
{
struct group *pa, *lookup_group();
char temp[512];
FILE *exec_filefd;

if(debug_flag)
	{
	fprintf(stdout, "completely_delete_group(): arg group=%s\n", group);
	}

/* argument check */
if(! group) return(0);

if(strcmp(group, "xagent.help") == 0)
	{
	fl_show_alert("HELP CANNOT BE DELETED", "NOTHING DELETED", "", 0);
	return(0);
	}

if(strlen(group) == 0)
	{
	fl_show_alert("NO GROUP SPECIFIED", "NOTHING DELETED", "", 0);
	return(0);
	}

/* test if group exists */
pa = lookup_group(group);
if(! pa)
	{
	fl_show_alert("NO SUCH GROUP:", group, "NOTHING DELETED", 0);
	return(0);
	}

/* unsubscribe */
pa -> subscribed_flag = 0;

/* I know about rm -rf, but just being carefull */

/* delete all files in group directory */
sprintf(temp, "rm %s/.xagent/%s/*", home_dir, group);
/*
yes I deleted my setup file and all posts,
and the groups.dat (loosing all subscribed groups)
*/
if(strstr(temp, "//") != 0) return(0);
exec_filefd = popen(temp, "r");
pclose(exec_filefd);
				
/* remove group directory */
sprintf(temp, "rmdir %s/.xagent/%s", home_dir, group);
if(strstr(temp, "//") != 0) return(0);
exec_filefd = popen(temp, "r");
pclose(exec_filefd);

return(1);
}/* end function completely_delete_group */


int set_all_filter_flags_in_articles()
{
struct group *pa;
int i;

if(debug_flag)
	{
	fprintf(stdout,\
	"set_all_filter_flags_in_articless() arg none\n");
	}

for(i = 0; i < GROUPS_HASH_SIZE; i++)/* for all structures at this position */
	{
	for(pa = grouptab[i]; pa != 0; pa = pa -> nxtentr)	
		{
		if(! pa -> subscribed_flag) continue;

		set_all_filter_flags_in_group(pa -> name);
		}/* end for all structures */
	}/* end for all entries in grouptab */
return(1);
}/* end function set_all_filter_flags_in_articles */


int apply_filters_to_group(char *group)
{
struct group *pa, *lookup_group();

if(debug_flag)
	{
	fprintf(stdout,\
	"apply_filters_to_group(): arg group=%s\n", group);
	}

/* argument check */
if(! group) return(0);

pa = lookup_group(group);
if(! pa) return(0);

get_filter_flags(\
group, "dummy", 0, "dummy", 0, SEARCH_GROUPS, &(pa -> filter_flags) );

return(1);
}/* end function apply_filters_to_group_header */


int set_all_filter_flags_in_groups()
{
struct group *pa;
int filter_flags;
int i;

if(debug_flag)
	{
	fprintf(stdout, "set_all_filter_flags_in_groups(): arg none\n");
	}
	
/* first test if we really HAVE to load all those groups..... */
for(i = 0; i < GROUPS_HASH_SIZE; i++)/* for all structures at this position */
	{
	for(pa = grouptab[i]; pa != 0; pa = pa -> nxtentr)
		{
		/* get the filter result */
		filter_flags = 0;/* the default */
		get_filter_flags(\
		pa -> name, "dummy" , 0, "dummy", 0, SEARCH_GROUPS, &filter_flags);
		if(filter_flags) break;
		}/* end for all structures */
	}/* end for all entries in grouptab */

if(! filter_flags) return(1);

load_groups();

for(i = 0; i < GROUPS_HASH_SIZE; i++)/* for all structures at this position */
	{
	for(pa = grouptab[i]; pa != 0; pa = pa -> nxtentr)
		{
		/* get the filter result */
		pa -> filter_flags = 0;/* the default */

		get_filter_flags(\
		pa -> name, "dummy" , 0, "dummy", 0, SEARCH_GROUPS,\
		&(pa -> filter_flags) );
		}/* end for all structures */
	}/* end for all entries in grouptab */

save_groups();
return(1);
}/* end function set_all_filter_flags_in_groups */


int select_first_group_and_article()
{
if(debug_flag)
	{
	fprintf(stdout, "select_first_group_and_article(): arg none\n");
	}

if(selected_group) free(selected_group);
selected_group = strsave("xagent.help");
selected_article = 1;

return(1);
}/* end function select_first_subscribed_group_and_article */


