/* 
xagent offline NNTP news reader
xagent is registered Copyright (C) 1997 <Jan Mourer>
email: pante@pi.net
snail mail: PO BOX 61  SINT ANNA  9076ZP HOLLAND

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "xagent.h"

extern FD_xagent *fd_xagent;
extern FD_xagent *fdui;

struct posting
	{
	char *name;/* this is the posting id long decimal */
	char *subject;
	char *from;
	char *to;
	char *content_type;
	char *references;
	char *summary;
	char *attachment;
	char *date;
	int browser_line;
	int lock_flag;
	int encoding;
	long lines;
	int send_status;
	int type;
	int custom_headers_flag;
	int filter_flags;
	struct posting *nxtentr;
	};
struct posting *postingtab[1];


struct posting *lookup_posting(char *name)
{
struct posting *pa;

if(debug_flag) fprintf(stdout, "lookup_posting(): arg =%s\n", name);

/*pa points to next entry*/
for(pa = postingtab[0]; pa != 0; pa = pa -> nxtentr)
	{
	if(strcmp(pa -> name, name) == 0) return(pa);
	}
return(0);/*not found*/
}/* end function lookup_posting */


struct posting *install_posting_at_end_of_list(char *name)
{
struct posting *pa, *pb, *pprev, *lookup_posting();
char *strsave();
char *malloc();

if(debug_flag)
	{
	fprintf(stdout, "install_posting_at_end_of_list arg=%s\n", name);
	}

pprev = 0;/* only for clean compilation with -Wall */

/* find end of the chain of structures */
pa = postingtab[0];/* start chain */
while(1)
	{
	if(! pa)/* found end of chain */
		{
		/* enter new structure at the end of the chain */
		pb = (struct posting *) malloc( sizeof(*pb) );
		if(! pb) return(0);
		pb -> name = strsave(name);
		if(! pb -> name) return(0);
		pb -> nxtentr = 0;/* new points to zero (is end) */

		if(! postingtab[0] ) postingtab[0] = pb;/* first one */
		else pprev -> nxtentr = pb;/* previous points to new */
		pb -> summary = strsave("");	/* NO NO MORE NULL or UNDEFINED
											pointers!! */
		pb -> custom_headers_flag = NO_CUSTOM_HEADERS;
		pb -> filter_flags = FILTER_OFF;
		return(pb);/* pointer to new structure */
		}
	pprev = pa;/* remember previous structure */
	pa = pa -> nxtentr;
	}/* end while all structures */
}/* end function install_posting_at_end_of_list */


int delete_posting(char *name)/* delete entry from table */
{
struct posting *pa, *pprev;

if(debug_flag)
	{
	fprintf(stdout, "delete_posting(): arg name=%s\n", name);
	}

pprev = 0;
for(pa = postingtab[0]; pa != 0; pa = pa -> nxtentr)
	{
	if(strcmp(name, pa -> name) == 0)/* found it */
		{
		if(pprev == 0)/* first entry */
			{
			postingtab[0] = pa -> nxtentr;
			}
		else/* not first one */
			{
			pprev -> nxtentr = pa -> nxtentr;
			}
		free(pa -> name);/* free name */
		free(pa -> subject);
		free(pa -> from);
		free(pa -> to);		
		free(pa -> content_type);
		free(pa -> references);
		free(pa -> summary);
		free(pa -> attachment);
		free(pa -> date);
		free(pa);/* free structure */
		return(1);
		}/* end found */
	pprev = pa;/* save this pointer */
	}/* end for all structures */
return(0);/* not found */
}/* end function delete_posting */


int delete_all_postings()/* delete all entries from table */
{
struct posting *pa;

if(debug_flag)
	{
	fprintf(stdout, "delete_all_postings(): arg none\n");
	show_structure();
	}	

while(1)
	{	
	pa = postingtab[0];
	if(! pa) break;
	postingtab[0] = pa -> nxtentr;/* postingtab entry points to next one,
															this could be 0 */
	free(pa -> name);/* free name */
	free(pa -> subject);
	free(pa -> from);
	free(pa -> to);
	free(pa -> content_type);
	free(pa -> references);
	free(pa -> summary);
	free(pa -> attachment);
	free(pa -> date);
	free(pa);/* free structure */
	}/* end while all structures */

if(debug_flag)fprintf(stdout, "returning from delete_all_postings\n");

return(1);/* not found */
}/* end function delete_all_postings */


int show_structure()
{
struct posting *pa;

if(debug_flag)fprintf(stdout, "show_structure(): arg none\n");

pa = postingtab[0];
while(1)
	{	
	if(! pa) break;
	fprintf(stdout, "pa=%ld pa -> nxtentr=%ld\n",\
	(long) pa, (long) pa -> nxtentr);
	fprintf(stdout, "name=%s\n", pa -> name);
	fprintf(stdout, "subject=%s\n", pa -> subject);
	fprintf(stdout, "from=%s\n", pa -> from);
	fprintf(stdout, "to=%s\n", pa -> to);
	fprintf(stdout, "content_type=%s\n", pa -> content_type);
	fprintf(stdout, "references=%s\n", pa -> references);
	fprintf(stdout, "summary=%s\n", pa -> summary);
	fprintf(stdout, "browser_line=%d\n", pa -> browser_line);
	fprintf(stdout, "lock_flag=%d\n", pa -> lock_flag);
	fprintf(stdout, "encoding=%d\n", pa -> encoding);
	fprintf(stdout, "lines=%ld\n", pa -> lines);
	fprintf(stdout, "send_status-%d\n", pa -> send_status);
	fprintf(stdout, "type=%d\n", pa -> type);
	fprintf(stdout, "custom_headers_flag=%d\n", pa -> custom_headers_flag);
	fprintf(stdout, "filter_flags=%d\n", pa -> filter_flags);
	pa = pa -> nxtentr;
	}
return(1);
}/* end function show_structure */


int load_postings()
{
int a;
FILE *postings_dat_file;
char temp[READSIZE];
char name[512];
int lock_flag;
int encoding;
long lines;
int browser_line;
int send_status;
int type;
int custom_headers_flag;
int filter_flags;
extern long atol();
struct posting *pa;
extern char *strsave();
char *subject;
char *from;
char *to;
char *references;
char *content_type;
char *summary;
char *attachment;
char *date;

if(debug_flag)
	{
	fprintf(stdout, "load_postings(): arg none\n");
	}

/*delete_all_postings();*/

sprintf(temp, "%s/.xagent/postings.dat", home_dir);
postings_dat_file = fopen(temp, "r");
if(! postings_dat_file)
	{
	fl_show_alert("could not load file", temp, "", 0);
	return(0);
	} 

while(1)
	{
	/* 
	This should make it possible to read old formats, that do not have a
	filter_flags.
	*/
	filter_flags = 0;
	
	a = readline(postings_dat_file, temp);
	if(a == EOF) return(0);/* file format error */
	sscanf(temp, "%s %d %d %ld %d %d %d %d %d",\
	name, &lock_flag, &encoding, &lines, &browser_line, &send_status,\
	&type, &custom_headers_flag, &filter_flags);
	
	/* Subject: */
	a = readline(postings_dat_file, temp);
	if(a == EOF) return(0);/* file format error */
	subject = strsave(temp);
	if(! subject) return(0);

	/* From: */
	a = readline(postings_dat_file, temp);
	if(a == EOF) return(0);/* file format error */
	from = strsave(temp);
	if(! from) return(0);
	 
	/* Newsgroup: */
	a = readline(postings_dat_file, temp);
	if(a == EOF) return(0);/* file format error */
	to = strsave(temp);
	if(! to) return(0);
	 
	/* Content-Type: */
	a = readline(postings_dat_file, temp);
	if(a == EOF) return(0);/* file format error */
	content_type = strsave(temp);
	if(! content_type) return(0);

	/* References: */
	a = readline(postings_dat_file, temp);
	if(a == EOF) return(0);/* file format error */
	references = strsave(temp);
	if(! references) return(0);

	/* Summary: */
	a = readline(postings_dat_file, temp);
	if(a == EOF) return(0);/* file format error */
	summary = strsave(temp);
	if(! summary) return(0);

	/* Attachment: */
	a = readline(postings_dat_file, temp);
	if(a == EOF) return(0);/* file format error */
	attachment = strsave(temp);
	if(! attachment) return(0);

	/* Date: */
	a = readline(postings_dat_file, temp);
	date = strsave(temp);
	if(! date) return(0);

	pa = install_posting_at_end_of_list(name);
	if(! pa)
		{
		if(debug_flag)
			{
			fprintf(stdout,\
			"load_postings: cannot install posting %s\n", temp);
			}
		return(0);
		}
	pa -> subject = subject;
	pa -> from = from;
	pa -> to = to;
	pa -> content_type = content_type;
	pa -> references = references;
	pa -> summary = summary;
	pa -> attachment = attachment;
	pa -> date = date;
	
	pa -> lock_flag = lock_flag;
	pa -> encoding = encoding;
	pa -> lines = lines;
	pa -> browser_line = browser_line;
	pa -> send_status = send_status;
	pa -> type = type;
	pa -> custom_headers_flag = custom_headers_flag;
	pa -> filter_flags = filter_flags;
	if(a == EOF)
		{
		return(1);
		}
	}
/* if here, no EOF on last line read, file format error */
sprintf(temp, "%s/.xagent/postings.dat", home_dir);
fl_show_alert("CANNOT ACCESS POSTINGS", "File format error", temp, 0);
return(0);
}/* end function load postings */


int save_postings()
{
FILE *postings_dat_file;
char temp[512];
struct posting *pa;
extern char *strsave();

if(debug_flag) fprintf(stdout, "save_postings(): arg none\n");

sprintf(temp, "%s/.xagent/postings.dat", home_dir);
postings_dat_file = fopen(temp, "w");
if(! postings_dat_file)
	{
	fl_show_alert( "could not open file", temp, "", 0);
	return(0);
	} 

for(pa = postingtab[0]; pa != 0; pa = pa -> nxtentr)	
	{
	fprintf(postings_dat_file, "%s %d %d %ld %d %d %d %d %d\n",\
	pa -> name, pa -> lock_flag, pa -> encoding, pa -> lines,\
	pa -> browser_line, pa -> send_status, pa -> type,\
	pa -> custom_headers_flag, pa -> filter_flags);
	fprintf(postings_dat_file, "%s\n", pa -> subject);
	fprintf(postings_dat_file, "%s\n", pa -> from);
	fprintf(postings_dat_file, "%s\n", pa -> to);
	fprintf(postings_dat_file, "%s\n", pa -> content_type);
	fprintf(postings_dat_file, "%s\n", pa -> references);
	fprintf(postings_dat_file, "%s\n", pa -> summary);
	fprintf(postings_dat_file, "%s\n", pa -> attachment);
	fprintf(postings_dat_file, "%s\n", pa -> date);
	}/* end for all structures in the chain starting from postingtab[0] */

fclose(postings_dat_file);

return(1);/* ok */
}/* end function save_postings */


int show_postings()
{
int i;
char temp[READSIZE];
struct posting *pa;
char formatstr[20];
char infostr[40];
int browser_line;
int browser_topline;
int browser_last_selected_line;
char *ptr;
char *cptr;

if(debug_flag)
	{
	fprintf(stdout, "show_postings(): arg none\n");
	}

/* remember the last line selected */
browser_last_selected_line = fl_get_browser(fdui -> posting_list_browser);

/* remember vertical slider position */
browser_topline = fl_get_browser_topline(fdui -> posting_list_browser);

/* first clear display in order not to confuse reader with old data */
fl_clear_browser(fdui -> posting_list_browser);

/* if load failed (no postings.dat file yet), continue anyway */	

browser_line = 1;

for(pa = postingtab[0]; pa != 0; pa = pa -> nxtentr)	
	{
	/* test if diplay allowed */
	if(filters_enabled_flag)
		{
		if(pa -> filter_flags & FILTER_DO_NOT_SHOW_HEADER) continue;
		}

	strcpy(formatstr, "@C2");/* green, error */
	if(pa -> send_status == TO_BE_SEND) strcpy(formatstr, "@C1");/* red */
	if(pa -> send_status == SEND_NOW) strcpy(formatstr, "@C5");/* magenta */
	if(pa -> send_status == SEND_LATER) strcpy(formatstr, "@C4");/* blue */
	if(pa -> send_status == SEND_OK) strcpy(formatstr, "@C0");/* black */
	if(pa -> send_status == SEND_FAILED) strcpy(formatstr, "@C2");/* green */
	if(pa -> lock_flag) strcpy(infostr, "*LOCK*");
	else strcpy(infostr, "            ");
	if(pa -> send_status == SEND_OK) strcpy(infostr, "*SENT*");

	if(!strstr(pa -> subject, "\001" ) )
		{
		sprintf(temp, "%s %s %s %ld %s (%s)",\
		formatstr, infostr, pa -> subject, pa -> lines,\
		pa -> date, pa -> name);

		fl_add_browser_line(fdui -> posting_list_browser, temp);
		pa -> browser_line = browser_line;
		browser_line++;
		}
	else /* a formatted subject, read from header file */
		{
		pa -> browser_line = browser_line;
		ptr = (char *) get_formatted_header_data(\
		"", atol(pa -> name), "Subject:");
		if(ptr)
			{
 			strcat(infostr, " ");
 			cptr = ptr;
			while(1)/* for all lines in the subject field */
				{
				/* ident some more for display clarity */
				sprintf(temp, "%s %s", formatstr, infostr);
				i = strlen(temp);
				while(1)/* extract a line */
					{
					if(*cptr == '\n') break;
					temp[i] = *cptr;
					if(! *cptr) break;
					i++;
					cptr++;
					}/* end while all characters in a line */
				temp[i] = 0;
				fl_add_browser_line(fdui -> posting_list_browser, temp);
				browser_line ++;

				if(! *cptr) break;
				cptr++;
				}/* end while for all lines in subject */
			free(ptr);
			}/* end if subject read OK */

		sprintf(temp, "%s %s %s %ld %s (%s)",\
		formatstr, infostr, "", pa -> lines,\
		pa -> date, pa  -> name);
		fl_add_browser_line(fdui -> posting_list_browser, temp);
		browser_line++;
		}/* end else formatted subject */

	/* if a summary field is present, show it*/	
	if(strlen(pa -> summary) != 0)
		{
		ptr = (char *) get_formatted_header_data(\
		"", atol(pa -> name), "Summary:");
		if(ptr)
			{
			strcat(infostr, "  ");

			sprintf(temp, "%s %sSummary:", formatstr, infostr);
			fl_add_browser_line(fdui -> posting_list_browser, temp);
			browser_line++;

 			cptr = ptr;
			while(1)/* for all lines in the summary field */
				{
				sprintf(temp, "%s %s", formatstr, infostr);
				i = strlen(temp);
				while(1)/* extract a line */
					{
					if(*cptr == '\n') break;
					temp[i] = *cptr;
					if(! *cptr) break;
					i++;
					cptr++;
					}/* end while all characters in a line */
				temp[i] = 0;
				fl_add_browser_line(fdui -> posting_list_browser, temp);
				browser_line ++;

				if(! *cptr) break;
				cptr++;
				}/* end while for all lines in summary */
			free(ptr);
			}/* end if summary read OK */
		}/* end if strlen pa -> summary != 0 */

	}/* end for all structures */

/* select the line that was last double_clicked again */
fl_select_browser_line(fdui -> posting_list_browser,\
browser_last_selected_line);

/* re adjust browser for same position of top line (vertical slider)*/	
fl_set_browser_topline(fdui -> posting_list_browser, browser_topline);

/* structure was modified */
save_postings();

return(1);
}/* end function show_postings */


int show_posting_body(long posting_id)
{
int c;
struct posting *pa, *lookup_posting();
char temp[READSIZE];	
FILE *load_filefd;
extern char *strsave();
struct stat *statptr;
char *space, *spaceptr;
char *ptr;
char *expanded_space;

if(debug_flag)
	{
	fprintf(stdout, "show_posting_body(): arg posting_id=%ld\n",\
	posting_id);
	}

/* parameter check */
if(posting_id < 0) return(0);

sprintf(temp, "%ld", posting_id);
pa = lookup_posting(temp);
if(! pa) return(0);

/* first clear display in order not to confuse reader with old data */
fl_set_input(fdui -> posting_body_editor, "");

fl_set_tabstop("aaaaaaaa");

fl_set_input(fdui -> posting_to_input_field, pa -> to);

if(!strstr(pa -> subject, "\001") )/* single line */
	{
	ptr = pa -> subject;
	}
else /* multi line subject, get from header */
	{
	ptr = get_formatted_header_data("", posting_id, "Subject:");
	if(! ptr) return(0);
	}
/*
xforms does not seem to listen the first time, the data is there,
if you read it back, but no display.
*/
fl_set_input(fdui -> posting_subject_input_field, ptr);
fl_set_input(fdui -> posting_subject_input_field, ptr);

fl_set_input(fdui -> posting_attachment_input_field, pa -> attachment);

/* display the summary in the summary_editor_input_field */
/* test if summary present */
if(strlen(pa -> summary) != 0)
	{
	/* read from header */
	ptr = (char *) get_formatted_header_data("", posting_id, "Summary:");
	if(! ptr) return(0);

	fl_set_input(fdui -> summary_editor_input_field, ptr);
	fl_set_object_color(fdui -> posting_summary_button,\
	FL_DARKORANGE,  FL_DARKORANGE);
	}
else /* no summary */
	{
	fl_set_input(fdui -> summary_editor_input_field, "");
	fl_set_object_color(fdui -> posting_summary_button, FL_COL1, FL_COL1);
	}

custom_headers_state = pa -> custom_headers_flag;

if( (pa -> custom_headers_flag & GLOBAL_HEADERS_ENABLED) ||\
(pa -> custom_headers_flag & LOCAL_HEADERS_ENABLED) )
	{
	fl_set_object_color(fdui -> posting_custom_headers_button,\
	FL_DARKORANGE, FL_DARKORANGE);
	}
else
	{	
	fl_set_object_color(fdui -> posting_custom_headers_button, FL_COL1, FL_COL1);
	}

if(pa -> custom_headers_flag & GLOBAL_HEADERS_ENABLED)
	{
	fl_set_button(fdui -> global_custom_headers_enable_button, 1);
	/* 
	Note: display of global headers by posting_custom_headers_button_cb() in
	xagent_cb.c
	*/
	}
else
	{
	fl_set_button(fdui -> global_custom_headers_enable_button, 0);
	}	

if(! pa -> custom_headers_flag)
	{
	fl_set_input(fdui -> local_custom_headers_editor_input_field, "");
	}

if(pa -> custom_headers_flag & LOCAL_HEADERS_ENABLED)
	{	
	fl_set_button(fdui -> local_custom_headers_enable_button, 1);

	/* display the custom headers in the custom headers editor */
	sprintf(temp, "%s/.xagent/custom.%ld", home_dir, posting_id);
	load_filefd = fopen(temp, "r");
	if(! load_filefd)
		{
		fl_show_alert("could not load file", temp, "", 0);
		return(0);
		}

	statptr = (struct stat*) malloc(sizeof(struct stat) );
	if(! statptr) return(0);
	
	fstat(fileno(load_filefd), statptr);

	space = malloc(statptr -> st_size + 1);
	free(statptr);
	if(! space)
		{
		if(debug_flag)
			{
			fprintf(stdout,\
"show_posting_body(): malloc could not allocate space for custom headers\n");
			}
		return(0);
		}

	spaceptr = space;
	while(1)
		{
		while(1)/* error re read */
			{
			c = getc(load_filefd);
			if(! ferror(load_filefd) ) break;
			perror("show_posting_body(): get custom header read failed ");
			}/* end while error re read */	

		if(feof(load_filefd) )
			{
			fclose(load_filefd);
			break;
			}
		*spaceptr = c;
		spaceptr++;
		}/* end while all lines from article body */
	*spaceptr = 0;/* string termination */

	fl_set_input(fdui -> local_custom_headers_editor_input_field, space);
	free(space);
	}
else
	{
/*	fl_set_input(fdui -> local_custom_headers_editor_input_field, "");*/
	fl_set_button(fdui -> local_custom_headers_enable_button, 0);
	}

/* posting present, load it */
sprintf(temp, "%s/.xagent/body.%ld", home_dir, posting_id);
load_filefd = fopen(temp, "r");
if(! load_filefd)
	{
	fl_show_alert("could not load file", temp, "", 0);
	return(0);
	}

statptr = (struct stat*) malloc(sizeof(struct stat) );
if(!statptr) return(0);

fstat(fileno(load_filefd), statptr);

space = malloc(statptr -> st_size + 1);
free(statptr);
if(! space)
	{
	if(debug_flag)
		{
		fprintf(stdout,\
		"show_posting_body(): malloc could not allocate space for body\n");
		}
	return(0);
	}

/* speed up things */
fl_freeze_form(fdui -> xagent);

spaceptr = space;
while(1)
	{
	while(1)/* error re read */
		{
		c = getc(load_filefd);
		if(! ferror(load_filefd) ) break;
		perror("show_posting_body(): body read failed ");
		}/* end while error re read */	

	if(feof(load_filefd) )
		{
		fclose(load_filefd);
		break;
		}
	*spaceptr = c;
	spaceptr++;
	}/* end while all lines from posting body */
*spaceptr = 0;/* string termination */

expanded_space = (char *) expand_tabs(space, tab_size);
free(space);

fl_set_input(fdui -> posting_body_editor, expanded_space);
free(expanded_space);

fl_unfreeze_form(fdui -> xagent);

return(1);
}/* end function show_posting_body */


int line_to_posting_id(int line, long *posting_id)
{
struct posting *pa;
extern long atol();

if(debug_flag)fprintf(stdout, "line_to_posting_id(): arg line=%d\n", line);

for(pa = postingtab[0]; pa != 0; pa = pa -> nxtentr)	
	{
	if(pa -> browser_line == line)
		{
		*posting_id = atol(pa -> name);
		return(1);
		}
	}
/* no save_postings, nothing was changed */
return(0);
}/* end function line_to_posting_id */


int fill_in_input_fields(char *group, long article_id, int type)
{
extern char *strsave();
char *get_header_data();
char *ptr;
char *ptr2;

if(debug_flag)
	{
	fprintf(stdout,\
	"fill_in_input_fields(): arg group=%s article_id=%ld type=%d\n",\
	group, article_id, type);
	}

/* argument check */
if(! group) return(0);
if(! type) return(0);
if( (type != NEW_ARTICLE) && (article_id < 0) ) return(0);

/* set for no attachment */
fl_set_input(fdui -> posting_attachment_input_field, "");

/* clear the editor if a new article */
if(type == NEW_ARTICLE)
	{
	/* set the "To" input field label  */
	fl_set_object_label(fdui -> posting_to_input_field, "Groups");

	/* set the "To" input field to the selected group */
	fl_set_input(fdui -> posting_to_input_field, selected_group);

	/* clear the subject */
	fl_set_input(fdui -> posting_subject_input_field, "");

	/* clear the summary */
	fl_set_input(fdui -> summary_editor_input_field, "");

	/* clear the editor */
	fl_set_input(fdui -> posting_body_editor, "");
	}
if(type == FOLLOW_UP_ARTICLE)
	{
	/* set the "To" input field label */
	fl_set_object_label(fdui -> posting_to_input_field, "Groups");

	/* set the "To" input field the same as the original article */
	ptr = (char *) get_header_data(group, article_id, "Newsgroups:");
	if(! ptr) return(0);
	fl_set_input(fdui -> posting_to_input_field, ptr);
	free(ptr);
	/* 
	set the "Subject" input field the same as the original article,
	prepend "Re: " if not already there.
	*/
	ptr = (char *) get_formatted_header_data(group, article_id, "Subject:");
	if(! ptr) return(0);
	if(strstr(ptr, "Re:") != ptr)
		{
		ptr2 = malloc(strlen(ptr) + strlen("Re: ") + 1);
		strcpy(ptr2, "Re: ");
		strcat(ptr2, ptr);
		fl_set_input(fdui -> posting_subject_input_field, ptr2);
		free(ptr2);
		}
	else /* already there */
		{
		/* 
		for some reason xforms does not always print subject input field,
		possibly illegal characters? or maybe xform bug?
		*/
		fl_set_input(fdui -> posting_subject_input_field, ptr);
		}	
	free(ptr);
	
	/* get the summary from the original article */
	ptr = (char *) get_formatted_header_data(group, article_id, "Summary:");
	if(ptr)
		{
		fl_set_input(fdui -> summary_editor_input_field, ptr);
		free(ptr);
		fl_set_object_color(fdui -> posting_summary_button,\
		FL_DARKORANGE,  FL_DARKORANGE);
		}
	else
		{
		fl_set_input(fdui -> summary_editor_input_field, "");
		fl_set_object_color(fdui -> posting_summary_button,\
		FL_COL1, FL_COL1);
		}

	/* insert the original text, idented with '>' */
	load_and_ident_article(group, article_id);
	}
if( type == REPLY_VIA_EMAIL)
	{
	/* set the to input field label */
	fl_set_object_label(fdui -> posting_to_input_field, "Email");

	/* 
	set the to input field to the email of the sender.
	First test for a Repy-To:, if not present use From:
	*/
	ptr = (char *) get_header_data(group, article_id, "Reply-To:");
	if(! ptr)/* not present */
		{
		ptr = (char *) get_header_data(group, article_id, "From:");
		if(! ptr) return(0);
		}
	fl_set_input(fdui -> posting_to_input_field, ptr);
	free(ptr);/* from the first OR from the second call to get_header_data */
	
	/* get the subject from the original article */
	ptr = (char *) get_formatted_header_data(group, article_id, "Subject:");
	if(! ptr) return(0);
	fl_set_input(fdui -> posting_subject_input_field, ptr);
	free(ptr);
	
	/* get the summary from the original article */
	ptr = (char *) get_formatted_header_data(group, article_id, "Summary:");
	if(ptr)
		{
		fl_set_input(fdui -> summary_editor_input_field, ptr);
		/* remember this text, in case modified and cancel pressed */
		free(ptr);
		fl_set_object_color(fdui -> posting_summary_button,\
		FL_DARKORANGE,  FL_DARKORANGE);
		}
	else
		{
		fl_set_input(fdui -> summary_editor_input_field, "");
		fl_set_object_color(fdui -> posting_summary_button,\
		FL_COL1, FL_COL1);
		}

	/* include the original text idented with '>' */
	load_and_ident_article(group, article_id);	
	}

custom_headers_state = 0;/* not know at this time yet */
fl_set_object_color(fdui -> posting_custom_headers_button, FL_COL1, FL_COL1);
fl_set_button(fdui -> global_custom_headers_enable_button, 0);
fl_set_button(fdui -> local_custom_headers_enable_button, 0);
fl_set_input(fdui -> local_custom_headers_editor_input_field, "");

posting_modified_flag = 0;
/* 
NOTE: at this point in time, the user may modify:
posting_to_input_field, posting_subject_input_field,
posting_attachment_input_field.
press summary and modify summary_editor_input_field,
press headers, then modify anything on the header form.
Then press send now -, send later - , or cancel button.
*/ 	
/* 
Note: all above data may be modified by show_posting_body().
*/
return(1);
}/* end function fill_in_input_fields */


int save_posting(char *group, long article_id, int mode)
{
char temp[512];
char name[50];
struct posting *pa, *pold, *lookup_posting();
char from[512];
extern char* strsave();
char *get_universal_time_as_string();
char *get_header_data();
char *ptr, *ptr2, *ptr3;
long lines;
int type;
FILE *exec_filefd;
int new_posting;
FILE *custom_headers_filefd;
char pathfilename[512];

if(debug_flag)
	{
	fprintf(stdout,\
	"save_posting(): arg group=%s article_id=%ld mode=%d\n\
	using posting_source=%d posting_menu_selection=%d\n",\
	group, article_id, mode, posting_source, posting_menu_selection);
	}

/* there are 3 possible sources, posting list, posting menu, online menu.
*/

/* argument check */
if(posting_source == POSTING_MENU)
	{
	if(! group) return(0);
	if( (posting_menu_selection == FOLLOW_UP_ARTICLE) ||\
	(posting_menu_selection == REPLY_VIA_EMAIL) )
		{
		if(article_id < 0) return(0);
		}
	}

/*
Because time changes, and we need the correct time in a send now,
we always create a new structure entry if send_now.
The only time we do not create a new structure is if
not modified and not send now.
*/
if( (! posting_modified_flag) && (mode == SEND_LATER)) return(1);
	
new_posting = 0;
if(posting_source == POSTING_MENU)
	{
	if(posting_menu_selection == NEW_ARTICLE) new_posting = 1;
	if(posting_menu_selection == FOLLOW_UP_ARTICLE) new_posting = 1;
	if(posting_menu_selection == REPLY_VIA_EMAIL) new_posting = 1;
	type = posting_menu_selection;
	} 
if(posting_source == ONLINE_MENU)
	{
	/* use old type */
	}
if(posting_source == POSTING_LIST)
	{
	/* use old type */
	}

pold = 0;
if(! new_posting)
	{
	/* send something we already have */
	/* get the old structure address */
	sprintf(name, "%ld", body_posting);
	pold = lookup_posting(name);
	if(! pold) return(0);
	type = pold -> type;
	}
else type = posting_menu_selection;
			
/* get a new unused structure name */
if(! get_new_posting_id(&body_posting) ) return(0);
	
/* create a new structure entry */
sprintf(name, "%ld", body_posting);
pa = install_posting_at_end_of_list(name);
if(! pa ) return(0);
/* pa now points to new structure data */

/* copy data to new structure */
/* set from */
sprintf(from, "%s (%s)", user_email_address, user_name);
pa -> from = strsave(from);

if(debug_flag) fprintf(stdout, "FROM=%s\n", pa -> from);

/* get subject from the subject input field (may be modified by user ) */
ptr = (char *) fl_get_input(fdui -> posting_subject_input_field);
if(! ptr)
	{
	fl_show_alert(\
	"save_posting():", "posting_subject_input_field returns NULL", "", 0);
	return(0);
	} 

/*free(pa -> subject);*/
/* if multiline, signal to use header */
if(!strstr(ptr, "\n") ) pa -> subject = strsave(ptr);
else pa -> subject = strsave("use header\001");

/* get the destination from the to input field (maybe modified by user) */
ptr = (char *) fl_get_input(fdui -> posting_to_input_field); 
if(! ptr)
	{
	fl_show_alert(\
	"save_posting():", "posting_to_input_field returns NULL", "", 0);
	return(0);
	}
pa -> to = strsave(ptr);

/* get the universal date and time from the system */
ptr = get_universal_time_as_string();
if(! ptr) return(0);
pa -> date = strsave(ptr); 
free(ptr);

/* 
get the attachment from the attachment input field (set by user,
either directly or via pushing attachment button and selecting a file
with the file browser)
*/
ptr = (char *) fl_get_input(fdui -> posting_attachment_input_field);
if(! ptr)
	{
	fl_show_alert(\
	"save_posting():", "posting_attachment_input_field returns NULL", "", 0);
	return(0);
	}
pa -> attachment = strsave(ptr);	

/* encode the attachment with mpack */
/* mpack -s no_subject -o outputfile frog.jpg */
if(strlen(pa -> attachment) != 0)
	{
	sprintf(temp, "mpack -s none -o %s/.xagent/attachment.%ld %s",\
	home_dir, body_posting, pa -> attachment);
	if(debug_flag) fprintf(stdout, "temp=%s\n", temp);
	exec_filefd = popen(temp, "r");
	pclose(exec_filefd);
	}
if(type == NEW_ARTICLE)
	{
	pa -> references = strsave("");/* no references */
	}
if(type == FOLLOW_UP_ARTICLE)
	{
	/* get the original article id */
	if(posting_source == POSTING_MENU)
		{
		ptr = (char *) get_header_data(group, article_id, "Message-ID:");
		if(! ptr) return(0);

		/* test if the original article has a references line */
		ptr2 = (char *) get_header_data(group, article_id, "References:");
	
		/*
		if no references line present, make one containing the original
		article id
		*/
		if(! ptr2)
			{
			pa -> references = strsave(ptr);
			}
		else
			{	
			/*
			if an original references line is present:
			add the original article id to the list of references.
			*/
			ptr3 =\
			(char *) malloc(strlen(ptr) + strlen(ptr2) + strlen(" ") + 1);
			strcpy(ptr3, ptr2);/* references */
			strcat(ptr3, " ");/* separator */
			strcat(ptr3, ptr);/* id */
			pa -> references = ptr3;
			free(ptr2);
			}
		free(ptr);
		}/* end posting_source == POSTING_MENU */
	else /* posting list, or online menu, group and article_id not set */
		{
		ptr = pold -> references;
		if(ptr) pa -> references = strsave(ptr);
		else pa -> references = strsave("");
		}	
	}/* end type = follow up article */

if(type == REPLY_VIA_EMAIL)
	{
	pa -> references = strsave("");/* no references */
	}/* end type is reply via email */

ptr = (char *) fl_get_input(fdui -> summary_editor_input_field);
if(! ptr)
	{
	fl_show_alert(\
	"save_posting():", "summary_editor_input_field returns NULL", "", 0);
	return(0);
	}
/*
Note: pa is a new structure, pa -> summary is undefined, so DO NOT FREE
*/
/*free(pa -> summary);*/
if(strlen(ptr) != 0) pa -> summary = strsave("use header");
else pa -> summary = strsave("");
	
pa -> type = type;
pa -> content_type = strsave("");/* to be done */
pa -> encoding = 0;/* to be done */
pa -> lock_flag = 0;
pa -> browser_line = 0;
pa -> send_status = mode;
pa -> custom_headers_flag = custom_headers_state; /* this last is a global */	

/* calculate the number of lines */
ptr = (char *) fl_get_input(fdui -> posting_body_editor);
if(! ptr)
	{
	fl_show_alert(\
	"save_posting():", "possting_body_editor returns NULL", "", 0);
	return(0);
	}
lines = 0;
while(*ptr++)
	{
	if(*ptr == '\n') lines++;
	}
pa -> lines = lines;

if(! save_posting_header(body_posting) ) return(0);
if(! save_posting_body(body_posting) ) return(0);

if(custom_headers_state & LOCAL_HEADERS_MODIFIED)
	{
	ptr =\
	(char *) fl_get_input(fdui -> local_custom_headers_editor_input_field);
	if(! ptr)
		{
		fl_show_alert(\
		"save_posting():", "custom_headers_editor returns NULL", "", 0);
		return(0);
		}
	/* save the custom headers_file */
	sprintf(pathfilename, "%s/.xagent/custom.%ld", home_dir, body_posting);
	custom_headers_filefd = fopen(pathfilename, "w");
	if(! custom_headers_filefd)
		{
		fl_show_alert("Cannot open file", pathfilename, "for write", 0);
		return(0);
		}
	fprintf(custom_headers_filefd, "%s", ptr);
	/* if the user did not provide the terminating cr */
/*
fprintf(stdout, "END=%c(%d)\n",\
*(ptr + strlen(ptr) - 1), *(ptr + strlen(ptr) - 1) );
*/
	if(*(ptr + strlen(ptr) - 1) != '\n')
		{
		fprintf(custom_headers_filefd, "\n");
		}
	fclose(custom_headers_filefd);
	}/* end local headers modified */

if(! new_posting)
	{
	pa -> lock_flag = pold -> lock_flag;/* over rule 0 above */

	/*
	if the old structure send_status was a SEND_OK, keep it,
	else delete it.
	*/
	if(pold -> send_status != SEND_OK)
		{
 		delete_posting_entry_header_body_attachment(atol(pold -> name) );
		}
	}

/* save the structure, it was modified */

save_postings();
return(1);
}/* end function save_posting */


int set_send_status(long posting_id, int status)
{
struct posting *pa, *lookup_posting();
char temp[80];

if(debug_flag)
	{
	fprintf(stdout,\
	"set_send_status(): arg  posting_id=%ld status=%d\n",\
	posting_id, status);
	}

sprintf(temp, "%ld", posting_id);
pa = lookup_posting(temp);
if(! pa) return(0);

pa -> send_status = status;

save_postings();

return(1);
}/* end function set_send_status */


int get_send_status()
{
struct posting *pa, *lookup_posting();
char temp[80];

if(debug_flag)fprintf(stdout, "get_send_mode(): arg using body_posting=%ld\n",\
body_posting);

sprintf(temp, "%ld", body_posting);
pa = lookup_posting(temp);
if(! pa) return(0);

return(pa -> send_status);
}/* end function set_send_status */


int save_posting_header(long posting_id)
{
char temp[512];	
FILE *header_filefd;
struct posting *pa;
extern char *strsave();
char *ptr;

if(debug_flag)
	{
	fprintf(stdout, "save_posting_header(): arg posting_id=%ld\n",\
	posting_id);
	}

/* argument check */
if(posting_id < 0) return(0);

sprintf(temp, "%s/.xagent/head.%ld", home_dir, posting_id);
header_filefd = fopen(temp, "w");
if(! header_filefd)
	{
	fl_show_alert("could not open file for write", temp, "", 0);
	return(0);
	}

sprintf(temp, "%ld", posting_id);
pa = lookup_posting(temp);
if(! pa) return(0);


/*
From: pante@epsilon.nl (Jan Mourer)
Newsgroups: comp.os.linux.development.apps
or
To:pante@epsilon.nl (Jan Mourer)
Subject: Posting test, I am writing an offline newsreader.
Date: 18 Feb 1997 04:28:10 -0100
Lines: 8
*/

/* Required lines */

/* From */
fprintf(header_filefd, "From: %s\n", pa -> from);

/* Newsgroups or To */
ptr = (char *) fl_get_input(fdui -> posting_to_input_field); 
if(! ptr) 
	{
	fl_show_alert(\
	"save_posting_header():", "to input returns NULL", "", 0);
	return(0);
	}
pa -> to = strsave(ptr);

if( ( pa -> type == NEW_ARTICLE) || (pa -> type == FOLLOW_UP_ARTICLE) )
	{
	fprintf(header_filefd, "Newsgroups: %s\n", pa -> to);
	}
if(pa -> type == REPLY_VIA_EMAIL)
	{
	fprintf(header_filefd, "To: %s\n", pa -> to);
	}

/* Subject */
ptr = (char *) fl_get_input(fdui -> posting_subject_input_field);
if(! ptr)
	{
	fl_show_alert(\
	"save_posting_header():", "subject input returns NULL", "", 0);
	return(0);
	}
fprintf(header_filefd, "Subject: %s\n", fold_text(ptr) );

/* Date */
fprintf(header_filefd, "Date: %s\n", pa -> date);

/* References (in case followup) */
if(pa -> type == FOLLOW_UP_ARTICLE)
	{
	fprintf(header_filefd, "References: %s\n", pa -> references);
	}

/* Optional lines */

/* Summary */
ptr = (char *) fl_get_input(fdui -> summary_editor_input_field);
if(! ptr)
	{
	fl_show_alert(\
	"save_posting_header():", "summary input returns NULL", "", 0);
	return(0);
	}
/* if usefull pa -> summary then write to header file */
if(strlen(ptr) != 0)
	{
	fprintf(header_filefd, "Summary: %s\n", fold_text(ptr) );
	}

/* ID */
fprintf(header_filefd,\
"X-Newsreader: %s (c) off line news reader (in development) by Jan Mourer\n",\
VERSION);

if(debug_flag) fprintf(stdout, "\n");

fclose(header_filefd);	

/* save structure, it has changed */
save_postings();
return(1);
}/* end function save_posting_header */


int save_posting_body(long posting_id)
{
char temp[512];	
FILE *save_filefd;
extern char *strsave();
char *text;

if(debug_flag)
	{
	fprintf(stdout,\
	"save_posting_body(): arg posting_id=%ld\n", posting_id);
	}

sprintf(temp, "%s/.xagent/body.%ld", home_dir, posting_id);
save_filefd = fopen(temp, "w");
if(! save_filefd)
	{
	fl_show_alert("could not open file for write ", temp, "", 0);
	return(0);
	}

text = (char *) fl_get_input(fdui -> posting_body_editor);	
if(! text)
	{
	fl_show_alert(\
	"save_posting_body():", "posting body editor returns NULL", "", 0);
	return(0);/* no text to send */
	}

fprintf(save_filefd, "%s", text);
/*
quarantee a '\n at the end, else if attachment or .\n follows,
attachment will overwrite last line body (on screen only) or no end of 
text marker.
*/
if(text[strlen(text) - 1] != '\n') fprintf(save_filefd, "\n");

fclose(save_filefd);	
return(1);
}/* end function save_posting_body */


int load_and_ident_article(char *group, long article_id)
{
int c;
char temp[512];
struct stat *statptr;
char *space, *spaceptr;
FILE *load_filefd;
char *expanded_space;

if(debug_flag)
	{
	fprintf(stdout, "load_and_ident(): arg group=%s article_id=%ld\n",\
	group, article_id);
	}

/* parameter check */
if(! group) return(0);

/* posting present, load it */
sprintf(temp, "%s/.xagent/%s/body.%ld", home_dir, group, article_id);
load_filefd = fopen(temp, "r");
if(! load_filefd)
	{
	fl_show_alert("could not load file", temp, "", 0);
	return(0);
	}

statptr = (struct stat*) malloc(sizeof(struct stat) );
if(! statptr) return(0);

fstat(fileno(load_filefd), statptr);

/* speed is important, no memory use here */
space = malloc(2 * (statptr -> st_size + 1) );/* 2 in case all LF */
free(statptr);
if(! space)
	{
	if(debug_flag)
		{
		fprintf(stdout,\
		"load_and_ident_article(): malloc could not allocate space\n");
		}
	return(0);
	}

spaceptr = space;
*spaceptr = '>';
spaceptr++;
while(1)
	{
	while(1)/* error re read */
		{
		c = getc(load_filefd);
		if(! ferror(load_filefd) ) break;
		perror("load_and_ident_article(): read failed ");
		}/* end while error re read */	

	if(feof(load_filefd) )
		{
		fclose(load_filefd);
		break;
		}
	*spaceptr = c;
	spaceptr++;
	if(c == '\n')
		{
		*spaceptr = '>';
		spaceptr++;
		}
	}/* end while all lines from article body */
*spaceptr = 0;/* string termination */

expanded_space = (char *) expand_tabs(space, tab_size);
free(space);

fl_set_input(fdui -> posting_body_editor, expanded_space);
free(expanded_space);

return(1);
}/* end function load_and_ident_article */


int get_new_posting_id(long *posting_id)
{
struct posting *pa;
long li;
char temp[50];

if(debug_flag) fprintf(stdout, "get_new_posting_id(): arg none\n");

for(li = 0; li < MAX_POSTINGS; li++)
	{
	sprintf(temp, "%ld", li);
	pa = lookup_posting(temp);/* place was free */
	if(! pa)	
		{
		*posting_id = li;
		return(1);
		}
	}
return(0);/* no free space */
}/* end function get_new_posting_id */


char *get_universal_time_as_string()
{
time_t now;
static char temp[512];
size_t strftime();
struct tm *universal_time, *gmtime();
char *ptr;
char *strsave();

if(debug_flag)
	{
	fprintf(stdout, "get_universal_time_as_string(): arg none\n");
	}

/* 
decided to adapt to universal time (UT == GMT).
New format:
from rfc1036.html:
Wdy, DD Mon YY HH:MM:SS TIMEZONE 
*/

now = time(0);
universal_time = gmtime(&now);
strftime (temp, 511, "%a, %d %b %y %H:%M:%S GMT", universal_time);

ptr = strsave(temp);

if(debug_flag)
	{
	fprintf(stdout, "get_universal_time_as_string(): ptr=%s\n", ptr);
	}

return(ptr);
}/* end function get_universal_time_as_string */


char *get_header_data(char *group, long article_id, char *field)
{
int c, i;
FILE *header_file;
char temp[READSIZE];
char *ptr;
int lf_flag;
int first_char_flag;
int first_char;
char *strsave();

if(debug_flag)
	{
	fprintf(stdout,
	"get_header_data(): arg group=%s article_id=%ld field=%s\n",\
	group, article_id, field);
	}

/* argument check */
if(! group) return(0);
if(! field) return(0);
if(article_id < 0) return(0);

first_char = 0;/* only for clean compilation with -Wall */

sprintf(temp, "%s/.xagent/%s/head.%ld", home_dir, group, article_id);
header_file = fopen(temp, "r");
if(! header_file)
	{
	perror("get_header_data():");
	fl_show_alert("cannot open file for read", temp, "", 0);
	return(0);
	}
	
first_char_flag = 0;
while(1)/* all entries (unfolded header lines) in header file */
	{
	i = 0;
	lf_flag = 0;
	while(1)/* all characters in a line */
		{
		if(first_char_flag)
			{
			temp[i] = first_char;
			i++;
			first_char_flag = 0;
			}

		while(1)/* error re read */
			{
			c = getc(header_file);
			if(! ferror(header_file) ) break;
			perror("get_header_data(): read failed ");
			}/* end while error re read */	

		if(feof(header_file) )
			{
			fclose(header_file);
			if( !lf_flag) return(0);
			break;
			}
		
		if(lf_flag)
			{
			if( (c != ' ') && (c != 9) ) /* TAB = 9, no LWS */
				{
				/*
				line ends, save this character, it is the first one in the
				next line.  
				*/
				first_char = c;
				first_char_flag = 1;
				break;
				}
			/* line continous */
			lf_flag = 0;
			/* store char in line */
			}
			
		if(c == '\n')
			{
			lf_flag = 1;		
			continue;/* get next char, test for linear white space */
			}
			
		temp[i] = c;
		i++;
		if(i >= READSIZE) break; /* prevent overflow */
		}/* end while all characters in a line */
	temp[i] = 0;/* string termination */

	if(strstr(temp, field) == temp)/* found */
		{
		fclose(header_file);
		ptr = strsave(strstr(temp, ":") + 2);
		return(ptr);/* must be freed by calling */
		}
	if(feof(header_file) ) return(0);
	}/* end while all entries (unfolded header lines) in header file */
}/* end function get_article_data */


char *get_formatted_header_data(\
char *group, long article_id, char *field)
{
int c, i;
FILE *header_file;
char temp[READSIZE];
char *ptr;
int lf_flag;
int first_char_flag;
int first_char;
char *strsave();

if(debug_flag)
	{
	fprintf(stdout,
	"get_formatted_header_data(): arg group=%s article_id=%ld field=%s\n",\
	group, article_id, field);
	}

/* argument check */
if(! group) return(0);
if(! field) return(0);
if(article_id < 0) return(0);

first_char  = 0;/* only for clean compilation with -Wall */

sprintf(temp, "%s/.xagent/%s/head.%ld", home_dir, group, article_id);
header_file = fopen(temp, "r");
if(! header_file)
	{
	perror("get_formatted_header_data():");
	fl_show_alert("cannot open file for read", temp, "", 0);
	return(0);
	}

first_char_flag = 0;
while(1)/* all entries (unfolded header lines) in header file */
	{
	i = 0;
	lf_flag = 0;
	while(1)/* all characters in a line */
		{
		if(first_char_flag)
			{
			temp[i] = first_char;
			i++;
			first_char_flag = 0;
			}

		while(1)/* error re read */
			{
			c = getc(header_file);
			if(! ferror(header_file) ) break;
			perror("get_formatted_header_data(): read failed ");
			}/* end while error re read */	

		if( (c == EOF) || (feof(header_file) ) )
			{
			fclose(header_file);
			if( !lf_flag) return(0);
			c = EOF;
			break;
			}
		
		if(lf_flag)
			{
			if( (c != ' ') && (c != 9) ) /* TAB = 9, no LWS */
				{
				/*
				line ends, save this character, it is the first one in the
				next line.  
				*/
				first_char = c;
				first_char_flag = 1;
				break;
				}
			/* line continous */
			temp[i] = '\n';
			i++;
			lf_flag = 0;
			/* store char in line */
			}
			
		if(c == '\n')
			{
			lf_flag = 1;		
			continue;/* get next char, test for linear white space */
			}
			
		temp[i] = c;
		i++;
		if(i >= READSIZE) break; /* prevent overflow */
		}/* end while all characters in a line */
	temp[i] = 0;/* string termination */

	if(strstr(temp, field) == temp)/* found */
		{
		fclose(header_file);
		ptr = strsave(strstr(temp, ":") + 2);
		return(ptr);/* must be freed by calling */
		}
	if(c == EOF) return(0);
	if(feof(header_file) ) return(0);
	}/* end while all entries (unfolded header lines) in header file */
}/* end function get_formatted_header_data */


int delete_posting_entry_header_body_attachment(long posting_id)
{
int a;
struct posting *pa;
char temp[512];

if(debug_flag)
	{
	fprintf(stdout, "delete_posting_entry_header_body(): arg posting_id=%ld\n",\
	posting_id);
	}

/* argument check */
if(posting_id < 0) return(0);

/* delete header */
sprintf(temp, "%s/.xagent/head.%ld", home_dir, posting_id);
unlink(temp);

/* delete attachment */
if(! test_if_posting_has_attachment(posting_id, &a) ) return(0);
if(a)
	{
	sprintf(temp, "%s/.xagent/attachment.%ld", home_dir, posting_id);
	unlink(temp);
	}

/* delete body */
sprintf(temp, "%s/.xagent/body.%ld", home_dir, posting_id);
unlink(temp);

sprintf(temp, "%ld", posting_id);
pa = lookup_posting(temp);
if(! pa) return(0);
if(pa -> custom_headers_flag)
	{
	/* delete custom header */
	sprintf(temp, "%s/.xagent/custom.%ld", home_dir, posting_id);
	unlink(temp);
	}

/* delete entry from structure */
sprintf(temp, "%ld", posting_id);
delete_posting(temp);

/* structure has changed, save it */
save_postings();

return(1);
}/* end function delete_posting_entry_header_body_attachment */


int post_email(long posting_id)
{
int a;
FILE *header_filefd;
FILE *custom_headers_filefd;
FILE *body_filefd;
FILE *attachment_filefd;
FILE *sendmail_filefd;
char temp[READSIZE];
char temp2[READSIZE];
char path_filename[512];
int custom_headers_flag;

struct posting *lookup_posting();

if(debug_flag)
	{
	fprintf(stdout, "post_email(): arg posting_id=%ld\n", posting_id);
	}

/* argument check */
if(posting_id < 0) return(0);

/*
get original "From", it will become Reply-To
original_from = (char *) get_header_data("", posting_id, "From:");
if(! original_from) return(0);
*/

/* open channel to sendmail */
sprintf(temp, "sendmail -t -oem -odb -oi");
sendmail_filefd = popen(temp, "w");
if(! sendmail_filefd)
	{
	my_show_alert("Cannot start sendmail", "", "");
	return(0);
	}

/* open header file */
sprintf(temp, "%s/.xagent/head.%ld", home_dir, posting_id);
header_filefd = fopen(temp, "r");
if(! header_filefd)
	{
	pclose(sendmail_filefd);
	sprintf(temp2, "Cannot open file %s for read", temp);
	my_show_alert("Cannot send email", temp2, "");
	return(0);
	}
while(1)
	{
	a = readline(header_filefd, temp);
	if(a == EOF) break;
	/*
	sendmail will use user@host as From, replies must go to the real email
 	address
 	*/

	/* If a From line, replace the original From with Reply-To */
	if(strstr(temp, "From:") == temp)
		{
		sprintf(temp2, "Reply-To: %s", strstr(temp, ":") + 2); 
		strcpy(temp, temp2);
		}	
	
	if(debug_flag) fprintf(stdout, "header to sendmail=%s\n", temp);	
	fprintf(sendmail_filefd, "%s\n", temp);
	if(ferror(sendmail_filefd) )
		{
		my_show_alert("Cannot send email", "cannot write to sendmail", "");
		fclose(header_filefd);
		pclose(sendmail_filefd);
		return(0);
		}	
	}/* end while read header file */

/* get custom_headers_flag */
if(! get_custom_headers_flag(posting_id, &custom_headers_flag) )
	{
	sprintf(temp, "Cannot get custom headers flag for posting_id %ld",\
	posting_id);
	my_show_alert("Cannot send email", temp, "");
	pclose(sendmail_filefd);
	return(0);
	}

/* test for global custom headers enabled */
if(custom_headers_flag & GLOBAL_HEADERS_ENABLED)
	{
	sprintf(path_filename, "%s/.xagent/global_custom_head",\
	home_dir);
	custom_headers_filefd = fopen(path_filename, "r");			
	if(! custom_headers_filefd)
		{
		sprintf(temp, "Cannot open file %s for read", path_filename);
		my_show_alert("Cannot send email", temp, "");
		pclose(sendmail_filefd);
		return(0);
		}
	/* inform user what is happening */
	sprintf(temp, "Posting %s", path_filename);
	my_command_status_display(temp);
	while(1)
		{
		a = readline(custom_headers_filefd, temp);
		if(a == EOF) break;
		/* prevent end of header detection if empty line */
		if(strlen(temp) == 0) continue;
		if(debug_flag) fprintf(stdout, "post_email():temp=%s\n", temp);
		fprintf(sendmail_filefd, "%s\n", temp);
		if(ferror(sendmail_filefd) )
			{
			my_show_alert(\
			"Cannot send email", "cannot write to sendmail", "");
			pclose(sendmail_filefd);
			fclose(custom_headers_filefd);
			return(0);
			}	
		}/* end while all characters in custom headers file */
	}/* end if a global custom headers enabled */

/* test for local custom headers enabled */
if(custom_headers_flag & LOCAL_HEADERS_ENABLED)
	{
	sprintf(path_filename, "%s/.xagent/custom.%ld",\
	home_dir, posting_id);
	custom_headers_filefd = fopen(path_filename, "r");			
	if(! custom_headers_filefd)
		{
		sprintf(temp, "Cannot open file %s for read", path_filename);
		my_show_alert("Cannot send email", temp, "");
		pclose(sendmail_filefd);
		return(0);
		}
	/* inform user what is happening */
	sprintf(temp, "Posting %s", path_filename);
	my_command_status_display(temp);
	while(1)
		{
		a = readline(custom_headers_filefd, temp);
		if(a == EOF) break;
		/* prevent end of header detection if empty line */
		if(strlen(temp) == 0) continue;
		if(debug_flag) fprintf(stdout, "post_email():temp=%s\n", temp);
		fprintf(sendmail_filefd, "%s\n", temp);
		if(ferror(sendmail_filefd) )
			{
			my_show_alert(\
			"Cannot send email", "cannot write to sendmail", "");
			pclose(sendmail_filefd);
			fclose(custom_headers_filefd);
			return(0);
			}	
		}/* end while all characters in custom headers file */
	}/* end if local custom headers enabled */

/* a blank line separates header and body */
fprintf(sendmail_filefd, "\n");
if(ferror(sendmail_filefd) )
	{
	my_show_alert("Cannot send email", "cannot write to sendmail", "");
	pclose(sendmail_filefd);
	return(0);
	}	

/* open body file */
sprintf(temp, "%s/.xagent/body.%ld", home_dir, posting_id);
body_filefd = fopen(temp, "r");
if(! body_filefd)
	{
	pclose(sendmail_filefd);
	sprintf(temp2, "Cannot open file %s for read", temp);
	my_show_alert("Cannot send email", temp2, "");
	return(0);
	}
while(1)
	{
	a = readline(body_filefd, temp);
	if(a == EOF) break;

	/*
	if the first character of a line is a period, quote it
	with another period
	*/
	if(temp[0] == '.') sprintf(temp2, ".%s", temp);
	else sprintf(temp2, "%s", temp);
	strcpy(temp, temp2);

	if(debug_flag) fprintf(stdout, "post_email():temp=%s\n", temp);
	fprintf(sendmail_filefd, "%s\n", temp);
	if(ferror(sendmail_filefd) )
		{
		my_show_alert("Cannot send email", "cannot write to sendmail", "");
		pclose(sendmail_filefd);
		fclose(body_filefd);
		return(0);
		}	
	}/* end while body file */

/* test if there is an attachment */
if(! test_if_posting_has_attachment(posting_id, &a) ) return(0);
if(a)
	{
	/* Try to open the attachment file attachment.%ld */
	sprintf(temp, "%s/.xagent/attachment.%ld", home_dir, posting_id);
	attachment_filefd = fopen(temp, "r");
	if(! attachment_filefd)
		{
		pclose(sendmail_filefd);
		sprintf(temp2, "Cannot open file %s for read", temp);
		my_show_alert("Cannot send email", temp2, "");
		return(0);
		}
	while(1)
		{
		a = readline(attachment_filefd, temp);
		if(a == EOF) break;
		if(debug_flag)fprintf(stdout, "post_email():temp=%s\n", temp);

		/* 
		if the first character of a line is a period, quote it
		with another period
		*/
		if(temp[0] == '.') sprintf(temp2, ".%s", temp);
		else sprintf(temp2, "%s", temp);
		strcpy(temp, temp2);

		fprintf(sendmail_filefd, "%s\n", temp);
		if(ferror(sendmail_filefd) )
			{
			my_show_alert("Cannot send email", "cannot write to sendmail", "");
			pclose(sendmail_filefd);
			fclose(attachment_filefd);
			return(0);
			}	
		}/* end while attachment file */
	}/* end strlen pa -> attachment not 0 */

/* Note: last line attachment not guaranteed to have a \n, up to mpack */

/* a single period on a line indicates end of text */
fprintf(sendmail_filefd, ".\n");
if(ferror(sendmail_filefd) )
	{
	my_show_alert("Cannot send email", "cannot write to sendmail", "");
	pclose(sendmail_filefd);
	return(0);
	}	
	
pclose(sendmail_filefd);
set_send_status(posting_id, SEND_OK);
my_command_status_display("");
return(1);
}/* end function post_mail */


int get_posting_type(long posting_id)
{
struct posting *pa, *lookup_posting();
char temp[50];

if(debug_flag)
	{
	fprintf(stdout, "get_posting_type(): arg posting_id=%ld\n", posting_id);
	}

sprintf(temp, "%ld", posting_id);
pa = lookup_posting(temp);
if(! pa) return(0);

return(pa -> type);
}/* end function get_posting_type */


int test_if_posting_has_attachment(long posting_article, int *present_flag)
{
struct posting *pa, *lookup_posting();
char temp[80];

if(debug_flag)
	{
	fprintf(stdout,\
	"test_if_posting_has_attachment(): arg posting_article=%ld\n",\
	posting_article);
	}

/* argument check */
if(posting_article < 0) return(0);

sprintf(temp, "%ld", posting_article);
pa = lookup_posting(temp);
if(! pa)
	{
	if(debug_flag) fprintf(stdout, "could not lookup_posting %s\n", temp);
	return(0);
	}
if(! pa -> attachment)
	{
	if(debug_flag) fprintf(stdout, "pa -> attachent == NULL\n");
	return(0);
	}
if(strlen(pa -> attachment) == 0)
	{
	*present_flag = 0;/* no attachment */
	return(1);
	}
*present_flag = 1;/* some attachment */	
return(1);
}/* end function test_if_posting_has_attachment */
 
 
int set_lock_posting(long posting_id, int status)
{
struct posting *pa, *lookup_posting();
char temp[80];

if(debug_flag)
	{
	fprintf(stdout, "set_lock_posting(); arg posting_id=%ld status=%d\n",\
	posting_id, status);
	}
	
/* parameter check */
if(posting_id < 0) return(0);
if( (status != 0) && (status != 1) ) return(0);	

sprintf(temp, "%ld", posting_id);
pa = lookup_posting(temp);
if(! pa) return(0);
pa -> lock_flag = status;

save_postings();

return(1);
}/* end function set_lock_posting */


int get_posting_status(long posting_id, int *send, int *lock)
{
struct posting *pa, *lookup_posting();
char temp[80];

if(debug_flag)
	{
	fprintf(stdout, "get_posting_status(); arg posting_id=%ld\n",\
	posting_id);
	}
	
/* parameter check */
if(posting_id < 0) return(0);

sprintf(temp, "%ld", posting_id);
pa = lookup_posting(temp);
if(! pa) return(0);

*send = pa -> send_status;
*lock = pa -> lock_flag;

if(debug_flag) fprintf(stdout, "send=%d lock=%d\n", *send, *lock);

return(1);
}/* end function get_posting_status */


int add_marked_posting_bodies_to_command_queue()
{
struct posting *pa;
int found_flag;
int type;

if(debug_flag)
	{
	fprintf(stdout, "add_marked_postings_to_commands_queue(): arg none\n");
	}

while(1)/* while there are postings to send (see below) */
	{
	found_flag = 0;
	for(pa = postingtab[0]; pa != 0; pa = pa -> nxtentr)
		{
		if (pa -> send_status == TO_BE_SEND)
			{
			if(debug_flag ) fprintf(stdout, "pa -> status is TO_BE_SEND\n");
			
			/* fill in the input fields (not really showing them) */
			body_posting = atol(pa -> name);
			
			if(! show_posting_body(body_posting) )	/* filter:
														permission denied
													*/
				{
				continue;/* _______possible bug____*/
				}
			
			/* Note: save_posting() uses body_posting */

			posting_source = POSTING_LIST;
			if(! save_posting("dummy", 99999999, SEND_NOW) ) return(0);
			/* Note: save_posting() has done a save_postings(); */	
			
			/* Note: structure posting now modified, cannot use pa */
			type = get_posting_type(body_posting);
			if(debug_flag) fprintf(stdout, "type detected=%d\n", type);
			
			if( (type == NEW_ARTICLE) || (type == FOLLOW_UP_ARTICLE))
				{
				if(debug_flag) fprintf(stdout, "ARTICLE\n");
				add_to_command_queue("dummy", body_posting, POST_ARTICLE);
				}
			if(type == REPLY_VIA_EMAIL)
				{
				if(debug_flag) fprintf(stdout, "EMAIL\n");
				add_to_command_queue("dummy", body_posting, EMAIL_ARTICLE);
				}
			/*
			this is tricky, the structure posting is now modified by 
			save_posting(), must leave the for, but there may be more
			to be send.
			*/
			found_flag = 1;
			}/* end if status to be send */
		if(found_flag) break;
		}/* end for all structures */
	if(debug_flag) fprintf(stdout, "re entering\n");
	if(! found_flag) break;/* no more to post */
	}/* end while something to post */	

return(1);
}/* end function add_marked_posting_bodies_to_command_queue */


int search_next_posting()
{
struct posting *pa, *lookup_posting();
char temp[80];
extern long atol();
extern char *strsave();

if(debug_flag)
	{
	fprintf(stdout,\
	"search_next_posting(): arg none,using selected_posting=%ld\n",\
	selected_posting);
	}

search_last_search = SEARCH_POSTINGS;

/* move to present position */
sprintf(temp, "%ld", selected_posting);
pa = lookup_posting(temp);
if(! pa) pa = postingtab[0];
else pa = pa -> nxtentr; /* select next position */

/* 
and if pa == 0 for() will end, and we are moved to the start of the structure.
*/

/*
Note: start_posting set to selected_posting by search_accept_button_cb.
*/

while(1)
	{
	/* search for next posting from this point onwards */
	for(pa = pa; pa != 0; pa = pa -> nxtentr)/* for all postings */
		{
		/* test for permission to show header */
		if(filters_enabled_flag)
			{
			if(pa -> filter_flags & FILTER_DO_NOT_SHOW_HEADER) continue;
			}/* end if filters_enabled_flag */
		/*
		a special case is when there is only one matching posting,
		and it is the selected one.
		If we do not stop here, it will be found over and over again.
		*/
		if(atol(pa -> name) == first_matching_posting)
			{
/*			ask_question_flag = 0;*/
			if(! search_articles_flag)
				{
				fl_show_alert(\
				"All postings have been searched", "", "", 0);
				return(0);/* round */
				}
			strcpy(start_group, selected_group);
			start_article = selected_article;
			go_search_next_article_from_idle_cb_flag = 1;
			return(0);
			}
		if(search_all(\
		"dummy", pa -> name, pa -> encoding,\
		pa -> attachment, 1, SEARCH_POSTINGS) ) /* have_body = 1 */
			{
			selected_posting = atol(pa -> name);
			show_browser(POSTING_BODY_BROWSER);
			refresh_screen(selected_group);
			if(first_matching_posting == -1)
				{
				first_matching_posting = atol(pa -> name);
				}
			return(1);
			}/* end found */
		/* test for round */
		if(atol(pa -> name) == start_posting)
			{
/*			ask_question_flag = 0;*/
			if(! search_articles_flag)
				{
				fl_show_alert(\
				"All postings have been searched", "", "", 0);
				return(0);/* round */
				}
			strcpy(start_group, selected_group);
			start_article = selected_article;
			go_search_next_article_from_idle_cb_flag = 1;
			return(0);
			}
		}/* end for all structures */
	pa = postingtab[0];
	}/* end while re enter for first part of structure */
}/* end function search_next_posting */


int get_custom_headers_flag(long posting_article, int *present_flag)
{
struct posting *pa, *lookup_posting();
char temp[80];

if(debug_flag)
	{
	fprintf(stdout,\
	"get_custom_headers_flag(): arg posting_article=%ld\n",\
	posting_article);
	}

/* argument check */
if(posting_article < 0) return(0);

sprintf(temp, "%ld", posting_article);
pa = lookup_posting(temp);
if(! pa)
	{
	if(debug_flag)
		{
		fprintf(stdout,\
		"get_custom_headers_flag(): could not lookup_posting %s\n", temp);
		}
	return(0);
	}
*present_flag = pa -> custom_headers_flag;/* some custom_headers */	
return(1);
}/* end function get_custom_headers_flag */
 
 
int set_all_filter_flags_in_postings()
{
struct posting *pa;

if(debug_flag)
	{
	fprintf(stdout, "set_all_filter_flags_in_postings(): arg none\n");
	}
	
for(pa = postingtab[0]; pa != 0; pa = pa -> nxtentr)
	{
	/* get the filter result */
	pa -> filter_flags = 0;/* the default */

	get_filter_flags(\
	"dummy", pa -> name, pa -> encoding, pa -> attachment,\
	1, SEARCH_POSTINGS, &(pa -> filter_flags) );
											/* Note: have_body_flag = 1 */
	}/* end for all structures */

save_postings();
return(1);
}/* end function set_all_filter_flags_in_postings */


int posting_show_permission(long posting_id)
{
struct posting *pa, *lookup_posting();
char temp[50];

if(debug_flag)
	{
	fprintf(stdout, "posting_show_permission(): arg posting_id=%ld\n",\
	posting_id);
	}

sprintf(temp, "%ld", posting_id);
pa = lookup_posting(temp);
if(! pa) return(0);

/* test if allowed */
if(filters_enabled_flag)
	{
	if(pa -> filter_flags & FILTER_DO_NOT_SHOW_BODY)
		{
		return(0);
		}
	}/* end if filters_enabled_flag */
return(1);
}/* end function posting_show_permission */


