/* 
xagent offline NNTP news reader
xagent is registered Copyright (C) 1997 <Jan Mourer>
email: pante@pi.net
snail mail: PO BOX 61  SINT ANNA  9076ZP HOLLAND

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "xagent.h"

extern FD_xagent *fd_xagent;
extern FD_xagent *fdui;


struct helpers
	{
	char *name;
	char *def;
	struct helpers *nxtentr;
	};
static struct helpers *helperstab[1];


struct helpers *lookup_helpers(char *name, int sequence)
{
struct helpers *pa;
int j;

j = 0;
/*pa points to next entry*/
for(pa = helperstab[0]; pa != 0; pa = pa -> nxtentr)
	{
	if(strcmp(pa -> name, name) == 0)
		{
		if(sequence == j) return(pa);/* found sequence entry */
		j++;
		}
	}
return(0);/*not found*/
}


struct helpers *install_helpers(char *name, char *def)
{
struct helpers *pa, *lookup_helpers();
extern char *strsave();
char *malloc();

/* Note: more then one entry with the same nam may exist */
	
pa = (struct helpers *) malloc( sizeof(*pa) );
if(!pa) return(0);
pa -> name = strsave(name);
if(! pa -> name) return(0);
pa -> def = strsave(def);
if(!pa -> def) return(0);
pa -> nxtentr = helperstab[0];
helperstab[0] = pa;
return(pa);
}


struct helpers *reverse_lookup_helpers(char *name, int sequence)
{
struct helpers *pa, *plastentry, *allentries[256];
int i, j;

/* want the first entry in the helpersfile to be the first to be tried */
/* there may be smarter ways, but I have tried ! */
/* load all structure addresses in array */
i = 0;
for(pa = helperstab[0]; pa != 0; pa = pa -> nxtentr)
	{
	if(i >= 256)
		{
		if(debug_flag)fprintf(stdout,\
		"\nreverse_lookup_helpers: To many entries in helpers\n");
		return(0);
		}
	allentries[i] = pa;
	if(pa) plastentry = pa;
	i++;
	}
/* read the array from the end */
j = 0;
for(i = i - 1; i >= 0; i--)
	{
	pa = allentries[i];
	if(strcmp(pa -> name, name) == 0)
		{
		if(sequence == j) return(pa);
		j++;
		}
	}
return(0);/*not found*/
}


int set_helpers(char *name, char *def)
{
struct helpers *install_helpers();

if(! install_helpers(name, def) ) return(0);
return(1);
}


int get_helpers(char *name, char *def, int sequence)
{
struct helpers *pa, *lookup_helpers(), *install_helpers();

pa = reverse_lookup_helpers(name, sequence);
if(! pa) return(0);
strcpy(def, pa -> def);
return(1);
}


int delete_helpers()
{
struct helpers *pa;

if(debug_flag)
	{
	fprintf(stdout, "delete helpers(): arg none\n");
	}

while(1)
	{
	pa = helperstab[0];
	if(! pa) break;
	helperstab[0] = pa -> nxtentr; /* helperstab entry points to next one,
											this could be 0 */
	free(pa -> name);
	free(pa -> def);
	free(pa);/* free structure */
	}/* end while all structures */ 

return(1);
}/* end function delete_helpers */


int readline(FILE *file, char *contents)
{
int a, c, i;

for(i = 0; i < READSIZE - 1; i++)
	{
	while(1)
		{
		c = getc(file);
		a = ferror(file);
		if(a)
			{
			perror("readline():");
			continue;
			}
		break;
		}
	if(feof(file) )
		{
		fclose(file);
		contents[i] = 0;/* EOF marker */
		return(EOF);
		}
	if(c == '\n')
		{
		contents[i] = 0;/* string termination */
		return(1);/* end of line */
		} 
	contents[i] = c;
	}
/*
mmm since we are here, the line must be quite long, possibly something
is wrong.
Since I do not always check for a return 0 in the use uf this function,
just to be safe, gona force a string termination.
This prevents stack overflow, and variables getting overwritten.
*/
contents[i] = 0;/* force string termination */
fclose(file);
return(0);
}/* end function readline */


int load_helpers()
{
int a;
char pathfilename[512];
char temp[READSIZE];
char arg0[512], arg1[512], arg2[512];
extern char *strstr();
extern char *strsave();
char *cp;
FILE *helpersfile;

if(debug_flag) fprintf(stdout, "load_helpers(): arg none\n");

/* clear the structure first */
delete_helpers();
/* test if a helpersfile helpers is present */
/* combine with home directory */
sprintf(pathfilename, "%s/.xagent/setup/helpers", home_dir);
helpersfile = fopen(pathfilename, "r");
if(! helpersfile)/* no helpers file present */
	{
	if(debug_flag)
		{
		fl_show_alert("Cannot open file for read:", pathfilename, "", 0);
		fprintf(stdout, "\nCannot open helpers file %s\n", pathfilename);
		}
	return(0);
	}
else /* a helpers file present */
	{
	/* read data from helpers helpersfile into database */
	while(1)
		{
		/* read a line from the helpers file */
		a = readline(helpersfile, temp);
		if(a == EOF)break;

		/* lines starting with '#' are scipped */
		if(temp[0] == '#') continue; 

		/* scip comments to end of line */
		cp = strstr(temp, "//");

		/* force a string termination on "//" */
		if(cp) *cp = 0;
		
		/* get the start of the second field */
		a = sscanf(temp, "%s %s %s", arg0, arg1, arg2);

		/* scip empty lines */
		if (a == -1) continue;

		if(debug_flag)
			{
			fprintf(stdout,\
			"\nInStAl:a=%d temp=%s arg0=%s arg1=%s",\
			a, temp, arg0, arg1);
			}

		/*
		test if any second field was present,
		need at least 3 arguments for a valid entry.
		*/
		if( (a < 3) || (! set_helpers(arg0, strstr(temp, arg1) ) ) )
			{
			fl_show_alert("Error in helpers file in line:", temp, "", 0);
			if(debug_flag)
				{
				fprintf(stdout,\
"\nCannot install data from helpersfile in database, line reads %s\n", temp);
				}
			if(helpersfile) fclose(helpersfile);
			return(0);
			}
		}/* end while read all lines from helpersfile */

	}/* end if helpers file present */

return(1);
}/* end function load_helpers */


int show_helpers()
{
int c;
char temp[512];
FILE *load_filefd;
struct stat *statptr;
char *space, *spaceptr;

sprintf(temp, "%s/.xagent/setup/helpers", home_dir);
load_filefd = fopen(temp, "r");
if(! load_filefd)
	{
	fl_show_alert("could not load file", temp, "", 0);
	return(0);
	}

statptr = (struct stat*) malloc(sizeof(struct stat) );
fstat(fileno(load_filefd), statptr);
space = malloc(statptr -> st_size + 1);
free(statptr);
if(! space)
	{
	if(debug_flag)
		{
		fprintf(stdout,\
		"show_helpers(): malloc could not allocate space\n");
		}
	return(0);
	}

spaceptr = space;
while(1)
	{
	while(1)/* error re read */
		{
		c = getc(load_filefd);
		if(! ferror(load_filefd) ) break;
		perror("show_helpers(): read failed ");
		}/* end while error re read */	

	if(feof(load_filefd) )
		{
		fclose(load_filefd);
		break;
		}
	*spaceptr = c;
	spaceptr++;
	}/* end while all lines from article body */
*spaceptr = 0;/* string termination */

fl_set_input(fdui -> helpers_editor, space);
free(space);

return(1);
}/* end function show_helpers */


int save_helpers()
{
int a;
char helpers[512];	
char helpers_bak[512];
FILE *save_filefd;
extern char *strsave();
char *text;

if(debug_flag) fprintf(stdout, "\nsave_helpers(): arg none\n");

/* make a path_filename */
sprintf(helpers, "%s/.xagent/setup/helpers", home_dir);

/* make a backup filename */
sprintf(helpers_bak, "%s.bak", helpers);

/* delete any previous backup filename */
unlink(helpers_bak);

/* rename the present helpers to helpers.bak */
a = rename(helpers, helpers_bak);
if(a == -1)
	{
	if(debug_flag)
		{
		fprintf(stdout,\
		"\nCould not rename %s into %s\n", helpers, helpers_bak);
		}	
	return(0);
	}
save_filefd = fopen(helpers, "w");
if(! save_filefd)
	{
	fl_show_alert("could not open file for write ", helpers, "", 0);
	return(0);
	}
text = (char *) fl_get_input(fdui -> helpers_editor);	
fprintf(save_filefd, "%s", text);

fclose(save_filefd);	
return(1);
}/* end function save_helpers */


int save_setting(char *type, char *string)
{
char pathfilename[512];
FILE *setting_file;

/* test if a helpersfile helpers is present */
/* combine with home directory */
sprintf(pathfilename, "%s/.xagent/setup/%s", home_dir, type);
setting_file = fopen(pathfilename, "w");
if(! setting_file)
	{
	fl_show_alert("Cannot open file for write:", pathfilename, "", 0);
	if(debug_flag)
		{
		fprintf(stdout, "\nCannot open setting file %s for write\n",\
		pathfilename);
		}
	return(0);
	}

fprintf(setting_file, "%s", string);
fclose(setting_file);
return(1);	
}/* end function save_setting */


char *load_setting(char *type)
{
int a;
char temp[1024];
char pathfilename[512];
FILE *setting_file;
extern char *strsave();

/* combine with home directory */
sprintf(pathfilename, "%s/.xagent/setup/%s", home_dir, type);
setting_file = fopen(pathfilename, "r");
if(! setting_file)
	{
	fl_show_alert("Cannot open file for read:", pathfilename, "", 0);
	if(debug_flag)
		{
		fprintf(stdout, "\nCannot open setting file %s for read\n",\
		pathfilename);
		}
	return(0);
	}

a = readline(setting_file, temp);/* closes file if EOF */	
if(a != EOF) fclose(setting_file);
return(strsave(temp) );
}/* end function load_settings */


int save_fontsizes()
{
char temp[512];
sprintf(temp, "%d %d %d %d %d %d %d %d %d %d %d",\
group_list_browser_fontsize,\
article_list_browser_fontsize,\
article_body_browser_fontsize,\
posting_list_browser_fontsize,\
posting_body_editor_fontsize,\
helpers_editor_fontsize,\
desc_input_fontsize,\
custom_headers_editor_fontsize,\
filter_form_fontsize,\
summary_editor_fontsize,\
header_form_fontsize);
if(! save_setting("fontsizes", temp) ) return(0);

return(1);
}/* end function save_fontsizes */


int load_fontsizes()
{
char *data_read;

data_read = (char *) load_setting("fontsizes");
if(! data_read) return(0);
sscanf(data_read, "%d %d %d %d %d %d %d %d %d %d %d",\
&group_list_browser_fontsize,\
&article_list_browser_fontsize,\
&article_body_browser_fontsize,\
&posting_list_browser_fontsize,\
&posting_body_editor_fontsize,\
&helpers_editor_fontsize,\
&desc_input_fontsize,\
&custom_headers_editor_fontsize,\
&filter_form_fontsize,\
&summary_editor_fontsize,\
&header_form_fontsize);
free(data_read);

return(1);
}/* end function load_fontsizes */


int load_preferences()
{
char *ptr;
extern char *strsave();


load_fontsizes();
fl_set_browser_fontsize(fdui -> group_list_browser,\
integer_to_fontsize(group_list_browser_fontsize) );
fl_set_browser_fontsize(fdui -> article_list_browser,\
integer_to_fontsize(article_list_browser_fontsize) );
fl_set_object_lsize(fdui -> article_body_input_field,\
integer_to_fontsize(article_body_browser_fontsize) );
fl_set_browser_fontsize(fdui -> posting_list_browser,\
integer_to_fontsize(posting_list_browser_fontsize) );
fl_set_object_lsize(fdui -> posting_body_editor,\
integer_to_fontsize(posting_body_editor_fontsize) );
fl_set_object_lsize(fdui -> helpers_editor,\
integer_to_fontsize(helpers_editor_fontsize) );
fl_set_object_lsize(fdui -> desc_input,\
integer_to_fontsize(desc_input_fontsize) );
fl_set_browser_fontsize(fdui -> filter_list_browser,\
integer_to_fontsize(filter_form_fontsize) );
fl_set_object_lsize(fdui -> summary_editor_input_field,\
integer_to_fontsize(summary_editor_fontsize) );
fl_set_object_lsize(fdui -> header_form_input_field,\
integer_to_fontsize(header_form_fontsize) );

fl_set_object_label(fdui -> description_display, "SUBSCRIBED GROUPS");
ptr = (char *) load_setting("news_server");
free(news_server_name);
news_server_name = (char *) strsave(ptr);
free(ptr);
fl_set_input(fd_xagent -> news_server_input_field, news_server_name);

ptr = load_setting("news_server_port");
news_server_port = atoi(ptr);
fl_set_input(fd_xagent -> news_server_port_input_field, ptr);
free(ptr);

ptr = load_setting("tab_size");
tab_size = atoi(ptr);
fl_set_input(fd_xagent -> tab_size_input_field, ptr);
free(ptr);
set_tabs(tab_size);

ptr = (char *) load_setting("mail_server");
free(mail_server_name);
mail_server_name = (char *) strsave(ptr);
free(ptr);
fl_set_input(fd_xagent -> mail_server_input_field, mail_server_name);

ptr = (char *) load_setting("user_email");
free(user_email_address);
user_email_address = (char *) strsave(ptr);
free(ptr);
fl_set_input(fd_xagent -> user_email_input_field, user_email_address);

ptr = (char *) load_setting("user_name");
free(user_name);
user_name = (char *) strsave(ptr);
free(ptr);
fl_set_input(fd_xagent -> user_name_input_field, user_name);

ptr = load_setting("maximum_headers");
maximum_headers = atoi(ptr);
fl_set_input(fd_xagent -> maximum_headers_per_group_input_field, ptr);
free(ptr);
	
ptr = load_setting("connect_timeout");
connect_to_server_timeout = atoi(ptr);
fl_set_input(fd_xagent -> connect_to_server_timeout_input_field, ptr);
free(ptr);

ptr = (char *) load_setting("connect_to_net_script");
free(connect_to_net_program_name);
connect_to_net_program_name = (char *) strsave(ptr);
free(ptr);
fl_set_input(fd_xagent -> auto_connect_to_net_input_field,\
connect_to_net_program_name);

ptr = (char *) load_setting("disconnect_from_net_script");
free(disconnect_from_net_program_name);
disconnect_from_net_program_name = (char *) strsave(ptr);
free(ptr);
fl_set_input(fd_xagent -> auto_disconnect_from_net_input_field,\
disconnect_from_net_program_name);

ptr = (char *) load_setting("auto_execute_script");
free(auto_execute_program_name);
auto_execute_program_name = (char *) strsave(ptr);
free(ptr);
fl_set_input(fd_xagent -> auto_execute_application_input_field,\
auto_execute_program_name);

ptr = (char *) load_setting("net_device_pid");
free(pid_test_program_name);
pid_test_program_name = (char *) strsave(ptr);
free(ptr);
fl_set_input(fd_xagent -> pid_test_input_field, pid_test_program_name);

ptr = (char *) load_setting("get_new_headers_time");
free(get_new_headers_time);
get_new_headers_time = (char *) strsave(ptr);
free(ptr);
fl_set_input(fd_xagent -> get_new_headers_time_input_field,\
get_new_headers_time);

ptr = (char *) load_setting("secret_password");
free(secret_password);
secret_password = (char *) strsave(ptr);
free(ptr);
fl_set_input(fd_xagent -> secret_password_input_field,\
secret_password);

ptr = (char *) load_setting("filter_password");
free(filter_password);
filter_password = (char *) strsave(ptr);
free(ptr);
fl_set_input(fd_xagent -> filter_password_input_field, "");
/*filter_password);*/

ptr = (char *) load_setting("search_groups_keywords");
free(search_groups_keywords);
search_groups_keywords = (char *) strsave(ptr);
free(ptr);
fl_set_input(fd_xagent -> search_groups_input_field,\
search_groups_keywords);

ptr = (char *) load_setting("search_keywords");
free(search_keywords);
search_keywords = (char *) strsave(ptr);
free(ptr);
fl_set_input(fd_xagent -> search_keywords_input_field,\
search_keywords);

/* load flags */
ptr = (char *) load_setting("flags");
sscanf(ptr, "%d %d %d %d %d %d ",\
&enable_description_popup_flag,\
&auto_connect_to_net_flag,\
&auto_disconnect_from_net_flag,\
&auto_execute_program_flag,\
&auto_get_new_headers_flag,\
&filters_enabled_flag);
free(ptr);

/* load search flags */
ptr = (char *) load_setting("search_flags");
sscanf(ptr, "%d %d %d %d %d %d %d %d %d %d %d ",\
&search_postings_flag,\
&search_articles_flag,\
&search_headers_flag,\
&search_headers_from_flag,\
&search_headers_to_flag,\
&search_headers_subject_flag,\
&search_headers_references_flag,\
&search_headers_others_flag,\
&search_bodies_flag,\
&search_attachments_flag,\
&search_case_sensitive_flag);
free(ptr);

fl_set_button(fdui -> desc_button, enable_description_popup_flag);
fl_set_button(fdui -> auto_connect_to_net_button, auto_connect_to_net_flag);
fl_set_button(fdui -> auto_disconnect_from_net_button,\
auto_disconnect_from_net_flag);
fl_set_button(fdui -> auto_execute_application_button,\
auto_execute_program_flag);
fl_set_button(fdui -> get_new_headers_enable_button,\
auto_get_new_headers_flag);
fl_set_button(fdui -> filter_enable_button,\
filters_enabled_flag);
		
fl_set_button(fdui -> search_articles_button, search_articles_flag);
fl_set_button(fdui -> search_headers_button, search_headers_flag);
fl_set_button(fdui -> search_headers_from_button, search_headers_from_flag);
fl_set_button(fdui -> search_headers_to_button, search_headers_to_flag);
fl_set_button(fdui -> search_headers_subject_button,\
search_headers_subject_flag);
fl_set_button(fdui -> search_headers_references_button,\
search_headers_references_flag);
fl_set_button(fdui -> search_headers_others_button,\
search_headers_others_flag);
fl_set_button(fdui -> search_bodies_button, search_bodies_flag);
fl_set_button(fdui -> search_attachments_button, search_attachments_flag);
fl_set_button(fdui -> search_postings_button, search_postings_flag);
fl_set_button(fdui -> search_case_sensitive_button,\
search_case_sensitive_flag);

/* set the color for the stop button */
set_stop_button_off_state_color();	


return(1);
}/* end function load_preferences */


int save_preferences()
{
char temp[512];
char *ptr;
char *crypt();

save_setting("news_server",\
(char *) fl_get_input(fdui -> news_server_input_field) );

save_setting("mail_server",\
(char *) fl_get_input(fdui -> mail_server_input_field) );

save_setting("user_name",\
(char *) fl_get_input(fdui -> user_name_input_field) );

save_setting("user_email",\
(char *) fl_get_input(fdui -> user_email_input_field) );

save_setting("news_server_port",\
(char *) fl_get_input(fdui -> news_server_port_input_field) );

save_setting("tab_size",\
(char *) fl_get_input(fdui -> tab_size_input_field) );

save_setting("maximum_headers",\
(char *) fl_get_input(fdui -> maximum_headers_per_group_input_field) );

save_setting("connect_timeout",\
(char *) fl_get_input(fdui -> connect_to_server_timeout_input_field) );

save_setting("connect_to_net_script",\
(char *) fl_get_input(fdui -> auto_connect_to_net_input_field) );

save_setting("disconnect_from_net_script",\
(char *) fl_get_input(fdui -> auto_disconnect_from_net_input_field) );

save_setting("auto_execute_script",\
(char *) fl_get_input(fdui -> auto_execute_application_input_field) );

save_setting("net_device_pid",\
(char *) fl_get_input(fdui -> pid_test_input_field) );

save_setting("get_new_headers_time",\
(char *) fl_get_input(fdui -> get_new_headers_time_input_field) );

save_setting("secret_password",\
(char *) fl_get_input(fdui -> secret_password_input_field) );

ptr = (char *) fl_get_input(fdui -> filter_password_input_field);
if(strlen(ptr) != 0)
	{
	save_setting("filter_password", (char *) crypt(ptr, "P8") );
	}

save_setting("search_groups_keywords",\
(char *) fl_get_input(fdui -> search_groups_input_field) );

save_setting("search_keywords",\
(char *) fl_get_input(fdui -> search_keywords_input_field) );

/* save flags */
sprintf(temp, "%d %d %d %d %d %d",\
enable_description_popup_flag,\
auto_connect_to_net_flag,\
auto_disconnect_from_net_flag,\
auto_execute_program_flag,\
auto_get_new_headers_flag,\
filters_enabled_flag);
save_setting("flags", temp);

/* save search_flags */
sprintf(temp, "%d %d %d %d %d %d %d %d %d %d %d",\
search_postings_flag,\
search_articles_flag,\
search_headers_flag,\
search_headers_from_flag,\
search_headers_to_flag,\
search_headers_subject_flag,\
search_headers_references_flag,\
search_headers_others_flag,\
search_bodies_flag,\
search_attachments_flag,\
search_case_sensitive_flag);
save_setting("search_flags", temp);

return(1);
}/* end function save_preferences */


int show_setup_form()
{

fl_show_form(fd_xagent -> setup, FL_PLACE_CENTER, FL_NOBORDER, "setup");

show_helpers();
return(1);
}/* end function show_setup */


int set_stop_button_off_state_color()
{
if(server_status != DISCONNECTED)
	{
	/* make the stop button red */
	fl_set_object_color(fdui -> stop_button, FL_RED, FL_COL1);
	return(1);
	}	
if(auto_get_new_headers_flag)
	{
	/* make the stop button orange */
	fl_set_object_color(fdui -> stop_button, FL_DARKORANGE, FL_COL1);
	return(1);
	}
/* make the stop button grey */
fl_set_object_color(fdui -> stop_button, FL_COL1, FL_COL1);
return(1);
}/* end function set_stop_button_off_state_color */


int set_tabs(int tabsize)
{
int i;
char *space;

if(debug_flag)
	{
	fprintf(stdout, "set_tabs(): arg tabsize=%d\n", tabsize);
	}

/* argument check */
if(tabsize < 0) return(0);

/* set the global (uses by expand_tabs ) */
tab_size = tabsize;

space = malloc(tabsize + 1);
if(! space) return(0);
for(i = 0; i < tabsize; i++)
	{
	space[i] = 'a';
	}
space[i] = 0;

fl_set_tabstop(space);
free(space);

return(1);
}/* end funcion set_tabs */


char *expand_tabs(char *text, int tabsize)
{
int c, i, j, k;
int tabs;
char *space;

if(debug_flag)
	{
	fprintf(stdout, "expand_tabsds(): arg text=%s tabsize=%d\n",\
	text, tabsize);
	}

/* argument check */
if(! text) return(0);
if(tabsize < 0) return(0);

/* determine hom many tabs, so we can calculate the required space */
i = 0;
tabs = 0;
while(1)
	{
	c = text[i];
	if(c == 0) break;
	if(c == 9) tabs++;
	i++;
	}/* end while all characters in text */

space = malloc(strlen(text) + (tabs * tabsize) - tabs + 1 + 1);
if(! space) return(0);

/* copy and expand */
i = 0;
j = 0;
while(1)
	{
	c = text[i];
	if(c == 9)
		{
		for(k = 0; k < tabsize; k++)
			{
			space[j] = ' ';
			j++;
			}
		i++;
		continue;
		}
	else
		{
		space[j] = c;
		}
	if(c == 0) break;
	i++;
	j++;
	}
return(space);
}/* end function expand_tabs */

