/* 
xagent offline NNTP news reader
xagent is registered Copyright (C) 1997 <Jan Mourer>
email: pante@pi.net
snail mail: PO BOX 61  SINT ANNA  9076ZP HOLLAND

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "xagent.h"

extern FD_xagent *fd_xagent;
extern FD_xagent *fdui;

#define ARTICLE_BODY_TIMEOUT	30
	
FILE *headerfile;/* must be preserved between calls */
FILE *bodyfile;
FILE *posting_headerfile;
FILE *posting_custom_headers_file;
FILE *posting_bodyfile;
FILE *posting_attachmentfile;
char rx_header_path_filename[512];
char rx_body_path_filename[512];

int header_file_open_flag;
int body_file_open_flag;
time_t last_body_line_time;
time_t now;
int test_if_article_finished_flag;
int retry_cnt;



char *strsave(char *s) /*save char array s somewhere*/
{
char *p, *malloc();

p = malloc(strlen(s) +  1);
if(p)  strcpy(p, s);
return(p);
}


int idle_cb(XEvent *ev, void *data)
{
long diff;
char temp[512];
char weekday[512], month[512];
int monthday, hour, minute, second, year;
int get_headers_hour, get_headers_minute;
float transfer_speed;
unsigned long seconds_elapsed;
float kilobytes_received;
static float old_kilobytes_received;/* prevent flashing display offline */
static unsigned long online_seconds;/* preserve after going offline */
int status;

/* this will kill any zombies left when viewer is destroyed */
waitpid (WAIT_ANY, &status, WNOHANG);

if(audio_alarm_flag)
	{
	fl_ringbell(100);
	audio_alarm_flag++;
	if(audio_alarm_flag == 65535) audio_alarm_flag = 0;
	/* beep,beep,,,,,beep,,beep,,,,,beep.... */
	}

/* 
handle if article did not have the required termination line,
a line with only a period on it
the program will hang if the termination line is not received
*/
if(body_file_open_flag)
	{
	now = time(0);
	diff = (long) now - last_body_line_time;
	if(diff == ARTICLE_BODY_TIMEOUT)
		{
/*		if(debug_flag) */
			{
			fprintf(stdout,\
			"in idle_cb(): * T I ME  O U T * article body, seconds=%ld\n",\
			diff); 
			}
		/* 
		make sure group command is only send once, and not several times 
		whithin this second 
		*/
		last_body_line_time--;
	
		test_if_article_finished_flag = 1;
		
		/*	
		This will indicate that we check the data stream for a reply 
		to the stat command, even though we do not realy expect numeric.
		This will only fail if the article did contain exactly the stat
		reply at this point.
		This is unlikely (Tjernobyl) since the author of the article
		could not know the article id at the time of writing.
		However the server COULD accidently send it.
		In that case the article would be corrupted.
		*/ 
		sprintf(temp, "STAT %ld\n", acquire_article);
		send_to_server(temp);
		}
	}/* end body_file_open_flag */

/* show the transfer speed */
seconds_elapsed = time(0) - online_timer;
/* avoid the divide by zero exeption :) */
if(seconds_elapsed)
	{
	if(server_status != DISCONNECTED)/* else speed shown will decrease */
		{
		transfer_speed = (float) bytes_received / seconds_elapsed;
		/* clear display */
		fl_set_object_label(fdui -> line_speed_display, "");
		sprintf(temp, "%.2f K / S", transfer_speed / 1000.0);
		fl_set_object_label(fdui -> line_speed_display, temp);
 		online_seconds = seconds_elapsed;
 		old_kilobytes_received = -1;/* even if same amount display */
 		}
 	else
 		{
		kilobytes_received = (float) bytes_received / 1000.0;
		if(kilobytes_received != old_kilobytes_received)
			{
			/* clear display */
			fl_set_object_label(fdui -> line_speed_display, "");
			/* not much space in display */
			if(kilobytes_received < 1)
				{
				sprintf(temp, "%.2f %ld",\
				kilobytes_received, online_seconds);
				}
			else if(kilobytes_received < 10)
				{
				sprintf(temp, "%.1f %ld",\
				kilobytes_received, online_seconds);
				}	
			else
				{
				sprintf(temp, "%.0f %ld",\
				kilobytes_received, online_seconds);
				}	
			fl_set_object_label(fdui -> line_speed_display, temp);
			old_kilobytes_received = kilobytes_received;
			}
 		}
 	}/* end if seconds_elapsed */
 	
if(ask_question_flag == 1)
	{
	fl_show_form(fd_xagent -> question,\
	FL_PLACE_CENTER, FL_NOBORDER, "");
	fl_set_object_label(fdui -> question_text, question_string);
	}
if(ask_question_flag == 0)
	{
	fl_hide_form(fd_xagent -> question);
	ask_question_flag = -1;
	}

if(go_search_next_article_from_idle_cb_flag)
	{
	search_next_article(search_groups_keywords);/* resets flag */
	}

if(command_status_string_flag)
	{
	fl_set_object_label(fdui -> command_status_display,\
	command_status_string);
	command_status_string_flag = 0;
	}
if(line_status_string_flag)
	{
	fl_set_object_label(fdui -> line_status_display,\
	line_status_string);
	line_status_string_flag = 0;
	}
if(description_string_flag)
	{
	fl_set_object_label(fdui -> description_display,\
	description_string);
	description_string_flag = 0;
	}	
if(alert_string_flag)
	{
	sprintf(description_string, "%s %s %s",\
	alert_string1, alert_string2, alert_string3);
	fl_set_object_label(fdui -> command_status_display,\
	description_string);
	alert_string_flag = 0;
	}

/* test if time to get new headers in subscribed groups */
if(auto_get_new_headers_flag && (! time_match_flag) )
	{
	/* get present date and time */
	ogettime(weekday, month, &monthday, &hour, &minute, &second, &year);
 	
 	/* parse the user specified get_new_headers_time */
	sscanf(get_new_headers_time, "%d:%d",\
	&get_headers_hour, &get_headers_minute);
	if(debug_flag)
		{
		fprintf(stdout, "ref hour=%d ref min=%d hour=%d min=%d\n",\
		get_headers_hour, get_headers_minute, hour, minute);
		}
 	if( (hour == get_headers_hour) && (minute == get_headers_minute) &&\
 	/* if not back here within one minute (fast connection or error) */
 	(time(0) - start_time > 60) )
 		{
		if(debug_flag)
			{
			fprintf(stdout, "in idle_cb: match on %s\n",\
			get_new_headers_time);
 			}

 		time_match_flag = 1;
		process_command_queue_error_flag = 0;

		/* Note: these things are executed in reverse order */

		/* this is executed first */
		/* post marked postings */
	    posting_source = ONLINE_MENU;
		add_marked_posting_bodies_to_command_queue();
	
		/* this is executed second */
		/* get marked article bodies */
		add_marked_article_bodies_in_subscribed_groups_to_command_queue();

		/* this is executed last */
		/* get new headers (now we should see our own postings). */
		add_new_headers_in_subscribed_groups_to_command_queue();

 		/* 
 		Just before xagent goes offline (when ALL new headers have been
 		received, and the command queue is empty), 
 		if there were headers marked for retrieval because of threads,
 		it will do an other:
 		add_marked_article_bodies_in_subscribed_groups_to_command_queue()
 		*/
 		start_time = time(0);
 		return(1); /* not really needed here */
 		}
 	}/* end if auto_get_new_headers_flag */
 	
if(command_in_progress) return(1);/* wait for io_callback to process command */

if(process_command_queue_error_flag) return(1);/* disconnected */

if(debug_flag)fprintf(stdout, "Calling process_command_queue\n");

if(! process_command_queue() )
	{
	process_command_queue_error_flag = 1; /* que empty or error */
	time_match_flag = 0;
	return(1);
	}
return(1);
}/*end function idle_cb */


void io_read_callback(int socket, long data)
{
char rxbuf[READSIZE];
int a;
long li;
char c;
char temp[READSIZE];/* beware of overflow rxbuf=512 + additional text */
char temp2[READSIZE];
char arg0[512], arg1[512], arg2[512], arg3[512], arg4[512];
static int num_response;/* preserve between calls !!!!! */
static long articles_in_group, first_article, last_article, article_ptr;
static char group[512];/* must be preserved between calls */
int error_flag;
char path_filename[512];
char *ptr;
extern long atol();
extern char *get_header_data();
int cr_flag;

if(debug_flag) fprintf(stdout, "io_read_callback(): socket=%d\n", socket);

cr_flag = 0;
/* read from socket */
li = 0;
while(1)
	{
	errno = 0;
	a = read(socket, &c, 1);
	if(a < 0)
		{
		if(errno == EAGAIN) continue;/* try again */
		
		if(debug_flag)
			{
			fprintf(stdout,\
			"\nread failed, connection to server lost errno=%d\n", errno);
			}

		perror("read failed: connection to server lost");

		fl_remove_io_callback(socketfd,\
		FL_READ, (FL_IO_CALLBACK) io_read_callback);

		/* close the server socket */
		close(socket);

		server_status = DISCONNECTED;
		process_command_queue_error_flag = 1;/* fatal error */

		command_in_progress = 0;
 	
 		if(auto_disconnect_from_net_flag) auto_disconnect_from_net();
 
		return;
		}
	if(a == 0)
		{
		sprintf(temp, "Connection closed by server (EOF)");
		my_command_status_display(temp);
		if(debug_flag)fprintf(stdout, "%s\n", temp);

		fl_remove_io_callback(socketfd,\
		FL_READ, (FL_IO_CALLBACK) io_read_callback);

		/* close the server socket */
		close(socket);

		server_status = DISCONNECTED;
		return;
		}

	/* valid byte received */

	bytes_received++;
	
	if(c == 10)/* LF */
		{
/*fprintf(stdout, "FOUND 10\n");*/
		if(cr_flag)
			{
			break;
			}
		continue;/* no store of LF */
		}
	
	if(c == 13)/* CR */
		{
/*fprintf(stdout, "FOUND 13\n");*/
		cr_flag = 1;
		continue;/* no store of CR */
		}
	else cr_flag = 0;	

	rxbuf[li] = c;
	li++;
	if(li >= READSIZE) break;
	}/* end while read all characters */
rxbuf[li] = 0;/* string terminator */
/* It is guaranteed that there are no 13 or 10 in the string */

if(debug_flag)
	{
	fprintf(stdout, "rxbuf:%s\n", rxbuf);
	}
/* process data */

/* 
first test the data stream for reply to the STAT command,
send by the client to figure out if an article with incorrect
termination line was actually finished
*/
if( (! expect_numeric_flag) && test_if_article_finished_flag &&\
(command_in_progress == GET_ARTICLE_BODY) && (body_file_open_flag) )
	{
/*	if(debug_flag)*/ fprintf(stdout, "IN TEST\n");
	arg0[0] = 0;
	arg1[0] = 0;
	arg2[0] = 0;
	arg3[0] = 0;
	arg4[0] = 0;
	sscanf(rxbuf, "%s %s %s %s %s", arg0, arg1, arg2, arg3, arg4);
	/* 
	this is wat a reply to stat looks like :
	223:  n &lt;a> article retrieved - request text separately 
	*/
	if(atoi(arg0) == 223)
		{
		if(atol(arg1) == acquire_article)
			{
		  	ptr =\
		  	get_header_data(acquire_group, acquire_article, "Message-ID");
			if(debug_flag) fprintf(stdout, "IN TEST ptr=%s\n", ptr);
            /* force close if to many retries */
            /* sometimes Message-ID does not seem to match */
            if(retry_cnt > 1)
            	{
            	free(ptr);
            	ptr = 0;
            	}
            /* if the test cannot be done, force a close bodyfile */
            if(! ptr)
            	{
				if(body_file_open_flag)
					{
					fclose(bodyfile);
					body_file_open_flag = 0;
					add_article_body(acquire_group, acquire_article);

					add_to_new_list(acquire_group, acquire_article);
					
					expect_numeric_flag = 1;
					command_in_progress = 0;
					test_if_article_finished_flag = 0;
/*					if(debug_flag) */
						{
						fprintf(stdout,\
						"IN TEST body file closed ptr zero, returning\n");
						}
					return;
					}
            	}
			if(strcmp(arg2, ptr) == 0) 
				{
				free(ptr);
				/* 
				we can be reasonable sure at this point the article
				was finished 
				*/
/*				if(debug_flag)*/
					{
					fprintf(stdout, "IN TEST match on Message-ID\n");					
					}
				if(body_file_open_flag)
					{
					fclose(bodyfile);
					body_file_open_flag = 0;
					add_article_body(acquire_group, acquire_article);
					
					add_to_new_list(acquire_group, acquire_article);

					expect_numeric_flag = 1;
					command_in_progress = 0;
					test_if_article_finished_flag = 0;
					if(debug_flag)
						{
						fprintf(stdout,\
						"IN TEST body file closed ptr ok, returning\n");
						}
					return;
					}
				}/* end correct Message-ID */
			else /* no Mesage-ID match */ 
				{
				free(ptr);
				retry_cnt++;
				}
			}/* end correct article number */
		}/* end reply to stat */
	test_if_article_finished_flag = 0;
	/* article has not finished yet (slow connection ?) */
	/* but we will try again */
	last_body_line_time = time(0);
/*	if(debug_flag)*/
		{
		fprintf(stdout, "IN TEST test failed trying again later\n");
		}
	}/* end incorrect article termination processing */

/* check for end of data */
if( (rxbuf[0] == '.') && (rxbuf[1] == 0) )
	{
	if(debug_flag)fprintf(stdout, "end of data detected\n");

	if(command_in_progress == LIST)
		{
		save_groups();
		command_in_progress = 0;
		return;
		}
	else if(command_in_progress == GET_NEW_HEADERS_IN_GROUP)
		{

		if(debug_flag) fprintf(stdout, "CLOSING headerfile\n");

		fclose(headerfile);		
		header_file_open_flag = 0;
		
		/* clear user info */
		my_command_status_display("");
		        
		/* add article info to articles.dat file in directory group */
		add_article_header(group, article_ptr);/* reads from headerfile */

		add_to_new_list(group, article_ptr);

		article_ptr++;
		set_article_ptr(group, article_ptr);
		if(article_ptr > last_article)
			{
			save_subscribed_groups();
			command_in_progress = 0;
			return;
			}
/*
		sprintf(temp, "HEAD %ld\n", article_ptr);
		send_to_server(temp);
*/
		expect_numeric_flag = 1;
		if(debug_flag)
			{
			fprintf(stdout, "looping for next header %s \n", temp);
 			}
 		redraw_browser();
		return;/* get next header */
		}
	else if(command_in_progress == GET_ARTICLE_BODY)
 		{
 		/* close body file here */
		
		if(debug_flag)fprintf(stdout, "CLOSING body file\n"); 		

		fclose(bodyfile);
		body_file_open_flag = 0;
		
		/* clear user info */
		my_command_status_display("");
		       
        /* 
        set the have_body_flag in article_headers,dat in dir group,
		and reset the retrieval_flag 
		*/
		add_article_body(acquire_group, acquire_article);
		
		add_to_new_list(acquire_group, acquire_article);
					
 		redraw_browser();

 		command_in_progress = 0;
 		return;
 		}
	
	}/* end end of data detected */

/* get the numeric response if any */
if(expect_numeric_flag)
	{
	arg0[0] = 0;
	arg1[0] = 0;
	arg2[0] = 0;
	arg3[0] = 0;
	arg4[0] = 0;
				
	sscanf(rxbuf, "%s %s %s %s %s", arg0, arg1, arg2, arg3, arg4);
	if( (! isdigit(arg0[0]) ) || (! isdigit(arg0[1]) ) ||\
	(! isdigit(arg0[2]) ) )
		{
		sprintf(temp,\
		"An online error occured: expecting numeric but rxbuf=%s\n",\
		rxbuf); 
		my_show_alert(temp, "", "");

/*		if(debug_flag)*/
			{
			fprintf(stdout,\
			"EN: An online error occured: expecting numeric but rxbuf=%s\n",\
			rxbuf);
			fprintf(stdout,\
			"EN: acquire_group=%s acquire_article=%ld\n",\
			acquire_group, acquire_article);
			fprintf(stdout,\
			"EN: last_acquire_group=%s\n", last_acquire_group);
			fprintf(stdout,\
			"EN: header_file_open_flag=%d body_file_open_flag=%d\n",\
			header_file_open_flag, body_file_open_flag);
			}
				
		/* 
		IGNORE??????? just waiting for numeric to show up,
		but that MAY be text!
		*/
		if(header_file_open_flag) 
			{
			fclose(headerfile);
			/* file may be incomplete, delete */
			unlink(rx_header_path_filename);
			}
		if(body_file_open_flag)	
			{
			fclose(bodyfile);
			/* file may be incomplete, delete */
			unlink(rx_body_path_filename);
			}

/*		return;*/

		my_command_status_display(temp);
		my_line_status_display("Offline");

		/* set the color for the stop button */
		set_stop_button_off_state_color();	

		send_to_server("QUIT\n");
		fl_remove_io_callback(socketfd,\
		FL_READ, (FL_IO_CALLBACK) io_read_callback);
		close(socketfd);
		server_status = DISCONNECTED;
		process_command_queue_error_flag = 1;
		command_in_progress = 0;
 	
 		if(auto_disconnect_from_net_flag) auto_disconnect_from_net();
 		
 		set_new_data_flag(1);
 		
 		if(filters_enabled_flag) set_all_filter_flags_for_new_articles();
 		
		return;
		}
	num_response = atoi(arg0);

	/* process the different commands */
	error_flag =0;	
	switch(num_response)
		{
		case 100:/* help text follows */
			break;
		case 199: /* debug output */
			break;/* ignore */
		case 200:/* server ready - posting allowed*/
			server_mode = POSTING_ALLOWED;
			sprintf(temp, "server ready - posting allowed");
			my_command_status_display(temp);
			break;
		case 201:/* server ready - no posting allowed */
			server_mode = NO_POSTING_ALLOWED;
			my_command_status_display("No posting allowed on this server");
			break;
		case 202: /* slave status noted */
			break;
		case 205: /* closing connection - goodbye! */
			fl_remove_io_callback(socketfd,\
			FL_READ, (FL_IO_CALLBACK) io_read_callback);

			close(socketfd);

			server_status = DISCONNECTED;

			my_show_alert("Connection closed by sever",\
			"server said:", rxbuf);
			my_line_status_display("Offline");
			my_command_status_display( "Connection closed by server");

			process_command_queue_error_flag = 1;
			command_in_progress = 0;
	
	 		if(auto_disconnect_from_net_flag) auto_disconnect_from_net();
		
			return;
			break;
		case 211: /* n f l s group selected */
			/* reply to GROUP */
			/* rxbuf:211 141 71282 71713 sci.astro */
			/*       0   1   2     3     4 */  

			/* Well, any writes MUST now go to this group
			*/
			if(acquire_group) free(acquire_group);
			acquire_group = strsave(arg4);

			/* 
			This is used in commands,c to prevent sending 'GROUP'
			for every body in the group
			*/
			if(last_acquire_group) free(last_acquire_group);
			last_acquire_group = strsave(arg4);
			
			if(command_in_progress == GET_NEW_HEADERS_IN_GROUP)
				{
				articles_in_group = atol(arg1);
				first_article = atol(arg2);
				last_article = atol(arg3);
				sprintf(group, "%s", arg4);
				get_article_ptr(group, &article_ptr);

				if(debug_flag)
					{
					fprintf(stdout,\
					"IN 211: group=%s art=%ld first=%ld last=%ld ptr=%ld\n",\
					group, articles_in_group, first_article, last_article,\
					article_ptr);
					}
				/* test if any new articles */
				if(article_ptr > last_article)/* nothing to do */
					{
					save_subscribed_groups();
					command_in_progress = 0;

					if(debug_flag)
						{
						fprintf(stdout,\
						"ending 211 because article_ptr > last_article\n");				
						}
					set_new_articles(group, 0);
					refresh_screen(selected_group);
					return;
					}
				/* test for not been here for a long time */
				if(article_ptr < first_article)
					{
					article_ptr = first_article;
					set_new_articles(group, articles_in_group);
					}
				/* limit maximum headers */
				if( (last_article - article_ptr) > maximum_headers + 1)
					{
					if(debug_flag)
						{
						fprintf(stdout,\
						"articles_in_group==%ld new articles set to %d\n",\
						articles_in_group, maximum_headers); 
						}
					article_ptr = last_article - maximum_headers + 1;
					/* and tell user */				
					set_new_articles(group, maximum_headers);
					}
				/* now in range */
				set_new_articles(group, last_article - article_ptr + 1);
				
				/* request first header */
/*
				sprintf(temp, "HEAD %ld\n", article_ptr);
*/
				/* 
				the ultimate secret how to speed things up,
				doing it one by one takes sometimes 4 seconds per
				article, for 10 groups and 200 articles per group
				that would take 2000 seconds = 33 minutes.
				Now try this!
				server must have a buffer for maxheaders per group lines.
				*/
				
				/* send the lot */
				for(li = article_ptr; li <= last_article; li++)
					{
					sprintf(temp, "HEAD %ld\n", li);
					send_to_server(temp);
					}

				/* expecting numeric */

				if(debug_flag)
					{
					fprintf(stdout,\
					"LEAVING 211 HEAD requested in GET_NEW_HEADERS_IN_GROUP\n");
					}
				refresh_screen(selected_group);
				return;/*________*/
				break;
				}
			else if(command_in_progress == GET_ARTICLE_BODY)
				{
				/* request body */
				sprintf(temp, "BODY %ld\n", acquire_article);
				send_to_server(temp);
				expect_numeric_flag = 1;

				if(debug_flag)
					{
					fprintf(stdout,\
					"LEAVING 211 BODY requested in GET_ARTICLE_BODY\n");
					}
				return;
				break;
				}	
			else
				{
				sprintf(temp,\
				"Incorrect numeric reply (%d) to GROUP (211)", num_response);
				error_flag = 1;
				break;
				}
		case 215: /* list of newsgroups follows */
			if(command_in_progress != LIST)
				{
				sprintf(temp,\
				"Incorrect numeric reply (%d) to LIST (215)",\
				num_response);
				error_flag = 1;
				break;
				}
			/* reply to LIST */
			load_groups();
			mark_all_groups_cancelled();
			to_command_status_display(\
			"Transfering active list, this may take a long time");
			expect_numeric_flag = 0;
			break;
		case 220: /* n &lt;a> article retrieved - head and body follow */
			break;
		case 221: /* n &lt;a> article retrieved - head follows */
			/* reply to HEAD */
			/* example 221  1096 <flipflap> article retrieved - head follows */
			/*         arg0 arg1 arg2 */ 

			if(debug_flag) fprintf(stdout, "RECEIVED 221 head follows\n");			

			article_ptr = atol(arg1);
			sprintf(rx_header_path_filename, "%s/.xagent/%s/head.%ld",\
			home_dir, acquire_group, article_ptr);

			if(debug_flag)
				{
				fprintf(stdout, "opening file %s\n",\
				rx_header_path_filename);
				}
			headerfile = fopen(rx_header_path_filename, "w");			
			if(! headerfile)
				{
				/* to be done */
				}
			else header_file_open_flag = 1;
				
			/* inform user what is happening */
			sprintf(temp, "Retrieving group %s header %ld",\
			acquire_group, article_ptr);
			my_command_status_display(temp);
			expect_numeric_flag = 0;
			break;
		case 222: /* n &lt;a> article retrieved - body follows */	
			/* 
			reply to BODY
			there are 2 ways to get here, directly if BODY command
			was send by process_command_queue, or indirectly if GROUP
			command was send by process_command_queue and
			command_in_process was GET_ARTILCE_BODY.
			*/

			if(debug_flag) fprintf(stdout, "RECEIVED 222 body follows\n");			

			/* acquire_article = atol(arg1); */
			sprintf(rx_body_path_filename, "%s/.xagent/%s/body.%ld",\
			home_dir, acquire_group, acquire_article);

			if(debug_flag)
				{
				fprintf(stdout, "opening file %s\n", rx_body_path_filename);
				}
			bodyfile = fopen(rx_body_path_filename, "w");			
			if(! bodyfile)
				{
				/* to be done */
				}
			else
				{
				body_file_open_flag = 1;
				retry_cnt = 0;
				}
			/* inform user what is happening */
			sprintf(temp, "Retrieving group %s body %ld",\
			acquire_group, acquire_article);
			my_command_status_display(temp);
			
			expect_numeric_flag = 0;
			break;
		case 223: /* n &lt;a> article retrieved - request text separately */
			/* reply to STAT */
			break;
		case 230: /* list of new articles by message-id follows	*/
			break;
		case 231: /* list of new newsgroups follows	*/
			break;
		case 235: /* article transferred ok */
			sprintf(temp, "article transferred ok");
			error_flag = 1;
			break;
		case 240: /* article posted ok */
			if(command_in_progress != POST_ARTICLE)
				{
				sprintf(temp,\
				"Incorrect numeric reply (%d) to command_in_progress %d",\
				num_response, command_in_progress);
				error_flag = 1;
				break;
				}
			set_send_status(posting_article, SEND_OK);
			refresh_screen(selected_group);

			sprintf(temp, "Article %ld posted OK", posting_article);
			my_command_status_display(temp);
			command_in_progress = 0;
 			return;
			break;
		case 335: /* 
					send article to be transferred.
				 	End with CR-LF>.CR-LF>
					*/
			sprintf(temp, "send article to be transferred");
			error_flag = 1;
			break;
		case 340:	/* 
					send article to be posted.
					End with CR-LF>.CR-LF> 
					*/
			if(command_in_progress != POST_ARTICLE)
				{
				sprintf(temp,\
				"Incorrect numeric reply (%d) to command_in_progress %d",\
				num_response, command_in_progress);
				error_flag = 1;
				break;
				}
				
			/* post header */
			sprintf(path_filename, "%s/.xagent/head.%ld",\
			home_dir, posting_article);
			posting_headerfile = fopen(path_filename, "r");			
			if(! posting_headerfile)
				{
				sprintf(temp,  "Cannot open file %s for read",\
				path_filename);
				my_command_status_display(temp);
				my_show_alert("An online error occured", temp, "");
				if(debug_flag) fprintf(stdout, "%s", temp);
				command_in_progress = 0;
				return;
				break;
				}
			/* inform user what is happening */
			sprintf(temp, "Posting %s", path_filename);
			my_command_status_display(temp);
			while(1)
				{
				a = readline(posting_headerfile, temp);
				if(a == EOF) break;
				strcat(temp, "\n");
				send_to_server(temp);
				}
					
			/* test for global custom_headers enabled */
			if(! get_custom_headers_flag(posting_article, &a) )
				{
				my_command_status_display(\
				"An online error occured, cannot read custom_headers_flag");
				command_in_progress = 0;
				return;
				break;
				}
			if(a & GLOBAL_HEADERS_ENABLED)
				{
				sprintf(path_filename, "%s/.xagent/global_custom_head",\
				home_dir);
				posting_custom_headers_file= fopen(path_filename, "r");			
				if(! posting_custom_headers_file)
					{
					sprintf(temp, "Cannot open file %s for read",\
					path_filename);
					my_command_status_display(temp);
					command_in_progress = 0;
					return;
					break;
					}
				/* inform user what is happening */
				sprintf(temp, "Posting %s", path_filename);
				my_command_status_display(temp);
				while(1)
					{
					a = readline(posting_custom_headers_file, temp);
					if(a == EOF) break;
					/* prevent end of header detection if empty line */
					if(strlen(temp) == 0) continue;
					strcat(temp, "\n");
					send_to_server(temp);
					}/* end while all characters in custom headers file */
				}/* end if a global custom headers enabled */

			/* test for local custom headers enabled */
			if(! get_custom_headers_flag(posting_article, &a) )
				{
				my_command_status_display(\
				"An online error occured, cannot read custom_headers_flag");
				command_in_progress = 0;
				return;
				break;
				}
			if(a & LOCAL_HEADERS_ENABLED)
				{
				sprintf(path_filename, "%s/.xagent/custom.%ld",\
				home_dir, posting_article);
				posting_custom_headers_file = fopen(path_filename, "r");			
				if(! posting_custom_headers_file)
					{
					sprintf(temp, "Cannot open file %s for read",\
					path_filename);
					my_command_status_display(temp);
					command_in_progress = 0;
					return;
					break;
					}
				/* inform user what is happening */
				sprintf(temp, "Posting %s", path_filename);
				my_command_status_display(temp);
				while(1)
					{
					a = readline(posting_custom_headers_file, temp);
					if(a == EOF) break;
					/* prevent end of header detection if empty line */
					if(strlen(temp) == 0) continue;
					strcat(temp, "\n");
					send_to_server(temp);
					}/* end while all characters in custom headers file */
				}/* end if local custom headers enabled */

			/* insert a blank line at the end of the header */
			send_to_server("\n");

			/* post body */
			sprintf(path_filename, "%s/.xagent/body.%ld",\
			home_dir, posting_article);
			posting_bodyfile = fopen(path_filename, "r");			
			if(! posting_bodyfile)
				{
				sprintf(temp, "Cannot open file %s for read",\
				path_filename);
				my_command_status_display(temp);
				command_in_progress = 0;
				return;
				break;
				}
			/* inform user what is happening */
			sprintf(temp, "Posting %s", path_filename);
			my_command_status_display(temp);
			while(1)
				{
				a = readline(posting_bodyfile, temp);
				if(a == EOF) break;
				/* 
				if the first character of a line is a period, quote it
				with another period 
				*/
				if(temp[0] == '.') sprintf(temp2, ".%s\n", temp);
				else sprintf(temp2, "%s\n", temp);
				send_to_server(temp2);
				}

			/* test for attachment */
			if(! test_if_posting_has_attachment(posting_article, &a) )
				{
				my_command_status_display(\
				"An online error occured, cannot read attachment state");
				command_in_progress = 0;
				return;
				break;
				}
			/* if attachment send attachment */
			if(a)
				{
				sprintf(path_filename, "%s/.xagent/attachment.%ld",\
				home_dir, posting_article);
				posting_attachmentfile= fopen(path_filename, "r");			
				if(! posting_attachmentfile)
					{
					sprintf(temp, "Cannot open file %s for read",\
					path_filename);
					my_command_status_display(temp);
					command_in_progress = 0;
					return;
					break;
					}
				/* inform user what is happening */
				sprintf(temp, "Posting %s", path_filename);
				my_command_status_display(temp);
				while(1)
					{
					a = readline(posting_attachmentfile, temp);
					if(a == EOF) break;
					/* 
					if the first character of a line is a period, quote it
					with another period 
					*/
					if(temp[0] == '.') sprintf(temp2, ".%s\n", temp);
					else sprintf(temp2, "%s\n", temp);
					send_to_server(temp2);
					}/* end while all characters in attachment */
				}/* end if an attachment present) */

			/* send end of text */
			
/*
			if(temp2[strlen(temp2) - 1] != '\n') send_to_server("\n");
			send_to_server("\n");
*/
			send_to_server(".\n");			
			expect_numeric_flag = 1;
			break;
		case 400: /* service discontinued */
			server_mode = OUT_OF_ORDER;
			sprintf(temp, "service discontinued");\
			error_flag = 1;
			break;
		case 411: /* no such news group */
			/* 
			cancel command, but do not go offline,
			there may be more and valid ones in queue
			*/
			my_show_alert("An online error occurred, server said:",\
			rxbuf, acquire_group);
/*			expect_numeric_flag = 0; */
			command_in_progress = 0;
			break;
		case 412: /* no newsgroup has been selected */
			sprintf(temp, "no newsgroup has been selected");
			error_flag = 1;
			break;
		case 420: /* no current article has been selected */
			sprintf(temp, "no current article has been selected");
			error_flag = 1;
			break;
		case 421: /* no next article in this group */
			sprintf(temp, "no next article in this group");
			error_flag = 1;
			break;
		case 422: /* no previous article in this group */
			sprintf(temp, "no previous article in this group");
			error_flag = 1;
			break;
		case 423: /* no such article number in this group */
			sprintf(temp, "no such article number in this group");
			if(debug_flag) fprintf(stdout, "%s\n", temp);
		case 430: /* no such article found */
			if(command_in_progress == GET_NEW_HEADERS_IN_GROUP)
				{
				/* there may be more articles, try the next one */
				if(debug_flag)
					{
					fprintf(stdout,\
"error: num_response %d: no article %ld in this group, trying next one\n",\
					num_response, article_ptr);
					}
/*
				if( article_ptr <= last_article) article_ptr++;
				else
*/
				if(article_ptr >= last_article)
					{
					article_ptr++;/* scip error one */
					set_article_ptr(group, article_ptr);
					command_in_progress = 0;
					return;
					}
/*
				sprintf(temp, "HEAD %ld\n", article_ptr);
				send_to_server(temp);
*/
				expect_numeric_flag = 1;

				if(debug_flag)
					{
					fprintf(stdout, "looping for next header %s \n", temp);
					}
				return;/* get next header */
				}
			else if(command_in_progress == GET_ARTICLE_BODY)
				{
				/* cancel this one */
				set_article_no_longer_availeble_flag(\
				acquire_group, acquire_article);
				
				redraw_browser();

				command_in_progress = 0;
				return;
				}
			break;
		case 435: /* article not wanted - do not send it */
			set_send_status(posting_article, SEND_FAILED);
			sprintf(temp, "article not wanted - do not send it");
			error_flag = 1;
			break;
		case 436: /* transfer failed - try again later */
			sprintf(temp, "transfer failed - try again later");
			error_flag = 1;
			break;
		case 437: /* article rejected - do not try again. */
			set_send_status(posting_article, SEND_FAILED);
			sprintf(temp, "article rejected - do not try again.");
			error_flag = 1;
			break;
		case 440: /* posting not allowed */
			set_send_status(posting_article, SEND_FAILED);
			sprintf(temp, "posting not allowed");
			error_flag = 1;
			break;
		case 441: /* posting failed */
			set_send_status(posting_article, SEND_FAILED);
			sprintf(temp, "posting failed");
			error_flag = 1;
			break;
		case 500: /* command not recognized */
			sprintf(temp, "command not recognized");
			error_flag = 1;
			break;
		case 501: /* command syntax error */
			sprintf(temp, "command syntax error");
			error_flag = 1;
			break;
		case 502: /* access restriction or permission denied */
			set_send_status(posting_article, SEND_FAILED);
			sprintf(temp, "access restriction or permission denied");
			error_flag = 1;
			break;
		case 503: /* program fault - command not performed */
			set_send_status(posting_article, SEND_FAILED);
			sprintf(temp, " program fault - command not performed");
			break;
		default: /* some unknown code */
			sprintf(temp, "unknowm numeric response %d from server\n",\
			num_response);
			error_flag = 1;
			break;
		}/* end switch num_response */	
	if(error_flag)
		{
		if(debug_flag) fprintf(stdout, "%s\n", temp);			
		my_command_status_display(temp);

fprintf(stdout, "num_responce=%d temp=%s rxbuf=%s\n",\
num_response, temp, rxbuf);

return;/*____________*/	

		fl_remove_io_callback(socketfd,\
		FL_READ, (FL_IO_CALLBACK) io_read_callback);
		send_to_server("QUIT\n");
		close(socketfd);
		server_status = DISCONNECTED;

		my_line_status_display("Offline");
		my_command_status_display("");
		my_show_alert("An online error occurred", "server said:", rxbuf);

		/* set the color for the stop button */
		set_stop_button_off_state_color();	

		process_command_queue_error_flag = 1;
		command_in_progress = 0;
	
		if(auto_disconnect_from_net_flag) auto_disconnect_from_net();
		
		set_new_data_flag(1);
		
		if(filters_enabled_flag) set_all_filter_flags_for_new_articles();
		
		}/* end if error_flag */
	return;
	}/* end expect_numeric_flag */

/* not numeric */
switch(num_response)
	{
	case 215: /* list of newsgroups folows */
		add_group(rxbuf);
		return;
		break;
	case 221: /* reading header, implied  */
		/* store header lines */

		if(debug_flag)fprintf(stdout, "HDR LI=%s\n", rxbuf);		

		fprintf(headerfile, "%s\n", rxbuf);

		/* get next line if not header end */
		return;
		break;
	case 222: /* reading body */
		/* store body lines */

		if(debug_flag)fprintf(stdout, "BODY LI=%s\n", rxbuf);		

		/* 
		if the first 2 characters in a line are periods, it was
		a quoted period 
		*/
		if(strstr(rxbuf, "..") == rxbuf)
			{
			fprintf(bodyfile, "%s\n", rxbuf + 1);/* scip the first dot */
			}
		else fprintf(bodyfile, "%s\n", rxbuf);
		last_body_line_time = time(0);
			
		/* get next line if not body end */
		return;
		break;
	}/* end switch num_response */		
}/* end function io_read_callback */


void desc_input_cb(FL_OBJECT *ob, long data)
{
if(debug_flag)
	{
	fprintf(stdout, "desc_input_cb(): arg none\n");
	}
}/* end function desc_input_cb */


void desc_close_button_cb(FL_OBJECT *ob, long data)
{
fl_hide_form(fd_xagent -> desc);
}/* end function desc_close_button_cb */


void groups_menu_cb(FL_OBJECT *ob, long data)
{
int i;
char temp[512];
FILE *exec_filefd;
int total_lines;
char s_group[512];
char *ptr;
int something_selected_flag;

groups_menu_selection = fl_get_menu(ob);

if(groups_menu_selection == SET_DISPLAY_FILTER)
	{
	ptr = (char *) fl_show_input(\
	"Filter? (clear selects all)", groups_display_filter);
	if(! ptr) return;/* cancel pressed */
	free(groups_display_filter);
	groups_display_filter = strsave(ptr);
	}
if(groups_menu_selection == SHOW_ALL_GROUPS)
	{
	load_groups();
	fl_set_menu_item_mode(ob, SHOW_ALL_GROUPS, FL_PUP_CHECK);
	fl_set_menu_item_mode(ob, SHOW_ONLY_SUBSCRIBED_GROUPS, FL_PUP_BOX);
	fl_set_menu_item_mode(ob, SHOW_ONLY_NEW_GROUPS, FL_PUP_BOX);
	fl_set_object_label(fdui -> description_display, "ALL GROUPS");
	groups_show_selection = SHOW_ALL_GROUPS;
	}
if(groups_menu_selection == SHOW_ONLY_SUBSCRIBED_GROUPS)
	{
	fl_set_menu_item_mode(ob, SHOW_ALL_GROUPS, FL_PUP_BOX);
	fl_set_menu_item_mode(ob, SHOW_ONLY_SUBSCRIBED_GROUPS, FL_PUP_CHECK);
	fl_set_menu_item_mode(ob, SHOW_ONLY_NEW_GROUPS, FL_PUP_BOX);
	fl_set_object_label(fdui -> description_display, "SUBSCRIBED GROUPS");
	groups_show_selection = SHOW_ONLY_SUBSCRIBED_GROUPS;
	}
if(groups_menu_selection == SHOW_ONLY_NEW_GROUPS)
	{
	load_groups();
	fl_set_menu_item_mode(ob, SHOW_ALL_GROUPS, FL_PUP_BOX);
	fl_set_menu_item_mode(ob, SHOW_ONLY_SUBSCRIBED_GROUPS, FL_PUP_BOX);
	fl_set_menu_item_mode(ob, SHOW_ONLY_NEW_GROUPS, FL_PUP_CHECK);
	fl_set_object_label(fdui -> description_display, "NEW GROUPS");
	groups_show_selection = SHOW_ONLY_NEW_GROUPS;
	}
if(groups_menu_selection ==\
DELETE_ALL_NOT_LOCKED_ARTICLES_IN_SUBSCRIBED_GROUPS)
	{
	delete_all_not_locked_articles_in_subscribed_groups();
	}
if(groups_menu_selection == SUBSCRIBE)
	{
	something_selected_flag = 0;
	if(visible_browser == GROUP_LIST_BROWSER)
		{
		/*
		if show all groups, subscribe to marked groups, else ask for
		group.
		*/
		if(groups_show_selection == SHOW_ALL_GROUPS)
			{
			total_lines = fl_get_browser_maxline(fdui -> group_list_browser);
			for(i = 1; i <= total_lines; i++)
				{
				if (fl_isselected_browser_line(fdui -> group_list_browser, i))
					{
					something_selected_flag = 1;
					strcpy(temp,\
					fl_get_browser_line(fdui -> group_list_browser, i) );
					sscanf(temp, "%s %s", temp, s_group);
					if(! subscribe_group(s_group, 1) ) return;
					sprintf(temp, "mkdir %s/.xagent/%s", home_dir, s_group);
					exec_filefd = popen(temp, "r");
					pclose(exec_filefd);
					}/* end if line selected */
				}/* end for all lines in group list browser */
			}/* end if show all groups */
		}/* end visible browser is group list browser */

	if(! something_selected_flag)
		{
		ptr = (char *) fl_show_input("News group?", "");
		if(!ptr) return;/* cancel pressed */
		load_groups();
		strcpy(s_group, ptr);
		if(! subscribe_group(s_group, 1) ) return;
		sprintf(temp, "mkdir %s/.xagent/%s", home_dir, s_group);
		exec_filefd = popen(temp, "r");
		pclose(exec_filefd);
		}
	save_subscribed_groups();
	}/* end subscribe groups */
if(groups_menu_selection == UNSUBSCRIBE)
	{
	something_selected_flag = 0;
	if(visible_browser == GROUP_LIST_BROWSER)
		{
		total_lines = fl_get_browser_maxline(fdui -> group_list_browser);
		for(i = 1; i <= total_lines; i++)
			{
			if (fl_isselected_browser_line(fdui -> group_list_browser, i))
				{
				something_selected_flag = 1;
				strcpy(temp,\
				fl_get_browser_line(fdui -> group_list_browser, i) );
				sscanf(temp, "%s %s", temp, s_group);
				if(! subscribe_group(s_group, 0) ) return;
				}/* end if line selected */
			}/* end for all lines in group list browser */
		}/* end visible browser is group list browser */

	if(! something_selected_flag)
		{
		ptr = (char *) fl_show_input("Unsubscribe group?", "");
		if(! ptr) return;/* cancel pressed */
		strcpy(s_group, ptr);
		if(! subscribe_group(s_group, 0) ) return;
		}
	save_subscribed_groups();

	/*
	To prevent hanging if now 'next unread' pressed,
	select something.
	*/
	select_first_group_and_article();
	}/* end unsubscribe groups */
if(groups_menu_selection == DELETE_GROUP)
	{
	something_selected_flag = 0;	
	if(visible_browser == GROUP_LIST_BROWSER)
		{
		total_lines = fl_get_browser_maxline(fdui -> group_list_browser);
		for(i = 1; i <= total_lines; i++)
			{
			if (fl_isselected_browser_line(fdui -> group_list_browser, i))
				{
				something_selected_flag = 1;
				strcpy(temp,\
				fl_get_browser_line(fdui -> group_list_browser, i) );
				sscanf(temp, "%s %s", temp, s_group);
					
				completely_delete_group(s_group);
				}/* end if line selected */
			}/* end for all lines in group list browser */
		}/* end visible browser is group list browser */
	
	if(! something_selected_flag)
		{
		ptr = (char *) fl_show_input("Delete group?", "");
		if(! ptr) return;/* cancel pressed */
		strcpy(s_group, ptr);

		completely_delete_group(s_group);
		}
	save_subscribed_groups();

	/*
	To prevent hanging if now 'next unread' pressed,
	select something.
	*/
	select_first_group_and_article();
	}/* end delete groups */
show_browser(GROUP_LIST_BROWSER);
refresh_screen(selected_group);
}/* end function groups_menu_cb */


void article_menu_cb(FL_OBJECT *ob, long data)
{
int a;
int total_lines;
int s_line;
long article_id;

if(! selected_group) return;

articles_menu_selection = fl_get_menu(ob);

if(visible_browser == ARTICLE_BODY_BROWSER)
	{
	if(articles_menu_selection == LOCK_ARTICLE)
		{
		set_lock_flag(selected_group, selected_article, 1);
		}
	if(articles_menu_selection == UNLOCK_ARTICLE)
		{
		set_lock_flag(selected_group, selected_article, 0);
		}
	if(articles_menu_selection == FOLLOW_SUBJECT)
		{
		set_follow_flag(\
		selected_group, selected_article, FOLLOW_SUBJECT);
		}
	if(articles_menu_selection == DO_NOT_FOLLOW_SUBJECT)
		{
		set_follow_flag(\
		selected_group, selected_article, DO_NOT_FOLLOW_SUBJECT);
		}
	if(articles_menu_selection == FOLLOW_THREAD)
		{
		set_follow_flag(\
		selected_group, selected_article, FOLLOW_THREAD);
		}
	if(articles_menu_selection == DO_NOT_FOLLOW_THREAD)
		{
		set_follow_flag(\
		selected_group, selected_article, DO_NOT_FOLLOW_THREAD);
		}
	return;
	}
if(visible_browser != ARTICLE_LIST_BROWSER) return;

total_lines = fl_get_browser_maxline(fdui -> article_list_browser);
for(s_line = 1; s_line <= total_lines; s_line++)
	{
	if(fl_isselected_browser_line(fdui -> article_list_browser, s_line) )
		{
		/* 
		lines in summary do not have an article_id,
		if clicking on a summary, the first previous article_id is used.
		*/
		a = s_line;
		while(1)
			{
			if(line_to_article_id(selected_group, a, &article_id) ) break;
			a--;
			if(a < 1) break;/* first browser line is 1 */
			}

		if(articles_menu_selection == MARK_ARTICLE_FOR_RETRIEVAL)
			{
			set_retrieval_flag(selected_group, article_id, 1);
			}/* end mark article for retrieval */
		if(articles_menu_selection == UNMARK_ARTICLE_FOR_RETRIEVAL)
			{
			set_retrieval_flag(selected_group, article_id, 0);
			}/* end unmark article for retrieval */
		if(articles_menu_selection == DELETE_ARTICLE)
			{
			if(strcmp(selected_group, "xagent.help") == 0)
				{
				fl_show_alert("articles in xagent.help cannot be deleted",\
				"", "", 0);
				return;				
				}
			delete_article_entry_header_body(selected_group, article_id);
			}/* end delete article */
		if(articles_menu_selection == MARK_ARTICLE_READ)
			{
			set_read_flag(selected_group, article_id, 1);
			}/* end mark article read */
		if(articles_menu_selection == MARK_ARTICLE_UNREAD)
			{
			set_read_flag(selected_group, article_id, 0);
			}/* end mark article unread */
		if(articles_menu_selection == LOCK_ARTICLE)
			{
			set_lock_flag(selected_group, article_id, 1);
			}
		if(articles_menu_selection == UNLOCK_ARTICLE)
			{
			set_lock_flag(selected_group, article_id, 0);
			}
		if(articles_menu_selection == FOLLOW_SUBJECT)
			{
			set_follow_flag(\
			selected_group, article_id, FOLLOW_SUBJECT);
			}
		if(articles_menu_selection == DO_NOT_FOLLOW_SUBJECT)
			{
			set_follow_flag(\
			selected_group, article_id, DO_NOT_FOLLOW_SUBJECT);
			}
		if(articles_menu_selection == FOLLOW_THREAD)
			{
			set_follow_flag(\
			selected_group, article_id, FOLLOW_THREAD);
			}
		if(articles_menu_selection == DO_NOT_FOLLOW_THREAD)
			{
			set_follow_flag(\
			selected_group, article_id, DO_NOT_FOLLOW_THREAD);
			}
		}/* end selected line */
	}/* end for all lines */
if(articles_menu_selection == MARK_ALL_NEW_ARTICLES_FOR_RETRIEVAL)
	{
	set_all_retrieval_flags_in_new_articles(selected_group, 1);
	}/* end mark all articles for retrieval */
if(articles_menu_selection == UNMARK_ALL_NEW_ARTICLES_FOR_RETRIEVAL)
	{
	set_all_retrieval_flags_in_new_articles(selected_group, 0);
	}/* end unmark all articles for retrieval */
if(articles_menu_selection == DELETE_ARTICLE)
	{
	get_and_set_unread_articles(selected_group);
	}/* end delete article */
if(articles_menu_selection == DELETE_ALL_NOT_LOCKED_ARTICLES)
	{
	delete_all_not_locked_articles(selected_group);
	get_and_set_unread_articles(selected_group);
	}/* end delete all not locked articles article */
if(articles_menu_selection == MARK_ALL_ARTICLES_READ)
	{
	set_all_read_flags(selected_group, 1);
	get_and_set_unread_articles(selected_group);
	}/* end mark all articles read */
if(articles_menu_selection == MARK_ALL_ARTICLES_UNREAD)
	{
	set_all_read_flags(selected_group, 0);
	get_and_set_unread_articles(selected_group);
	}/* end mark all articles unread */
refresh_screen(selected_group);
}/* end function article_menu_cb */


void posting_menu_cb(FL_OBJECT *ob, long data)
{
int i;
int total_lines;
long posting_id;
int send_status, lock;

posting_menu_selection = fl_get_menu(ob);

/* if nothing selected, but mouse in menu, no action */
if(posting_menu_selection <= 0) return;

posting_source = POSTING_MENU;
if(posting_menu_selection == NEW_ARTICLE)
	{
	if( (visible_browser != ARTICLE_LIST_BROWSER) &&\
	(visible_browser != ARTICLE_BODY_BROWSER) )
		{
		fl_show_alert("Select a news group first", "", "", 0);
		return;
		}
	/* fill in the input fields for the posting editor */
	if(! fill_in_input_fields(\
	selected_group, 99999999, posting_menu_selection) ) return;
	/* show the editor */
	fl_show_form(fd_xagent -> posting_editor,\
	FL_PLACE_CENTER, FL_NOBORDER, "");
	return;
	}
if(posting_menu_selection == FOLLOW_UP_ARTICLE)
	{
	if(visible_browser != ARTICLE_BODY_BROWSER)
		{
		fl_show_alert("Select an article first", "", "", 0);
		return;
		}
	/* fill in the input fields for the posting editor */
	if(! fill_in_input_fields(\
	selected_group, selected_article, posting_menu_selection) ) return;

	/* show the editor */
	fl_show_form(fd_xagent -> posting_editor,\
	FL_PLACE_CENTER, FL_NOBORDER, "");
	return;
	}
if(posting_menu_selection == REPLY_VIA_EMAIL)
	{
	if(visible_browser != ARTICLE_BODY_BROWSER)
		{
		fl_show_alert("Select an article first", "", "", 0);
		return;
		}
	/* fill in the input fields for the posting editor */
	if(! fill_in_input_fields(\
	selected_group, selected_article, posting_menu_selection) ) return;

	/* show the editor */
	fl_show_form(fd_xagent -> posting_editor,\
	FL_PLACE_CENTER, FL_NOBORDER, "");
	return;
	}

/* the following commands all need the posting list */
if (visible_browser != POSTING_LIST_BROWSER)
	{
	fl_show_alert("Select posting list first",\
	"select window then posting list", "", 0);
	return;
	}

total_lines = fl_get_browser_maxline(fdui -> posting_list_browser);
for(i = 1; i <= total_lines; i++)
	{
	if (fl_isselected_browser_line(fdui -> posting_list_browser, i))
		{
		line_to_posting_id(i, &posting_id);
		if(posting_menu_selection == MARK_FOR_TRANSMIT)
			{
			set_send_status(posting_id, TO_BE_SEND);
			}
		if(posting_menu_selection == UNMARK_FOR_TRANSMIT)
			{
			set_send_status(posting_id, SEND_LATER);
			}
		if(posting_menu_selection == LOCK_POSTING)
			{
			set_lock_posting(posting_id, 1);
			}
		if(posting_menu_selection == UNLOCK_POSTING)
			{
			set_lock_posting(posting_id, 0);
			}
		if(posting_menu_selection == MARK_SEND)
			{
			set_send_status(posting_id, SEND_OK);
			}
		if(posting_menu_selection == UNMARK_SEND)
			{
			set_send_status(posting_id, SEND_LATER);
			}
		if(posting_menu_selection == DELETE_SELECTED_POSTINGS)
			{
			get_posting_status(posting_id, &send_status, &lock);
			if(lock) continue;
			if(send_status == SEND_OK) continue;
			if(send_status == TO_BE_SEND) continue;
			delete_posting_entry_header_body_attachment(posting_id);
			}
		}/* end if line selected */
	}/* end for all lines */
refresh_screen(selected_group);
}/* end function posting_menu_cb*/


void options_menu_cb(FL_OBJECT *ob, long data)
{
if(debug_flag)
	{
	fprintf(stdout, "options_menu_selection=%d\n", options_menu_selection);
	}

options_menu_selection = fl_get_menu(ob);

if(options_menu_selection == DEBUG_MODE)	
	{
	debug_flag = 1 - debug_flag;		
	if(debug_flag)
		{
		fl_set_menu_item_mode(ob, DEBUG_MODE, FL_PUP_CHECK);
		}
	else
		{
		fl_set_menu_item_mode(ob, DEBUG_MODE, FL_PUP_BOX);
		}
	}
if(options_menu_selection == SETUP)
	{
	show_setup_form();
	}
if(options_menu_selection == LOCAL_SEARCH)
	{
	show_search_form();
	}
if(options_menu_selection == FILTER)
	{
	show_filter_form();
	}
if(options_menu_selection == SECRET_AGENT)
	{
	show_secret_form();
	}
if(options_menu_selection == XAGENT_INFO)
	{
	show_info_form();
	}

}/* end function options_menu_cb */


void online_menu_cb(FL_OBJECT *ob, long data)
{
int i;
char s_group[512];
int total_lines;
char temp[512];
long posting_id;
int type;

process_command_queue_error_flag = 0;

online_menu_selection = fl_get_menu(ob);

if(online_menu_selection == GET_NEW_HEADERS_IN_SELECTED_GROUPS)
	{
	total_lines  = fl_get_browser_maxline(fdui -> group_list_browser);
	for(i = 1; i <= total_lines; i++)
		{
		if (fl_isselected_browser_line(fdui -> group_list_browser, i))
			{
			strcpy(temp,\
			fl_get_browser_line(fdui -> group_list_browser, i) );
			sscanf(temp, "%s %s", temp, s_group);
		
			add_to_command_queue(s_group, 0, GET_NEW_HEADERS_IN_GROUP);
			}
		}
	}
if(online_menu_selection == GET_NEW_HEADERS_IN_SUBSCRIBED_GROUPS)
	{
	add_new_headers_in_subscribed_groups_to_command_queue();/* in groups.c */
	}
if(online_menu_selection == GET_MARKED_ARTICLE_BODIES)
	{
	add_marked_article_bodies_in_subscribed_groups_to_command_queue();
	}
if(online_menu_selection == REFRESH_GROUP_LIST)
	{
	add_to_command_queue("dummy", 0, LIST);
	}
if(online_menu_selection == POST_SELECTED_POSTINGS)
	{
	if(visible_browser != POSTING_LIST_BROWSER)
		{
		fl_show_alert(\
		"Select posting list first", "(select window, then posting list)",\
		"", 0);
		return;
		}
	total_lines = fl_get_browser_maxline(fdui -> posting_list_browser);
	for(i = 1; i <= total_lines; i++)
		{
		if (fl_isselected_browser_line(fdui -> posting_list_browser, i))
			{
			line_to_posting_id(i, &posting_id);
			body_posting = posting_id;
			type = get_posting_type(body_posting);
			posting_source = ONLINE_MENU;

			/* fill in the input fields (not really showing them) */
			if(! show_posting_body(body_posting) ) /* filter: permission
													denied 
													*/
				{
				continue;
				}
			if(! save_posting("dummy", 99999999, SEND_NOW) ) return;
			if( (type == NEW_ARTICLE) || (type == FOLLOW_UP_ARTICLE) )
				{
				add_to_command_queue("dummy", body_posting , POST_ARTICLE);
				}
			if(type == REPLY_VIA_EMAIL)
				{
				add_to_command_queue("dummy", body_posting , EMAIL_ARTICLE);
				}
			}/* end if line selected */
		}/* end for all lines */
	}
if(online_menu_selection == POST_MARKED_POSTINGS)
	{
	posting_source = ONLINE_MENU;
	add_marked_posting_bodies_to_command_queue();
	}
}/* end function online_menu_cb */


void file_menu_cb(FL_OBJECT *ob, long data)
{
file_menu_selection = fl_get_menu(ob);

if(file_menu_selection == 1) 
	{
/*	save_groups();*/
/*	save_subscribed_groups();*/
	exit(1);
	}
}/* end function file_menu_cb */


void window_menu_cb(FL_OBJECT *ob, long data)
{
window_menu_selection = fl_get_menu(ob);

if(window_menu_selection == GROUP_LIST_WINDOW)
	{
	group_list_button_cb(fdui -> group_list_button, 0);
	}
if(window_menu_selection == ARTICLE_LIST_WINDOW)
	{
	article_list_button_cb(fdui -> article_list_button, 0);
	}
if(window_menu_selection == ARTICLE_BODY_WINDOW)
	{
	article_body_button_cb(fdui -> article_body_button, 0);
	}
if(window_menu_selection == POSTING_LIST_WINDOW)
	{
	posting_list_button_cb();/* button does not exist, just a function */
	}
}/* end function window_menu_cb */


void group_list_browser_cb(FL_OBJECT *ob, long data)
{
}/* end function group_list_browser_cb */


void group_list_browser_double_cb(FL_OBJECT *ob, long data)
{
int a;
char temp[512];
char group[512];

if(debug_flag)
	{
	fprintf(stdout, "group_list_browser_double_cb(): arg none\n");
	}

a = fl_get_browser(ob);
if(! a) return;
		
/* sometimes a is negative why? deselect */
strcpy(temp, fl_get_browser_line(ob, abs(a) ) );
sscanf(temp, "%s %s", temp, group);/* loose format string @C1 */
if(selected_group) free(selected_group);
selected_group = strsave(group);

/* 
select the line, as reference when switching between group list
and article list 
*/ 
fl_select_browser_line(fdui -> group_list_browser, abs(a) );

if(! subscribed_to_group(selected_group) ) return;

fl_set_object_label(fdui -> description_display, selected_group);

show_browser(ARTICLE_LIST_BROWSER);
refresh_screen(selected_group);
}/* end function group_list_browser_double_cb */


void article_list_browser_cb(FL_OBJECT *ob, long data)
{
}/* end function article_list_browser_cb */


void article_list_browser_double_cb(FL_OBJECT *ob, long data)
{
int a, b, selected_line;
long article_id;
int view_permission;
 
if(debug_flag)
	{
	fprintf(stdout, "article_list_browser_double_cb() arg none\n");
	}
selected_line = fl_get_browser(ob);
if(! selected_line) return;

/* sometimes a is negative why? deselect */

/* 
lines in summary do not have an article_id,
if clicking on a summary, the previous article_id is used.
*/
a = abs(selected_line);
while(1)
	{
	if(line_to_article_id(selected_group, a, &article_id) ) break;
	a--;
	if(a < 1) break;/* first browser line is 1 */
	}

selected_article = article_id;

if(debug_flag)
	{
	fprintf(stdout, "SELECTED a=%d article_id=%ld\n", a, article_id);
	}

/* 
select the line, as reference when switching between article list
and article body 
*/ 
fl_select_browser_line(fdui -> article_list_browser, abs(a) );

/* 
NOTE: exists toggles pa -> retrieval_flag,
if no permission to view body and filters enabled, it also returns 0.
*/
if( exists_article_body(selected_group, selected_article) ) 
	{
	/* 
	test if binary attachment was launched before,
	if so do it again 
	*/
	a = get_article_encoding(\
	selected_group, selected_article, &b, &view_permission);
	if(! view_permission)
		{
		fl_set_object_label(fdui -> command_status_display,\
		"Filter: Permission denied");
		return;
		}
		
	if(! a) return;

	if(a && b)
		{
		lounch_binary_attachment(selected_group, selected_article);
		return;
		}
	else
		{
		show_browser(ARTICLE_BODY_BROWSER);
		}
	}/* end article exists */
refresh_screen(selected_group);
}/* end function article_list_browser_double__cb */


void article_body_input_field_cb(FL_OBJECT *ob, long data)
{
show_article_body(selected_group, selected_article);

fl_show_alert(\
"This text cannot be modified",\
"to reply, select an option from the posting pull down menu",\
"", 0);
}/* end function article_body_input_field_cb */


void posting_list_browser_cb(FL_OBJECT *ob, long data)
{
}/* end function posting_list_browser_cb */


void posting_list_browser_double_cb(FL_OBJECT *ob, long data)
{
int a;
int selected_line;

selected_line = fl_get_browser(ob);
	
/* 
lines in summary do not have an posting_id,
if clicking on a summary, the previous posting_id is used.
*/
a = abs(selected_line);
if(! a) return;
while(1)
	{
	if(line_to_posting_id(a, &selected_posting) ) break;
	a--;
	if(a < 1) break;/* first browser line is 1 */
	}

/*
select the line, as reference when switching between posting list
and body
*/ 
fl_select_browser_line(fdui -> posting_list_browser, a);

posting_modified_flag = 0;
posting_source = POSTING_LIST;
body_posting = selected_posting;/* in case send now button pressed */
if(debug_flag)fprintf(stdout, "selected_posting=%ld\n", selected_posting);

if(! posting_show_permission(selected_posting) )
	{
	fl_set_object_label(fdui -> command_status_display,\
	"Filter: Permission denied");
	return;
	}

/* show the editor */
show_posting_body(selected_posting);
fl_show_form(fd_xagent -> posting_editor,\
FL_PLACE_CENTER, FL_NOBORDER, "");
}/* end function posting_list_browser_double_cb */


void group_list_button_cb(FL_OBJECT *ob, long data)
{
char temp[512];

if(visible_browser == GROUP_LIST_BROWSER) return;

if(groups_show_selection == SHOW_ONLY_NEW_GROUPS)
	{
	sprintf(temp, "NEW GROUPS");
	}
if(groups_show_selection == SHOW_ONLY_SUBSCRIBED_GROUPS)
	{
	sprintf(temp, "SUBSCRIBED GROUPS");
	}
if(groups_show_selection == SHOW_ALL_GROUPS) sprintf(temp, "ALL GROUPS");

fl_set_object_label(fdui -> description_display, temp);

show_browser(GROUP_LIST_BROWSER);
refresh_screen(selected_group);
}/* end function group_list_button_cb */


void article_list_button_cb(FL_OBJECT *ob, long data)
{
char temp[512];

if(! selected_group) return;

if(visible_browser == ARTICLE_LIST_BROWSER) return;

sprintf(temp, "%s", selected_group);
fl_set_object_label(fdui -> description_display, temp);

show_browser(ARTICLE_LIST_BROWSER);
refresh_screen(selected_group);
}/* end function article_list_button_cb */


void article_body_button_cb(FL_OBJECT *ob, long data)
{

if(selected_article < 0) return;

if(visible_browser == ARTICLE_BODY_BROWSER) return;

show_browser(ARTICLE_BODY_BROWSER);
refresh_screen(selected_group);
}/* end function article_body_button_cb */


void header_button_cb(FL_OBJECT *ob, long data)
{
if(selected_article < 0) return;

show_article_header(selected_group, selected_article);

fl_show_form(fd_xagent -> header_form,\
FL_PLACE_CENTER, FL_NOBORDER, "");
}/* end function header_button_cb */


int posting_list_button_cb()
/* this button does not exist!! called by window_menu_cb */
{

if(visible_browser == POSTING_LIST_BROWSER) return(0);

fl_set_object_label(fdui -> description_display, "POSTING LIST");
show_browser(POSTING_LIST_BROWSER);
refresh_screen("dummy");/* 
						selected_group = NULL on startup, 
						so if first thing selected is WINDOW then
						POSTING LIST no show, because return 0 
						*/
return(1);
}/* end function posting_list_button_cb */


void next_thread_button_cb(FL_OBJECT *ob, long data)
{
display_next_unread_article_in_subject();
}/* end function next_thread_button_cb */


void next_unread_button_cb(FL_OBJECT *ob, long data)
{
display_next_unread_article();
}/* end function next_unread_button_cb */


void browser_fontsize_up_button_cb(FL_OBJECT *ob, long data)
{
int size;
int fontsize;

fontsize = 0;/* only for clean compilation with -Wall */

if(visible_browser == GROUP_LIST_BROWSER)
	{
	fontsize = group_list_browser_fontsize;
	}
if(visible_browser == ARTICLE_LIST_BROWSER)
	{
	fontsize = article_list_browser_fontsize;
	}
if(visible_browser == ARTICLE_BODY_BROWSER)
	{
	fontsize = article_body_browser_fontsize;
	}
if(visible_browser == POSTING_LIST_BROWSER)
	{
	fontsize = posting_list_browser_fontsize;
	}

if(fontsize < 5) fontsize++;
size = integer_to_fontsize(fontsize);

if(visible_browser == GROUP_LIST_BROWSER)
	{
	fl_set_browser_fontsize(fdui -> group_list_browser, size);
	group_list_browser_fontsize = fontsize;
	}
if(visible_browser == ARTICLE_LIST_BROWSER)
	{
	fl_set_browser_fontsize(fdui -> article_list_browser, size);
	article_list_browser_fontsize = fontsize;
	}
if(visible_browser == ARTICLE_BODY_BROWSER)
	{
	fl_set_object_lsize(fdui -> article_body_input_field, size);
	article_body_browser_fontsize = fontsize;
	}
if(visible_browser == POSTING_LIST_BROWSER)
	{
	fl_set_browser_fontsize(fdui -> posting_list_browser, size);
	posting_list_browser_fontsize = fontsize;
	}

save_fontsizes();
}/* end function size_up_button_cb */


void browser_fontsize_down_button_cb(FL_OBJECT *ob, long data)
{
int size;
int fontsize;

fontsize = 0;/* only for clean compilation with -Wall */

if(visible_browser == GROUP_LIST_BROWSER)
	{
	fontsize = group_list_browser_fontsize;
	}
if(visible_browser == ARTICLE_LIST_BROWSER)
	{
	fontsize = article_list_browser_fontsize;
	}
if(visible_browser == ARTICLE_BODY_BROWSER)
	{
	fontsize = article_body_browser_fontsize;
	}
if(visible_browser == POSTING_LIST_BROWSER)
	{
	fontsize = posting_list_browser_fontsize;
	}

if(fontsize > 0) fontsize--;
size = integer_to_fontsize(fontsize);

if(visible_browser == GROUP_LIST_BROWSER)
	{
	fl_set_browser_fontsize(fdui -> group_list_browser, size);
	group_list_browser_fontsize = fontsize;
	}
if(visible_browser == ARTICLE_LIST_BROWSER)
	{
	fl_set_browser_fontsize(fdui -> article_list_browser, size);
	article_list_browser_fontsize = fontsize;
	}
if(visible_browser == ARTICLE_BODY_BROWSER)
	{
	fl_set_object_lsize(fdui -> article_body_input_field, size);
	article_body_browser_fontsize = fontsize;
	}
if(visible_browser == POSTING_LIST_BROWSER)
	{
	fl_set_browser_fontsize(fdui -> posting_list_browser, size);
	posting_list_browser_fontsize = fontsize;
	}

save_fontsizes();
}/* end function size_down_button_cb */


void desc_input_fontsize_up_button_cb(FL_OBJECT *ob, long data)
{
int size;

if(desc_input_fontsize < 5) desc_input_fontsize++;
size = integer_to_fontsize(desc_input_fontsize);

fl_set_object_lsize(fdui -> desc_input, size);
save_fontsizes();
}/* end function desc_input_fontsize_up_button_cb */


void desc_input_fontsize_down_button_cb(FL_OBJECT *ob, long data)
{
int size;

if(desc_input_fontsize > 0) desc_input_fontsize--;
size = integer_to_fontsize(desc_input_fontsize);

fl_set_object_lsize(fdui -> desc_input, size);
save_fontsizes();
}/* end function desc_input_fontsize_down_button_cb */


void lounch_button_cb(FL_OBJECT *ob, long data)
{
if(visible_browser != ARTICLE_BODY_BROWSER) return;

lounch_binary_attachment(selected_group, selected_article);
return;
}/* end function lounch_button_cb */


void stop_button_cb(FL_OBJECT *ob, long data)
{
/* alarm quitering */
audio_alarm_flag = 0;

if(header_file_open_flag) 
	{
	fclose(headerfile);
	/* file may be incomplete, delete */
	unlink(rx_header_path_filename);
	}
if(body_file_open_flag)
	{
	fclose(bodyfile);
	/* file may be incomplete, delete */
	unlink(rx_body_path_filename);
	}

if(process_command_queue_error_flag) return;

fl_remove_io_callback(socketfd, FL_READ, (FL_IO_CALLBACK) io_read_callback);

send_to_server("QUIT\n");

close(socketfd);

server_status = DISCONNECTED;

my_command_status_display("");
my_line_status_display("Offline");

/* set the color for the stop button */
set_stop_button_off_state_color();	

process_command_queue_error_flag = 1;
delete_all_commands();
command_in_progress = 0;

if(auto_disconnect_from_net_flag) auto_disconnect_from_net();

set_new_data_flag(1);

if(filters_enabled_flag) set_all_filter_flags_for_new_articles();

/* prevent looping within this minute (fast connection, little data ) */
if(time_match_flag) get_new_headers_time -= 60;
time_match_flag = 0;
}/* end function stop_button_cb */


int integer_to_fontsize(int fontsize)
{
int size;

size = 0;/* only for clean compilation with -Wall */

/* tiny snall normal medium large huge */
if(fontsize == 0) size = FL_TINY_SIZE;
if(fontsize == 1) size = FL_SMALL_SIZE;
if(fontsize == 2) size = FL_NORMAL_SIZE;
if(fontsize == 3) size = FL_MEDIUM_SIZE;
if(fontsize == 4) size = FL_LARGE_SIZE;
if(fontsize == 5) size = FL_HUGE_SIZE;

return(size);
}/* end function integer to fontsize */


int redraw_browser()
{
/* argument check */
if(! selected_group) return(0);
if(! acquire_group) return(0);

/* ifacquire group is shown, redraw it (lines will turn blue from magenta) */
if(strcmp(acquire_group, selected_group) == 0)
	{
	refresh_screen(selected_group);
	}
return(1);
}


int show_browser(int browser)
{
if(debug_flag)
	{
	fprintf(stdout, "show_browser(): browser=%d visible_browser=%d\n",\
	browser, visible_browser);
	}

/* test if already selected */
if(browser == visible_browser) return(1);

/* hide the visible one */
if(visible_browser == GROUP_LIST_BROWSER)
	{
	fl_hide_object(fdui -> group_list_browser);
	}
if(visible_browser == ARTICLE_LIST_BROWSER)
	{
	fl_hide_object(fdui -> article_list_browser);
	}
if(visible_browser == ARTICLE_BODY_BROWSER)
	{
	fl_hide_object(fdui -> article_body_input_field);
	fl_hide_object(fdui -> header_button);
	fl_show_object(fdui -> article_body_button);
	}
if(visible_browser == POSTING_LIST_BROWSER)
	{
	fl_hide_object(fdui -> posting_list_browser);
	}

/* show the selected one */
if(browser == GROUP_LIST_BROWSER)
	{
	fl_show_object(fdui -> group_list_browser);
	}
if(browser == ARTICLE_LIST_BROWSER)
	{
	fl_show_object(fdui -> article_list_browser);
	}
if(browser == ARTICLE_BODY_BROWSER)
	{
	fl_show_object(fdui -> article_body_input_field);
	fl_hide_object(fdui -> article_body_button);
	fl_show_object(fdui -> header_button);
	}
if(browser == POSTING_LIST_BROWSER)
	{
	fl_show_object(fdui -> posting_list_browser);
	}

visible_browser = browser;
/* 
clear the command status display (from "helper program xxx started")
this is a compromise, but put it in every button if you want,
and then offline AND online?
*/

my_command_status_display("");
return(1);
}/* end function show browser */


int my_command_status_display(char *text)
{
sprintf(command_status_string, "%s", text);
command_status_string_flag = 1;
return(1);
}/* end function my_command_status_display */


int my_line_status_display(char *text)
{
sprintf(line_status_string, "%s", text);
line_status_string_flag = 1;
return(1);
}/* end function my_line_status_display */


int my_description_display(char *text)
{
sprintf(description_string, "%s", text);
description_string_flag = 1;
return(1);
}/* end function my_description_display */


int my_show_alert(char *text1, char *text2, char *text3)
{
sprintf(alert_string1, "%s", text1);
sprintf(alert_string2, "%s", text2);
sprintf(alert_string3, "%s", text3);
alert_string_flag = 1;
return(1);
}/* end function my_show_alert */
 

int refresh_screen(char *group)
{
if(debug_flag)
	{
	fprintf(stdout,\
	"refresh_screen(): arg group=%s using selected_group=%s\
 visible_browser=%d\n",\
	group, selected_group, visible_browser);
	}

/* argument check */
if(! group) return(0);

/* warn user that diplaying about 28219 lines (groups) takes time */
if(visible_browser == GROUP_LIST_BROWSER)
	{
	if(groups_loaded_flag)
		{
		if( (groups_menu_selection == SHOW_ALL_GROUPS) ||\
		(groups_menu_selection == SHOW_ONLY_NEW_GROUPS) ||\
		(groups_menu_selection == SUBSCRIBE) ||\
		(groups_menu_selection == UNSUBSCRIBE) ||\
		(groups_menu_selection == DELETE_GROUP) )
			{
			to_command_status_display(\
			"building display, this may take a while");
			}
		}/* end if all groups loaded */
	}/* end warn user */

/* prevent flashing */
fl_freeze_form(fdui -> xagent);

if(visible_browser == GROUP_LIST_BROWSER) show_groups();
if(visible_browser == ARTICLE_LIST_BROWSER)
	{
	/* 
	The following strcmp is EXTREMELY important!!!!
	if omitted, show_articles() will load selected_group into the
	structure articles, EVEN if the group being processed is a different
	one, like for example select article list, and then from the groups
	menu, delete all not locked articles in al groups.
	That would mess up ALL articles.dat files.
	The same goes if online called from within io_callback if data is
	read for a different group.
	The calling routine will not know the structure has been replaced
	with that of an other group, and will write to the wrong group
	(i.e. selected group).
	This was a bug in xagent.05 and before, and HARD to find.
	I added the group argument to refresh_screen()
	*/		
	if(strcmp(group, selected_group) == 0) show_articles(selected_group);
	}
if(visible_browser == ARTICLE_BODY_BROWSER)
	{
	/* 
	The following strcmp is EXTREMELY important!!!!
	if omitted, show_article_body() will load selected_group into the
	structure articles, EVEN if the group being processed is a different
	one, like for example select article list, and then from the groups
	menu, delete all not locked articles in al groups.
	That would mess up ALL articles.dat files.
	The same goes if online called from within io_callback if data is
	read for a different group.
	The calling routine will not know the structure has been replaced
	with that of an other group, and will write to the wrong group
	(i.e. selected group).
	This was a bug in xagent.05 and before, and HARD to find.
	I added the group argument to refresh_screen()
	*/		
	if(strcmp(group, selected_group) == 0)
		{
		show_article_body(selected_group, selected_article);
		}
	}
if(visible_browser == POSTING_LIST_BROWSER) show_postings();

fl_unfreeze_form(fdui -> xagent);

return(1);
}/* end function refresh screen */


int ogettime(
char *weekday,
char *month,
int *monthday,
int *hour,
int *minute,
int *second,
int *year
)
{
unsigned long int I;
char *datestring;

time(&I); /* sets unsigned long int I to seconds since start 1970 */
datestring = ctime(&I);
					/* char * converts I to character string date and time */

sscanf(datestring,"%s %s %d %d:%d:%d %d",\
weekday, month, monthday, hour, minute, second, year);
												/* no &, already pointers */

return(1);
}/* end function ogettime */


